<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Controllers;

use Models\Configuration;
use Models\Logging;
use Models\Zone;

/** This class allow to check if a zone or a record is expired or available. It
    activate/disable the associated element */
class Cron
{
    /** The cronPath configuration */
    private $cronPath;

    /** Attributes to connect to the database */
    private $dbconfig = array("dsn" => null,
                             "username" => null,
                             "password" => null,
                             "driver_options" => null,
                             "tableprefix" => "");

    /** The start timestamp to check the cron process time */
    private $startTime;

    /** The constructor manage the temporary directory */
    public function __construct()
    {
        $this->startTime = time();
        $configuration = new Configuration();
        $this->cronPath = $configuration->get("cronPath");
        if (! file_exists($this->cronPath)) {
            mkdir($this->cronPath, 0777, true);
        }
        if (! is_dir($this->cronPath)) {
            throw new \Exception("Cronpath is not a directory", 500);
        }
        if (! is_readable($this->cronPath)) {
            throw new \Exception("Cronpath is not readable", 500);
        }
        if (! is_writeable($this->cronPath)) {
            throw new \Exception("Cronpath is not writeable", 500);
        }
        $this->dbconfig = $configuration->get("database");
        if ($this->dbconfig["dsn"] === null) {
            throw new \Exception(
                _("The database DSN parameter must be configured"),
                500
            );
        }
        if (!isset($this->dbconfig["tableprefix"])) {
            $this->dbconfig["tableprefix"] = "";
        }
    }

    /** Check the records and the zones.
        Actually, the records are not functionnal */
    public function all()
    {
        return $this->zones() && $this->records() && $this->cleanCache();
    }

    /** Check for the zones */
    public function zones()
    {
        // Get all the zones and store the new state. Check if there is a difference
        // between the old one.
        $old = array();
        $zonesJSONfile = $this->cronPath . "/zones.json";
        if (file_exists($zonesJSONfile)) {
            $old = json_decode(file_get_contents($zonesJSONfile), true);
        }
        $zones = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $new = $zones->read();
        $new = $zones->addDisable($new);


        // Can't use array_diff as it is array of array.
        $TBExported = false;
        $TBSaved = true;
        // Construct an array with the id as key
        $tmpnew = array();
        foreach ($new as $knew => $vnew) {
            $tmpnew[$vnew["id"]] = $vnew;
        }

        // Look at zones modification. The new zones are not exported by cron.
        foreach ($tmpnew as $id => $vnew) {
            if (! isset($old[$id])) {
                continue;
            }
            $msg = "";
            if ($old[$id]["disabled"] !== $vnew["disabled"]) {
                $msg .= "Zone change : " . $vnew["zone"] . "/" . $vnew["viewname"] . " : ";
                if ($vnew["disabled"] === true) {
                    $msg .= "disable\n";
                } else {
                    $msg .= "enable\n";
                }
                Logging::log(LOG_NOTICE, "Cron : $msg");
                $TBExported = true;
            }
        }
        if ($TBExported === true) {
            echo "==> Zone change state : export to server\n";
            $servers = new Servers(["email" => "cron"]);
            try {
                Logging::log(LOG_NOTICE, "Cron : Update servers");
                $servers->zonelistconfiguration();
                $servers->reconfig();
            } catch (\Exception $e) {
                echo $e->getMessage() . "\n";
                $TBSaved = false;
            }
        }

        if ($TBSaved === true) {
            return !! file_put_contents($zonesJSONfile, json_encode($tmpnew));
        }
        return false;
    }

    /** Check for the records */
    public function records()
    {
        // Get all the records and store the new state. Check if there is a
        // difference between the old one, then restart the associated zone.
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $zones = $z->read();
        $zones = $z->addDisable($zones);
        $r = new Records(["email" => "cron"]);
        foreach ($zones as $zone) {
            $TBExported = false;
            $TBSaved = true;
            // Get the records for this zone/viewname
            $new = $r->read($zone["zone"], $zone["viewname"]);
            $old = array();
            $recordsJSONfile = $this->cronPath . "/" . $zone["zone"] . "-" .
                                             $zone["viewname"] . ".json";
            if (file_exists($recordsJSONfile)) {
                $old = json_decode(file_get_contents($recordsJSONfile), true);
            }
            // Construct an array with the id as key
            $tmpnew = array();
            foreach ($new as $knew => $vnew) {
                $tmpnew[$vnew["id"]] = $vnew;
            }
            foreach ($tmpnew as $id => $vnew) {
                // Look at records modification. The new records are not exported by
                // cron.
                if (time() - $this->startTime > 58) {
                    die("Process CRON : time exceeded\n");
                }
                if (! isset($old[$id])) {
                    continue;
                }
                $msg = "";
                if ($old[$id]["disabled"] !== $vnew["disabled"]) {
                    $msg .= "Record change in " . $zone["zone"] . "/" . $zone["viewname"] .
                     " : " . $vnew["source"] . " " . $vnew["class"] . " " . $vnew["rr"] . " " .
                     $vnew["addvalues"] . " : ";
                    if ($vnew["disabled"] === true) {
                        $msg .= "disable\n";
                    } else {
                        $msg .= "enable\n";
                    }
                    Logging::log(LOG_NOTICE, "Cron : $msg");
                    $TBExported = true;
                }
            }

            if ($TBExported === true) {
                echo "==> Zone " . $zone["zone"] . "/" . $zone["viewname"] . " change record " .
                 "state : export to server\n";
                $servers = new Servers(["email" => "cron"]);
                try {
                    Logging::log(
                        LOG_NOTICE,
                        "Cron : Update zone " . $zone["zone"] . "/" .
                        $zone["viewname"] . " on servers"
                    );
                    $servers->zoneexport($zone["zone"], $zone["viewname"]);
                    // Not reload the server if the zone is disabled
                    if ($zone["disabled"] === false) {
                        Logging::log(
                            LOG_NOTICE,
                            "Cron : Reload zone " . $zone["zone"] . "/" .
                            $zone["viewname"] . " on servers"
                        );
                        $servers->reload($zone["zone"], $zone["viewname"]);
                    }
                } catch (\Exception $e) {
                    echo $e->getMessage() . "\n";
                    Logging::log(LOG_ERROR, "Cron ERROR : " . $e->getMessage());
                    $TBSaved = false;
                }
            }

            if ($TBSaved === true) {
                file_put_contents($recordsJSONfile, json_encode($tmpnew));
            }
        }
        return true;
    }

    /** Clean the cache dir of unused files */
    public function cleanCache()
    {
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $tmp = $z->read();
        $zones = array();
        foreach ($tmp as $t) {
            $zones[] = $this->cronPath . "/" . $t["zone"] . "-" . $t["viewname"] . ".json";
        }
        $zones[] = $this->cronPath . "/zones.json";
        foreach (glob($this->cronPath . "/*") as $file) {
            if (! in_array($file, $zones)) {
                echo "cron : cleanning of unused $file\n";
                unlink($file);
            }
        }
        return true;
    }
}
