<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Controllers;

use Controllers\Servers;
use Controllers\Userprefs;
use Controllers\Zones;
use Domframework\Authzgroups;
use Models\Bind;
use Models\Configuration;
use Models\Logging;
use Models\Record;
use Models\Zone;

/** Manage the records in a zone */
class Records
{
    /** Attributes to connect to the database */
    private $dbconfig = array("dsn" => null,
                             "username" => null,
                             "password" => null,
                             "driver_options" => null,
                             "tableprefix" => "");

    /** The anonymous user */
    private $auth = array("email" => "anonymous");

    /** Check if the configuration of database is done */
    public function __construct($auth)
    {
        // Read the configuration and populate local variables
        $configuration = new Configuration();
        $this->dbconfig = $configuration->get("database");
        if ($this->dbconfig["dsn"] === null) {
            throw new \Exception(
                _("The database DSN parameter must be configured"),
                500
            );
        }
        if (! isset($this->dbconfig["tableprefix"])) {
            $this->dbconfig["tableprefix"] = "";
        }
        if (
            ! is_array($auth) ||
            ! array_key_exists("email", $auth)
        ) {
            throw new \Exception(
                _("Invalid auth provided to 'records' class"),
                403
            );
        }
        $this->auth = $auth;
    }

    /** Insert multiple record in a range
        The %d will be change from
        @param array $data Data to be recorded in the new record :
           - string zone The zone name
           - string viewname The viewname
           - source with one or two %d,
             class, rr, addvalues, target with one or two %d,
             ttl, comment, opendate,
             closedate
           - start : integer
           - end : integer
        @return array|bool if errors, true if all is OK*/
    public function insertRange($data)
    {
        if (! is_array($data)) {
            throw new \Exception(_("Data provided are not an array"), 405);
        }
        if (!isset($data["zone"])) {
            throw new \Exception(_("Zone not found"), 404);
        }
        if (!isset($data["viewname"])) {
            throw new \Exception(_("Viewname not found"), 404);
        }
        if (!isset($data["rr"])) {
            throw new \Exception(_("RR not found"), 404);
        }
        if (!isset($data["start"])) {
            throw new \Exception(_("No start point provided"), 404);
        }
        if (!isset($data["end"])) {
            throw new \Exception(_("No end point provided"), 404);
        }
        if (! isset($data["source"])) {
            throw new \Exception(_("No source data provided"), 404);
        }
        if (! isset($data["target"])) {
            throw new \Exception(_("No target data provided"), 404);
        }
        $z = new zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $rc = $z->read(array(array("zone", $data["zone"]),
                           array("viewname", $data["viewname"])));
        unset($z);
        $zoneid = reset($rc);
        if (!isset($zoneid["id"])) {
            throw new \Exception(_("Zone not found"), 404);
        }
        $data["zoneid"] = $zoneid["id"];
        if ($this->accessright($data["zone"], $data["viewname"]) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $data["zone"],
                    $data["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright($data["zone"], $data["viewname"]) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $data["zone"],
                    $data["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $errors = array();
        if ($data["rr"] === "A") {
            if (strspn($data["start"], "0123456789") !== strlen($data["start"])) {
                $errors["start"] = array("error", _("Invalid start point provided"));
            }
            if (strspn($data["end"], "0123456789") !== strlen($data["end"])) {
                $errors["end"] = array("error", _("Invalid end point provided"));
            }
            $start = $data["start"];
            $end = $data["end"];
        } elseif ($data["rr"] === "AAAA") {
            if (
                strspn($data["start"], "0123456789abcdef") !==
                strlen($data["start"])
            ) {
                $errors["start"] = array("error", _("Invalid start point provided"));
            }
            if (strspn($data["end"], "0123456789abcdef") !== strlen($data["end"])) {
                $errors["end"] = array("error", _("Invalid end point provided"));
            }
            $start = hexdec($data["start"]);
            $end = hexdec($data["end"]);
        } elseif ($data["rr"] === "PTR") {
            if (substr($data["zone"], -14) === ".in-addr.arpa.") {
                // IPv4
                if (strspn($data["start"], "0123456789") !== strlen($data["start"])) {
                    $errors["start"] = array("error", _("Invalid start point provided"));
                }
                if (strspn($data["end"], "0123456789") !== strlen($data["end"])) {
                    $errors["end"] = array("error", _("Invalid end point provided"));
                }
                $start = $data["start"];
                $end = $data["end"];
            } elseif (substr($data["zone"], -10) === ".ip6.arpa.") {
                // IPv6
                if (
                    strspn($data["start"], "0123456789abcdef") !==
                    strlen($data["start"])
                ) {
                    $errors["start"] = array("error", _("Invalid start point provided"));
                }
                if (strspn($data["end"], "0123456789abcdef") !== strlen($data["end"])) {
                    $errors["end"] = array("error", _("Invalid end point provided"));
                }
                $start = hexdec($data["start"]);
                $end = hexdec($data["end"]);
            } else {
                throw new \Exception(_("PTR not in reverse zone"), 406);
            }
        } else {
            throw new \Exception(
                _("Can only add multiple records in A,AAAA or PTR"),
                406
            );
        }

        if ($data["start"] > $data["end"]) {
            throw new \Exception(
                _("Start must be lower or equal than End value"),
                406
            );
        }
        if (strpos($data["source"], "%d") === false) {
            $errors["source"] = array("error",
                                 _("No %d in source : can't allow the range"));
        }
        if (strpos($data["target"], "%d") === false) {
            $errors["target"] = array("error",
                                 _("No %d in target : can't allow the range"));
        }
        if (
            substr_count($data["source"], "%d") !==
            substr_count($data["target"], "%d")
        ) {
            $errors["target"] = array("error",
                          _("Not the same number of %d in source and target"));
        }
        if (substr_count($data["source"], "%d") === 0) {
            $errors["source"] = array("error",
                                 _("Must have one %d in source"));
        } elseif (substr_count($data["source"], "%d") !== 1) {
            $errors["source"] = array("error",
                                 _("Only one %d is allowed in source"));
        }
        if (substr_count($data["target"], "%d") === 0) {
            $errors["target"] = array("error",
                                 _("Must have one %d in target"));
        } elseif (substr_count($data["target"], "%d") !== 1) {
            $errors["target"] = array("error",
                                 _("Only one %d is allowed in target"));
        }
        if (count($errors)) {
            return $errors;
        }
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        if (! isset($data["ttl"]) || $data["ttl"] === "") {
            $data["ttl"] = null;
        }
        if (isset($data["opendate"]) && $data["opendate"] !== "") {
            $data["opendate"] .= ":00";
        }
        if (isset($data["closedate"]) && $data["closedate"] !== "") {
            $data["closedate"] .= ":00";
        }
        if (! isset($data["opendate"])) {
            $data["opendate"] = gmdate("Y-m-d H:i:s");
        }
        for ($i = $start; $i <= $end; $i++) {
            $dataTmp = $data;
            if ($data["rr"] === "A") {
                $source = sprintf($data["source"], $i);
                $target = sprintf($data["target"], $i);
            } elseif ($data["rr"] === "AAAA") {
                $source = sprintf(str_replace("%d", "%x", $data["source"]), $i);
                $target = sprintf(str_replace("%d", "%x", $data["target"]), $i);
            } elseif ($data["rr"] === "PTR") {
                if (substr($data["zone"], -14) === ".in-addr.arpa.") {
                    // IPv4
                    $source = sprintf($data["source"], $i);
                    $target = sprintf($data["target"], $i);
                } elseif (substr($data["zone"], -10) === ".ip6.arpa.") {
                    // IPv6 : The source IP must be split and reversed
                    $isource = dechex($i);
                    $isource = strrev(implode(".", str_split($isource)));
                    $source = sprintf(
                        str_replace("%d", "%s", $data["source"]),
                        $isource
                    );
                    $target = sprintf(str_replace("%d", "%x", $data["target"]), $i);
                }
            }
            $dataTmp["source"] = $source;
            $dataTmp["target"] = $target;
            $errors = $r->verify($dataTmp);
            if (count($errors)) {
                return $errors;
            }
            $rc = $r->insert($dataTmp);
            $tuples = $r->read(array(array("id", $rc)));
            Logging::log(
                LOG_WARNING,
                "'" . $this->auth["email"] . "' create record in zone '" .
                $data["zone"] . "/" . $data["viewname"] . "' => ",
                $tuples
            );
        }
        unset($r);
        $this->automaticDNSupdate($data["zone"], $data["viewname"]);
        return true;
    }

    /** Insert a new record
        @param array $data Data to be recorded in the new record :
           - string zone The zone name
           - string viewname The viewname
           - source, class, rr, addvalues, target, ttl, comment, opendate,
             closedate
        @return string The last ID inserted */
    public function insert($data)
    {
        if (! is_array($data)) {
            throw new \Exception(_("Data provided are not an array"), 405);
        }
        if (!isset($data["zone"])) {
            throw new \Exception(_("Zone not found"), 404);
        }
        if (!isset($data["viewname"])) {
            throw new \Exception(_("Viewname not found"), 404);
        }
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $rc = $z->read(array(array("zone", $data["zone"]),
                           array("viewname", $data["viewname"])));
        unset($z);
        $zoneid = reset($rc);
        if (!isset($zoneid["id"])) {
            throw new \Exception(_("Zone not found"), 404);
        }
        $data["zoneid"] = $zoneid["id"];
        if ($this->accessright($data["zone"], $data["viewname"]) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $data["zone"],
                    $data["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright($data["zone"], $data["viewname"]) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $data["zone"],
                    $data["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        if (! isset($data["ttl"]) || $data["ttl"] === "") {
            $data["ttl"] = null;
        }
        if (isset($data["opendate"]) && $data["opendate"] !== "") {
            $data["opendate"] .= ":00";
        }
        if (isset($data["closedate"]) && $data["closedate"] !== "") {
            $data["closedate"] .= ":00";
        }
        if (! isset($data["opendate"])) {
            $data["opendate"] = gmdate("Y-m-d H:i:s");
        }
        $rc = $r->insert($data);
        $tuples = $r->read(array(array("id", $rc)));
        unset($r);
        $this->automaticDNSupdate($data["zone"], $data["viewname"]);
        Logging::log(
            LOG_WARNING,
            "'" . $this->auth["email"] . "' create record in zone '" .
            $data["zone"] . "/" . $data["viewname"] . "' => ",
            $tuples
        );
        return $rc;
    }

    /** Read all the records in the zone/view
        @param string $zone The zone name
        @param string $viewname The view name
        @param string|null $recordname The record ID to see
        @param array|null $display The fields to display
        @param array|null $order The order and direction to sort the entries
        @return array An array containing the record(s) */
    public function read(
        $zone,
        $viewname,
        $recordname = null,
        $display = null,
        $order = null
    ) {
        if ($this->accessright($zone, $viewname) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if (! is_string($zone)) {
            throw new \Exception(_("Zone provided is not a string"), 405);
        }
        if ($recordname !== null && ! is_string($recordname)) {
            throw new \Exception(_("Record name provided is not a string"), 405);
        }
        if ($display !== null && ! is_array($display)) {
            throw new \Exception(_("Displayed columns asked are not an array"), 405);
        }
        if ($order !== null && ! is_array($order)) {
            throw new \Exception(_("Order field asked is not an array"), 405);
        }
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $rc = $z->read(array(array("zone", $zone),
                           array("viewname", $viewname)));
        $zoneid = reset($rc);
        if (!isset($zoneid["id"])) {
            throw new \Exception(_("Zone not found"), 404);
        }
        $zoneid = $zoneid["id"];
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        $select[] = array("zoneid", $zoneid);
        if ($recordname !== null) {
            $select[] = array("id", $recordname);
        }
        $rc = $r->read($select, $display, $order);
        $rc = $r->addDisable($rc);
        unset($r);
        return ($rc);
    }

    /** Update a record identified by his key
        @param string $key The record ID to be updated
        @param array $data : The data to be updated
           - zoneid, source, class, rr, addvalues, target, ttl, comment, opendate,
             closedate */
    public function update($key, $data)
    {
        if (is_int($key)) {
            $key = "$key";
        }
        if (! is_string($key)) {
            throw new \Exception(_("Record key provided is not a string"), 405);
        }
        if (! is_array($data)) {
            throw new \Exception(_("Data provided are not an array"), 405);
        }
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        // Look for zone and viewname
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $zoneid = $r->read(array(array("id", $key)));
        $zoneid = reset($zoneid);
        if (! isset($zoneid)) {
            throw new \Exception(_("Record not found"), 404);
        }
        $zoneid = $zoneid["zoneid"];
        $dataZone = $z->read(array(array("id", $zoneid)));
        $dataZone = reset($dataZone);
        if ($this->accessright($dataZone["zone"], $dataZone["viewname"]) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $dataZone["zone"],
                    $dataZone["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright($dataZone["zone"], $dataZone["viewname"]) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $dataZone["zone"],
                    $dataZone["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($data["ttl"] === "") {
            $data["ttl"] = null;
        }
        if (isset($data["opendate"]) && $data["opendate"] !== "") {
            $data["opendate"] .= ":00";
        }
        if (isset($data["closedate"]) && $data["closedate"] !== "") {
            $data["closedate"] .= ":00";
        }
        $tuples = $r->read(array(array("id", $key)));
        $rc = $r->update($key, $data);
        if ($rc === 0) {
            throw new \Exception(_("No data modified"));
        } elseif ($rc !== 1) {
            throw new \Exception(_("More than one line modified"));
        }
        $this->automaticDNSupdate($dataZone["zone"], $dataZone["viewname"]);
        Logging::log(
            LOG_WARNING,
            "'" . $this->auth["email"] . "' update record in zone '" .
                  $data["zone"] . "/" . $data["viewname"] . "' => ",
            reset($tuples),
            " ==> ",
            $data
        );
        return true;
    }

    /** Delete a record identified by his key
      * @param string $key The record to delete
      * @param boolean|null $disableServerUpdate If true, disable the update of
      * the server after the deletion. The user must do it if it wants.
      * By default, use the user settings
      */
    public function delete($key, $disableServerUpdate = false)
    {
        if (is_int($key)) {
            $key = "$key";
        }
        if (! is_string($key)) {
            throw new \Exception(_("Record key provided is not a string"), 405);
        }
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        // Look for zone and viewname
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $zoneid = $r->read(array(array("id", $key)));
        $zoneid = reset($zoneid);
        if (! isset($zoneid)) {
            throw new \Exception(_("Record not found"), 404);
        }
        $zoneid = $zoneid["zoneid"];
        $dataZone = $z->read(array(array("id", $zoneid)));
        $dataZone = reset($dataZone);
        if ($this->accessright($dataZone["zone"], $dataZone["viewname"]) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $dataZone["zone"],
                    $dataZone["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright($dataZone["zone"], $dataZone["viewname"]) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $dataZone["zone"],
                    $dataZone["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $tuples = $r->read(array(array("id", $key)));
        $rc = $r->delete($key);
        unset($r);
        if ($rc === 0) {
            throw new \Exception(_("Can't remove the record : not found"), 404);
        }

        if ($disableServerUpdate === false) {
            $this->automaticDNSupdate($dataZone["zone"], $dataZone["viewname"]);
        }
        Logging::log(
            LOG_WARNING,
            "'" . $this->auth["email"] . "' delete record in zone '" .
                  $dataZone["zone"] . "/" . $dataZone["viewname"] . "' => ",
            reset($tuples)
        );
        return true;
    }

    /** Import the data into the zone
        The data are added to the zone. The zone should be cleared before using
        import for a complete and fresh import
        @param string $zone The zone name
        @param string $viewname The view name
        @param string $dataTXT The data to be imported in Bind format */
    public function importBind($zone, $viewname, $dataTXT)
    {
        if (! is_string($zone)) {
            throw new \Exception(_("Zone provided is not a string"), 405);
        }
        if (! is_string($viewname)) {
            throw new \Exception(_("View name provided is not a string"), 405);
        }
        if (!is_string($dataTXT) || $dataTXT === "") {
            throw new \Exception(_("No data provided"), 405);
        }
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $rc = $z->read(array(array("zone", $zone),
                           array("viewname", $viewname)));
        $zoneid = reset($rc);
        if (!isset($zoneid["id"])) {
            throw new \Exception(_("Zone not found"), 404);
        }
        if ($this->accessright($zone, $viewname) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright($zone, $viewname) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $b = new Bind();
        $data = $b->import($dataTXT, $zone);
        // array("source"=>$source, "class"=>$class, "rr"=>$rr,
        //                    "addvalues"=>$addvalues, "target"=>$target,
        //                    "ttl"=>$ttl,
        //                    "comment"=>$comment, "closedate"=>$closedate);
        $opendate = gmdate("Y-m-d H:i:s");
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        $r->beginTransaction();
        foreach ($data as $d) {
            $tmp = $d;
            $d["zoneid"] = $zoneid["id"];
            $d["zone"] = $zone;
            $d["view"] = $viewname;
            $d["opendate"] = $opendate;
            Logging::log(LOG_WARNING, "Import Bind : " . var_export($d, true));
            try {
                $rc = $r->insert($d);
            } catch (\Exception $e) {
                throw new \Exception(
                    $e->getMessage() . " - " . implode(" ", $tmp),
                    $e->getCode()
                );
            }
        }
        // If there is an error, the commit will not be applied and no change will
        // be done
        $r->commit();
        unset($r);

        $this->automaticDNSupdate($zone, $viewname);
        Logging::log(
            LOG_WARNING,
            "'" . $this->auth["email"] . "' import bind records in zone '" .
            $zone . "/" . $viewname . "' => ",
            $dataTXT
        );
        return true;
    }

    /** Update the SOA timestamp for the zone/view provided
        If there is no SOA record, don't generate any error
        @param string $zone The zone name
        @param string $viewname The view name
        @return true or an exception
      */
    public function updateSOA($zone, $viewname)
    {
        if (! is_string($zone)) {
            throw new \Exception(_("Zone provided is not a string"), 405);
        }
        if (! is_string($viewname)) {
            throw new \Exception(_("View name provided is not a string"), 405);
        }
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $rc = $z->read(array(array("zone", $zone),
                           array("viewname", $viewname)));
        $zoneid = reset($rc);
        if (!isset($zoneid["id"])) {
            throw new \Exception(_("Zone not found"), 404);
        }
        if ($this->accessright($zone, $viewname) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright($zone, $viewname) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $zoneid = $zoneid["id"];
        $r = new record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        $soarow = $r->read(array(array("zoneid", $zoneid),
                               array("rr", "SOA")));
        if (count($soarow) === 0) {
            return true;
        }
        //  throw new \Exception (_("No SOA defined in the zone"), 404);
        if (count($soarow) > 1) {
            throw new \Exception(_("More than one SOA defined in the zone"), 500);
        }
        $id = $soarow[0]["id"];
        $oldtarget = $soarow[0]["target"];
        preg_match('/\(\s*(\d+)\s+/', $oldtarget, $matches);
        if (! isset($matches[1])) {
            throw new \Exception(
                _("Can't capture the actual SOA : incorrect format"),
                500
            );
        }
        $serial = $matches[1];
        $timestamp = substr($serial, 0, -2);
        $version = substr($serial, -2, 2);
        $today = gmdate("Ymd");
        if (intval($timestamp) === intval($today)) {
            // There already have some modification today : update version number
            $newserial = $today . sprintf("%02d", intval($version) + 1);
        } else {
            // First modification today : put the actual date with version number 01
            $newserial = $today . "01";
        }

        $newtarget = preg_replace('/\(\s*(\d+)\s+/', "( $newserial ", $oldtarget);
        $rc = $r->update($id, array("source" => "@", "rr" => "SOA", "zoneid" => $zoneid,
                                  "target" => $newtarget, "id" => $id));
        if ($rc !== 1) {
            throw new \Exception(_("SOA update modify more than one line"), 500);
        }
        return true;
    }

    /** Return the translation of the titles (the columns in the database) */
    public function titles()
    {
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        return $r->titles();
    }

    public function verifyOne($field, $val)
    {
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        if ($field === "opendate" && $val !== "") {
            $val .= ":00";
        }
        if ($field === "closedate" && $val !== "") {
            $val .= ":00";
        }
        return $r->verifyOne($field, $val);
    }

    public function verifyAll($data)
    {
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        if (isset($data["opendate"]) && $data["opendate"] !== "") {
            $data["opendate"] .= ":00";
        }
        if (isset($data["closedate"]) && $data["closedate"] !== "") {
            $data["closedate"] .= ":00";
        }
        return $r->verifyAll($data);
    }

    public function verify($data, $updatekey = false)
    {
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        if (isset($data["opendate"]) && $data["opendate"] !== "") {
            $data["opendate"] .= ":00";
        }
        if (isset($data["closedate"]) && $data["closedate"] !== "") {
            $data["closedate"] .= ":00";
        }
        return $r->verify($data, $updatekey);
    }

    public function verifyMultiple($data)
    {
        $res = [];
        if (! isset($data["start"]) || strlen(trim($data["start"])) < 1) {
            $res["start"] = array("error", _("Not enough chars"));
        }
        if (! isset($data["end"]) || strlen(trim($data["end"])) < 1) {
            $res["end"] = array("error", _("Not enough chars"));
        }
        if (strpos($data["source"], "%d") === false) {
            $res["source"] = array("error", _("Missing %d in source"));
        }
        if (strpos($data["target"], "%d") === false) {
            $res["target"] = array("error", _("Missing %d in target"));
        }
        if (! empty($res)) {
            return $res;
        }
        //$res = $this->verify ($data);
        return $res;
    }

    private function automaticDNSupdate($zone, $viewname)
    {
        // Look at autoactivation of zones after updates

        $userpref = new Userprefs($this->auth);
        $auto = $userpref->read("autoactivate-$zone/$viewname");
        $default = $userpref->read("autoactivate-DEFAULT");
        $zones = new Zones($this->auth);
        $disabled = $zones->disabled($zone, $viewname);
        if (count($auto)  && $auto["value"] === "0") {
            return true;
        }
        if (
            (count($auto)  && $auto["value"] === "1") ||
            count($default) === 0 ||
            (count($default) && $default["value"] === "1")
        ) {
            // Automatic export and reload of DNS servers
            Logging::log(LOG_DEBUG, "Automatic Update for zone $zone/$viewname");
            try {
                $servers = new Servers($this->auth);
                $servers->zoneexport($zone, $viewname);
                // Do not reload the servers if the zone is disabled
                if ($disabled === false) {
                    try {
                        $servers->reload($zone, $viewname);
                    } catch (\Exception $e) {
                        throw new \Exception(_("Can't reload the DNS servers: ") .
                            $e->getMessage());
                    }
                }
            } catch (\Exception $e) {
                throw new \Exception(_("Can't export the zone to DNS servers: ") .
                             $e->getMessage());
            }
        }
    }

    /** Return TRUE if the zone/viewname is in READONLY for the user or FALSE if
        the user have the RW rights */
    public function readonly($zone, $viewname)
    {
        if ($this->auth["email"] === "cron") {
            return true;
        }
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $rc = $authz->allow("domdns", $this->auth["email"], "/$zone/$viewname");
        Logging::log(
            LOG_DEBUG,
            "RecordAuth : readonly('" . $this->auth["email"] . "'," .
            "'/$zone/$viewname')=$rc"
        );
        if ($rc === "RW") {
            return false;
        }
        return true;
    }

    /** Return TRUE if the user right allow to edit the zone
        Return FALSE else */
    public function editright($zone, $viewname)
    {
        // Cron can update the SOA. So it need RW access
        if ($this->auth["email"] === "cron") {
            return true;
        }
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $rc = $authz->allow("domdns", $this->auth["email"], "/$zone/$viewname");
        Logging::log(
            LOG_DEBUG,
            "RecordAuth : editright('" . $this->auth["email"] . "'," .
            "'/$zone/$viewname')=$rc"
        );
        if ($rc === "RW") {
            return true;
        }
        return false;
    }

    /** Return TRUE if the user right allow to see the zones
        Return FALSE else */
    private function accessright($zone, $viewname)
    {
        if ($this->auth["email"] === "cron") {
            return true;
        }
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $rc = $authz->allow("domdns", $this->auth["email"], "/$zone/$viewname");
        Logging::log(
            LOG_DEBUG,
            "RecordAuth : accessright('" . $this->auth["email"] . "'," .
            "'/$zone/$viewname')=$rc"
        );
        if ($rc !== "NO") {
            return true;
        }
        return false;
    }


    /** Return the IPv6 address decompressed (all the fields exists)
        If the provided address is IPv4, nothing is changed
        Throw an exception if the provided parameter is incorrect
        Based on http://www.weberdev.com/get_example.php3?ExampleID=3921 */
    public function adressesDecompressIPv6($ip)
    {
        if (! is_string($ip) || $ip === "") {
            throw new \Exception(
                "Provided IP address is not a string or is empty",
                500
            );
        }
        if (strstr($ip, "::")) {
            $e = explode(":", $ip);
            // Case starting by ::
            if ($e[0] === "") {
                $e[0] = "0";
            }
            // Case ending by ::
            if ($e[count($e) - 1] === "") {
                $e[count($e) - 1] = "0";
            }
            $s = 8 - count($e);
            foreach ($e as $key => $val) {
                if ($val === "") {
                    for ($i = 0; $i <= $s; $i++) {
                        $newipv6[] = 0;
                    }
                } else {
                    $newipv6[] = $val;
                }
            }
            $ip = implode(":", $newipv6);
        }
        return $ip;
    }

    /** Return the IP address with the fields completed by the missing zeros
        Change something in IPv6 but don't change anything if an IPv4 address is
        provided */
    public function adressesCompletedWithZeros($ip)
    {
        if (! is_string($ip) || $ip === "") {
            throw new \Exception(
                "Provided IP address is not a string or is empty",
                500
            );
        }
        $ip = $this->adressesDecompressIPv6($ip);
        if (substr_count($ip, ":") === 7) {
            // IPv6
            $ips = explode(":", $ip);
            $ipnew = array();
            foreach ($ips as $iptmp) {
                $ipnew[] = sprintf("%04x", hexdec($iptmp));
            }
            return implode(":", $ipnew);
        } elseif (substr_count($ip, ".") === 31) {
            // Full IPv6 with dots
            return $ip;
        } elseif (substr_count($ip, ".") === 3) {
            // IPv4
            return $ip;
        }
        throw new \Exception("Provided IP address is not IPv4 and not IPv6 " .
                          "'$ip'", 500);
    }

    /** Reverse the provided IP address
        The IPv6 are returned in format :
        x.x.x.x.x.x.x.x.x.x.x.d.x.x.x.x.x.x.x.x.x.x.x.x.x.x.x.x.x.x.x.x
        The IPv4 are returned in format :
        DDD.CCC.BBB.AAA */
    public function reverseIPAddress($ipReverse)
    {
        if (!is_string($ipReverse) || $ipReverse === "") {
            throw new \Exception(
                "Provided IP address is not a string or is empty",
                500
            );
        }
        $ipReverse = $this->adressesCompletedWithZeros($ipReverse);
        if (substr_count($ipReverse, ":") === 7) {
            // Complete IPv6 with quadruplets and colons
            $ip = str_replace(":", "", $ipReverse);
            $tmp2 = array_reverse(str_split($ip));
            return implode(".", $tmp2);
        } elseif (substr_count($ipReverse, ".") === 31) {
            // IPv6 withs dots
            $tmp2 = explode(".", $ipReverse);
            $tmp2 = array_reverse($tmp2);
            $ipnew = implode(".", $tmp2);
            return $ipnew;
        } elseif (substr_count($ipReverse, ".") === 3) {
            // IPv4
            $tmp2 = explode(".", $ipReverse);
            $tmp2 = array_reverse($tmp2);
            $ipnew = implode(".", $tmp2);
            return $ipnew;
        }
        throw new \Exception("Provided IP address is not IPv4 and not IPv6 " .
                          "'$ipReverse'", 500);
    }

    /** Compress the IPv6 address:
        2.0.0.1.0.6.6.0.5.3.0.d.0.0.0.0.0.0.0.0.1.1.1.1.1.1.1.1.1.1.1.1 ->
        2001:0660:530d:0000:0000:1111:1111:1111*/
    public function compressIPAddress($ip)
    {
        if (substr_count($ip, ".") === 31) {
            $ip = str_replace(".", "", $ip);
            $quad = str_split($ip, 4);
            foreach ($quad as $key => $val) {
                $quad[$key] = dechex(hexdec($val));
            }
            $ip = implode(":", $quad);
            $ip = inet_ntop(inet_pton($ip));
            return $ip;
        } else {
            return $ip;
        }
    }
}
