<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Controllers;

use Controllers\Records;
use Controllers\Zones;
use Domframework\Authzgroups;
use Domframework\Ratelimitfile;
use Models\Bind;
use Models\Configuration;
use Models\Logging;
use Models\Ssh;

/** Drive the DNS servers :
    Reload them after configuration modification
    Upload them a DNS zone */
class Servers
{
    /** Servers must be configured like :
       "192.168.56.110" => array (
            "type"=>"bind",                // DNS server type (bind only actually)
            "methodCLI"=>"ssh",            // can be ssh or sudo
            "methodFILE"=>"scp",           // can be scp or sudo
            "username"=>"root",            // used to connect by SSH
            "password"=>"l1d3sq6c",        // used to connect by SSH
            "sshPort"=>22,                 // used to connect by SSH
            "fingerprint"=>FALSE,          // used to connect by SSH
            "publickeyfile"=>FALSE,        // used to connect by SSH
            "privatekeyfile"=>FALSE,       // used to connect by SSH
            "passphrase"=>FALSE,           // used to connect by SSH
            "zonepath"=>"/etc/bind/zones", // Directory where zones are stored
        ), */
    private $servers = array();
    /** Database configuration */
    private $dbconfig = array("dsn" => null,
                             "username" => null,
                             "password" => null,
                             "driver_options" => null);
    /** Display a trace on screen to do waiting the user */
    public $trace = false;

    /** The anonymous user */
    private $auth = array("email" => "anonymous");

    /** DomDNS has the view support enable or disable */
    private $viewSupport = false;

    /** Check if the configuration is available */
    public function __construct($auth)
    {
        // Read the configuration and populate local variables
        $configuration = new Configuration();
        $this->servers = $configuration->get("servers");
        $this->dbconfig = $configuration->get("database");
        if (isset($this->servers["not configured"])) {
            throw new \Exception(_("No server configured"), 500);
        }
        if ($this->dbconfig["dsn"] === null) {
            throw new \Exception(
                _("The database DSN parameter must be configured"),
                500
            );
        }
        $this->viewSupport = $configuration->get("viewSupport");
        if (
            ! is_array($auth) ||
            ! array_key_exists("email", $auth)
        ) {
            throw new \Exception(
                _("Invalid auth provided to 'records' class"),
                403
            );
        }
        $this->auth = $auth;
    }

    /** Display a trace when doing the actions (The user can see the progression)
      */
    private function trace($message)
    {
        if ($this->trace !== true) {
            return;
        }
        if (php_sapi_name() === "cli") {
            echo $message . "\n";
        } else {
            echo nl2br("$message\n");
        }
        @ob_flush();
        @flush();
    }

    /** Reload the DNS servers after an update of zones.
        If the zone and viewname are provided, reload only this zone */
    public function reload($zone = null, $viewname = null)
    {
        if ($this->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s'",
                    $this->auth["email"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($zone === null) {
            // limit the reload without zone to editrightZone right
            if ($this->editrightZone() !== true) {
                Logging::log(
                    LOG_WARNING,
                    sprintf(
                        "Can't reload all the zones : no RW right for " .
                        "'%s'",
                        $this->auth["email"]
                    )
                );
                throw new \Exception(sprintf(
                    _("Access forbidden for '%s'"),
                    $this->auth["email"]
                ), 403);
            }
        }
        $ratelimiter = new Ratelimitfile();
        // TODO : Rate-limiter values defined in the configuration (10 reload/60s by
        // default max)
        if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
            $ipClient = $_SERVER["HTTP_X_FORWARDED_FOR"];
        } elseif (isset($_SERVER["REMOTE_ADDR"])) {
            $ipClient = $_SERVER["REMOTE_ADDR"];
        } else {
            $ipClient = false;
        }
        if ($ipClient !== false && $ratelimiter->set("loggin-$ipClient") === false) {
            Logging::log(LOG_WARNING, "Ratelimiting for $ipClient");
            $_SESSION["domdns"]["message"] = _("Too much connections");
            throw new \Exception(_("Too much connections"), 401);
        }

        if ($zone !== null) {
            $zones = new Zones($this->auth);
            $zonesData = $zones->read($zone, $viewname);
            if (
                ! isset($zonesData[0]["viewclients"]) ||
                $zonesData[0]["viewclients"] === ""
            ) {
                $zonesData[0]["viewclients"] = "any";
            }
            $zonesClients = explode(",", $zonesData[0]["viewclients"]);
            foreach ($zonesClients as $key => $val) {
                $zonesClients[$key] = trim($val);
            }
        } else {
            $zonesClients = array("unused");
        }

        foreach ($this->servers as $server => $params) {
            if (!isset($params["type"])) {
                throw new \Exception(sprintf(
                    _("No defined server type for %s server"),
                    $server
                ), 500);
            }
            switch ($params["methodCLI"]) {
                case "ssh":
                    $this->trace("Connect by SSH on server...");
                    $ssh = new Ssh();
                    if (isset($params["username"])) {
                        $ssh->username = $params["username"];
                    }
                    if (isset($params["password"])) {
                        $ssh->password = $params["password"];
                    }
                    if (isset($params["fingerprint"])) {
                        $ssh->fingerprint = $params["fingerprint"];
                    }
                    if (isset($params["publickeyfile"])) {
                        $ssh->publickeyfile = $params["publickeyfile"];
                    }
                    if (isset($params["privatekeyfile"])) {
                        $ssh->privatekeyfile = $params["privatekeyfile"];
                    }
                    if (isset($params["passphrase"])) {
                        $ssh->passphrase = $params["passphrase"];
                    }
                    $ssh->connect($server, $params["sshPort"]);
                    $this->trace("Reloading DNS server...");
                    if ($params["type"] === "bind") {
                        $cmd = "";
                        $id = 0;
                        foreach (["freeze", "reload", "thaw"] as $action) {
                            if ($id > 0) {
                                $cmd .= " ; ";
                            }
                            $id++;
                            foreach ($zonesClients as $client) {
                                if (isset($params["rndcPath"])) {
                                    $cmd .= $params["rndcPath"];
                                } else {
                                    $cmd .= "/usr/sbin/rndc";
                                }
                                $cmd .= " $action";
                                if ($zone !== null) {
                                    $cmd .= " " . escapeshellarg($zone);
                                    if (
                                        $viewname !== null && (
                                            $params["serverViewSupport"] === true ||
                                        $params["serverViewSupport"] === null &&
                                        $this->viewSupport === true
                                        )
                                    ) {
                                        $cmd .= " in " . escapeshellarg($client);
                                    }
                                }
                            }
                        }
                        $res = $ssh->exec($cmd);
                    } else {
                        throw new \Exception(sprintf(
                            _("Unknown server type '%s' for %s server"),
                            $params["type"],
                            $server
                        ), 500);
                    }
                    if ($res["stderr"] !== "") {
                        throw new \Exception(
                            sprintf(
                                _("Error when reloading server %s : %s"),
                                $server,
                                str_replace(
                                    "\n",
                                    ";",
                                    $res["stderr"] . ":" .
                                    $res["stdout"]
                                )
                            ),
                            500
                        );
                    }
                    $this->trace("DONE");
                    $ssh->disconnect();
                    break;
                case "sudo":
                    $this->trace("Reloading DNS server...");
                    if ($params["type"] === "bind") {
                        $cmd = "";
                        $id = 0;
                        foreach (["freeze", "reload", "thaw"] as $action) {
                            if ($id > 0) {
                                $cmd .= " ; ";
                            }
                            $id++;
                            foreach ($zonesClients as $client) {
                                if (isset($params["rndcPath"])) {
                                    $cmd = $params["rndcPath"] . " reload";
                                } else {
                                    $cmd .= "sudo -n -u " . $params["username"];
                                    $cmd .= " /usr/sbin/rndc reload";
                                }
                                if ($zone !== null) {
                                    $cmd .= " " . escapeshellarg($zone);
                                    if (
                                        $viewname !== null && (
                                            $params["serverViewSupport"] === true ||
                                        $params["serverViewSupport"] === null &&
                                        $this->viewSupport === true
                                        )
                                    ) {
                                        $cmd .= " in " . escapeshellarg($client);
                                    }
                                }
                                $cmd .= " 2>&1";
                            }
                        }
                        exec($cmd, $output, $rc);
                    } else {
                        throw new \Exception(sprintf(
                            _("Unknown server type '%s' for %s server"),
                            $params["type"],
                            $server
                        ), 500);
                    }
                    if ($rc !== 0) {
                        throw new \Exception(sprintf(
                            _("Error when reloading server %s : %s"),
                            $server,
                            implode(";", $output)
                        ), 500);
                    }
                    $this->trace("DONE\n");
                    break;
                default:
                    throw new \Exception(sprintf(
                        _("Unknown configured methodCLI for server %s"),
                        $server
                    ), 500);
            }
        }

        return true;
    }

    /** Reconfig the DNS servers after an update of zones list in named.conf */
    public function reconfig()
    {
        if ($this->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s'",
                    $this->auth["email"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $ratelimiter = new ratelimitfile();
        // TODO : Rate-limiter values defined in the configuration (10 reload/60s by
        // default max)
        if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
            $ipClient = $_SERVER["HTTP_X_FORWARDED_FOR"];
        } elseif (isset($_SERVER["REMOTE_ADDR"])) {
            $ipClient = $_SERVER["REMOTE_ADDR"];
        } else {
            $ipClient = false;
        }
        if ($ipClient !== false && $ratelimiter->set("loggin-$ipClient") === false) {
            Logging::log(LOG_WARNING, "Ratelimiting for $ipClient");
            $_SESSION["domdns"]["message"] = _("Too much connections");
            throw new \Exception(_("Too much connections"), 401);
        }

        foreach ($this->servers as $server => $params) {
            if (!isset($params["type"])) {
                throw new \Exception(sprintf(
                    _("No defined server type for %s server"),
                    $server
                ), 500);
            }
            switch ($params["methodCLI"]) {
                case "ssh":
                    $this->trace("Connect by SSH on server...");
                    $ssh = new ssh();
                    if (isset($params["username"])) {
                        $ssh->username = $params["username"];
                    }
                    if (isset($params["password"])) {
                        $ssh->password = $params["password"];
                    }
                    if (isset($params["fingerprint"])) {
                        $ssh->fingerprint = $params["fingerprint"];
                    }
                    if (isset($params["publickeyfile"])) {
                        $ssh->publickeyfile = $params["publickeyfile"];
                    }
                    if (isset($params["privatekeyfile"])) {
                        $ssh->privatekeyfile = $params["privatekeyfile"];
                    }
                    if (isset($params["passphrase"])) {
                        $ssh->passphrase = $params["passphrase"];
                    }
                    $ssh->connect($server, $params["sshPort"]);
                    $cmd = "/usr/sbin/rndc reconfig";
                    if (isset($params["rndcPath"])) {
                        $cmd = $params["rndcPath"] . " reconfig";
                    }
                    $this->trace("Reloading DNS server...");
                    if ($params["type"] === "bind") {
                        $res = $ssh->exec($cmd);
                    } else {
                        throw new \Exception(sprintf(
                            _("Unknown server type '%s' for %s server"),
                            $params["type"],
                            $server
                        ), 500);
                    }
                    if ($res["stderr"] !== "") {
                        throw new \Exception(
                            sprintf(
                                _("Error when reconfig server %s : %s"),
                                $server,
                                str_replace(
                                    "\n",
                                    ";",
                                    $res["stderr"] . ":" .
                                    $res["stdout"]
                                )
                            ),
                            500
                        );
                    }
                    $this->trace("DONE");
                    $ssh->disconnect();
                    break;
                case "sudo":
                    $this->trace("Reloading DNS server...");
                    $cmd = "sudo -n -u " . $params["username"] .
                    $cmd = " /usr/sbin/rndc reconfig 2>&1";
                    if (isset($params["rndcPath"])) {
                        $cmd = $params["rndcPath"] . " reconfig 2>&1";
                    }
                    if ($params["type"] === "bind") {
                        exec($cmd, $output, $rc);
                    } else {
                        throw new \Exception(sprintf(
                            _("Unknown server type '%s' for %s server"),
                            $params["type"],
                            $server
                        ), 500);
                    }
                    if ($rc !== 0) {
                        throw new \Exception(sprintf(
                            _("Error when reconfig server %s : %s"),
                            $server,
                            implode(";", $output)
                        ), 500);
                    }
                    $this->trace("DONE\n");
                    break;
                default:
                    throw new \Exception(sprintf(
                        _("Unknown configured methodCLI for server %s"),
                        $server
                    ), 500);
            }
        }

        return true;
    }

    /** Return the text which will be stored in bind server */
    public function zoneexportTextBind($zone, $viewname)
    {
        if ($this->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright($zone, $viewname) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if (! is_string($zone)) {
            throw new \Exception(_("Zone provided is not a string"), 405);
        }
        if (! is_string($viewname)) {
            throw new \Exception(_("View name provided is not a string"), 405);
        }
        $zoneDB = new Zones($this->auth);
        $zones = $zoneDB->read($zone, $viewname);
        if (!isset($zones[0]["id"])) {
            throw new \Exception(sprintf(_("Zone '%s' not found"), $zone), 404);
        }
        $recordDB = new Records($this->auth);
        $records = $recordDB->read($zone, $viewname);
        $bind = new Bind();
        return $bind->createZoneFile($records);
    }

    /** Export zone to servers and override existing file
        @param string $zone The zone name
        @param string $viewname The view name */
    public function zoneexport($zone, $viewname)
    {
        if ($this->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright($zone, $viewname) !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        // 0. Update the SOA timestamp
        // 1. Extract zone in memory
        // For each configured server :
        //   2. Construct server format (bind)
        //   3. Upload zone to server (SSH, local file)
        if (! is_string($zone)) {
            throw new \Exception(_("Zone provided is not a string"), 405);
        }
        if (! is_string($viewname)) {
            throw new \Exception(_("View name provided is not a string"), 405);
        }
        $zoneDB = new zones($this->auth);
        $zones = $zoneDB->read($zone, $viewname);
        if (!isset($zones[0]["id"])) {
            throw new \Exception(sprintf(_("Zone '%s' not found"), $zone), 404);
        }
        // We need one zone file by viewclient not by viewname, or DNSSEC won't work
        $zoneArr = $zones[0];
        $viewclients = array();
        foreach ($zones as $zoneArr) {
            if (
                $zoneArr["viewclients"] === null ||
                trim($zoneArr["viewclients"]) === ""
            ) {
                $viewclients["any"][] = $zoneArr["zone"] . "-" . $zoneArr["viewname"];
                continue;
            }
            foreach (explode(",", $zoneArr["viewclients"]) as $viewclient) {
                $viewclients[$viewclient][] = $zoneArr["zone"] . "-" . $zoneArr["viewname"];
            }
        }
        $recordDB = new Records($this->auth);
        $this->trace("Updating SOA timestamp");
        $recordDB->updateSOA($zone, $viewname);
        $records = $recordDB->read($zone, $viewname);
        foreach ($this->servers as $server => $params) {
            if (!isset($params["type"])) {
                throw new \Exception(sprintf(
                    _("No defined server type for %s server"),
                    $server
                ), 500);
            }
            switch ($params["type"]) {
                case "bind":
                    // Construct the file in bind format for the server
                    $this->trace("Construct Bind zone");
                    $bind = new Bind();
                    $dataFile = $bind->createZoneFile($records);
                    $tmpfname = tempnam("/tmp", "domdnsZoneExport-");
                    if ($tmpfname === false) {
                        throw new \Exception(_("Couldn't create temporary file"), 500);
                    }
                    file_put_contents($tmpfname, $dataFile);
                    break;
                default:
                    throw new \Exception(sprintf(
                        _("Unknown server type '%s' for %s server"),
                        $params["type"],
                        $server
                    ), 500);
            }

            chmod($tmpfname, 0644);

            // Export file to server
            switch ($params["methodFILE"]) {
                case "scp":
                      $this->trace("Export zone to server by SCP");
                      $ssh = new \Models\Ssh();
                    if (isset($params["username"])) {
                        $ssh->username = $params["username"];
                    }
                    if (isset($params["password"])) {
                        $ssh->password = $params["password"];
                    }
                    if (isset($params["fingerprint"])) {
                        $ssh->fingerprint = $params["fingerprint"];
                    }
                    if (isset($params["publickeyfile"])) {
                        $ssh->publickeyfile = $params["publickeyfile"];
                    }
                    if (isset($params["privatekeyfile"])) {
                        $ssh->privatekeyfile = $params["privatekeyfile"];
                    }
                    if (isset($params["passphrase"])) {
                        $ssh->passphrase = $params["passphrase"];
                    }
                    if (! isset($params["zonepathSSHorSUDO"])) {
                        $params["zonepathSSHorSUDO"] = $params["zonepath"];
                    }
                    try {
                        $ssh->connect($server, $params["sshPort"]);
                        foreach ($viewclients as $viewclient => $content) {
                            $ssh->scpSend($tmpfname, $params["zonepathSSHorSUDO"] . "/$zone-" .
                            str_replace("/", "_", $viewclient));
                        }
                        $ssh->disconnect();
                    } catch (\Exception $e) {
                        unlink($tmpfname);
                        throw new \Exception($e->getMessage(), $e->getCode());
                    }
                    unlink($tmpfname);
                    $this->trace("DONE\n");
                    break;
                case "sudo":
                    $this->trace("Export zone to server by sudo");
                    if (! isset($params["zonepathSSHorSUDO"])) {
                        $params["zonepathSSHorSUDO"] = $params["zonepath"];
                    }
                    foreach ($viewclients as $viewclient => $content) {
                        exec(
                            "sudo -n -u " . $params["username"] . " /bin/cp $tmpfname " .
                            $params["zonepathSSHorSUDO"] . "/$zone-" .
                            str_replace("/", "_", $viewclient) . " 2>&1",
                            $output,
                            $rc
                        );
                        if ($rc !== 0) {
                            unlink($tmpfname);
                            throw new \Exception(sprintf(
                                _("Error when saving zone %s into server %s by sudo : %s"),
                                $zone,
                                $server,
                                implode("\n", $output)
                            ), 500);
                        }
                    }
                    unlink($tmpfname);
                    $this->trace("DONE\n");
                    break;
                default:
                    unlink($tmpfname);
                    throw new \Exception(sprintf(
                        _("Unknown configured methodFILE for server %s"),
                        $server
                    ), 500);
            }
        }

        return true;
    }

    /** Update the zone list files of the server */
    public function zonelistconfiguration()
    {
        // Limit to edit right
        if ($this->editrightZone() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to zones in RW",
                    $this->auth["email"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $zoneDB = new Zones($this->auth);
        $zones = $zoneDB->read(
            null,
            null,
            null,
            array(array("zone", "viewname"))
        );
        foreach ($this->servers as $server => $params) {
            if (!isset($params["type"])) {
                throw new \Exception(sprintf(
                    _("No defined server type for %s server"),
                    $server
                ), 500);
            }
            if (!isset($params["zonepath"])) {
                throw new \Exception(sprintf(
                    _("No defined zonepath for %s server"),
                    $server
                ), 500);
            }
            switch ($params["type"]) {
                case "bind":
                  // Construct the files in bind format for the server
                    $bind = new Bind();
                    if ($this->viewSupport === false) {
                        $this->trace("Construct Bind conf.local.withoutView");
                        $dataFile = $bind->createConfLocalWithoutView(
                            $zones,
                            $params["zonepath"]
                        );
                        $tmpfname = tempnam("/tmp", "domdns.conf.local.withoutView-");
                        if ($tmpfname === false) {
                            throw new \Exception(_("Couldn't create temporary file"), 500);
                        }
                        file_put_contents($tmpfname, $dataFile);
                    } else {
                        $this->trace("Construct Bind conf.local.withView");
                        $dataFile = $bind->createConfLocalWithView(
                            $zones,
                            $params["zonepath"]
                        );
                        $tmpfname = tempnam("/tmp", "domdns.conf.local.withView-");
                        if ($tmpfname === false) {
                            throw new \Exception(_("Couldn't create temporary file"), 500);
                        }
                        file_put_contents($tmpfname, $dataFile);
                    }
                    break;
                default:
                    throw new \Exception(sprintf(
                        _("Unknown server type '%s' for %s server"),
                        $params["type"],
                        $server
                    ), 500);
            }

            chmod($tmpfname, 0644);

            // Export file to server
            if (! isset($params["zonepathSSHorSUDO"])) {
                $params["zonepathSSHorSUDO"] = $params["zonepath"];
            }
            switch ($params["methodFILE"]) {
                case "scp":
                    $this->trace("Export named.conf.local to server by SCP");
                    $ssh = new \Models\Ssh();
                    if (isset($params["username"])) {
                        $ssh->username = $params["username"];
                    }
                    if (isset($params["password"])) {
                        $ssh->password = $params["password"];
                    }
                    if (isset($params["fingerprint"])) {
                        $ssh->fingerprint = $params["fingerprint"];
                    }
                    if (isset($params["publickeyfile"])) {
                        $ssh->publickeyfile = $params["publickeyfile"];
                    }
                    if (isset($params["privatekeyfile"])) {
                        $ssh->privatekeyfile = $params["privatekeyfile"];
                    }
                    if (isset($params["passphrase"])) {
                        $ssh->passphrase = $params["passphrase"];
                    }
                    try {
                        $ssh->connect($server, $params["sshPort"]);
                        if ($this->viewSupport === false) {
                            $ssh->scpSend($tmpfname, $params["zonepathSSHorSUDO"] .
                                       "/named.conf.local.withoutView");
                        } else {
                            $ssh->scpSend($tmpfname, $params["zonepathSSHorSUDO"] .
                                       "/named.conf.local.withView");
                        }
                        $ssh->disconnect();
                    } catch (\Exception $e) {
                        unlink($tmpfname);
                        throw new \Exception($e->getMessage(), $e->getCode());
                    }
                    unlink($tmpfname);
                    $this->trace("DONE\n");
                    break;
                case "sudo":
                    $this->trace("Export named.conf.local to server by sudo");
                    if ($this->viewSupport === false) {
                        exec(
                            "sudo -n -u " . $params["username"] . " /bin/cp $tmpfname " .
                            $params["zonepathSSHorSUDO"] . "/named.conf.local.withoutView 2>&1",
                            $output,
                            $rc
                        );
                    } else {
                        exec(
                            "sudo -n -u " . $params["username"] . " /bin/cp $tmpfname " .
                            $params["zonepathSSHorSUDO"] . "/named.conf.local.withView 2>&1",
                            $output,
                            $rc
                        );
                    }
                    if ($rc !== 0) {
                        unlink($tmpfname);
                        throw new \Exception(
                            sprintf(
                                _("Error when saving named.conf.local %s into dir %s " .
                                "by sudo : %s"),
                                $tmpfname,
                                $params["zonepathSSHorSUDO"],
                                implode("\n", $output)
                            ),
                            500
                        );
                    }
                    unlink($tmpfname);
                    $this->trace("DONE\n");
                    break;
                default:
                    unlink($tmpfname);
                    throw new \Exception(sprintf(
                        _("Unknown configured methodFILE for server %s"),
                        $server
                    ), 500);
            }
        }

        return true;
    }

    /** Return TRUE if the user right allow to edit the zone
        Return FALSE else */
    public function editright($zone, $viewname)
    {
        if ($this->auth["email"] === "cron") {
            return true;
        }
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        if (
            $authz->allow(
                "domdns",
                $this->auth["email"],
                "/$zone/$viewname"
            ) === "RW"
        ) {
            return true;
        }
        return false;
    }

    /** Return TRUE if the user right allow to see the zones
        Return FALSE else */
    public function accessright()
    {
        if ($this->auth["email"] === "cron") {
            return true;
        }
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        if (
            $authz->allow(
                "domdns",
                $this->auth["email"],
                "/"
            ) !== "NO"
        ) {
            return true;
        }
        return false;
    }

    /** Return TRUE if the user right allow to modify the zones
        Return FALSE else */
    public function editrightZone()
    {
        if ($this->auth["email"] === "cron") {
            return true;
        }
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        if (
            $authz->allow(
                "domdns",
                $this->auth["email"],
                "/"
            ) === "RW"
        ) {
            return true;
        }
        return false;
    }
    /** Create a new zone in the server */
//  public function zoneadd ($zone, $viewname)
//  {
//
//  }

  /** Delete an existing zone from the server */
//  public function zonedel ($zone, $viewname)
//  {
//
//  }

  /** Disable an existing zone from the server */
//  public function zonedisable ($zone)
//  {
//
//  }

  /** Enable an existing zone from the server */
//  public function zoneenable ($zone)
//  {
//
//  }
}
