<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Controllers;

use Domframework\Verify;
use Controllers\Zones;
use Models\Configuration;
use Models\Logging;
use Models\Userpref;

/** All the management of the user preferences */
class Userprefs
{
    /** Attributes to connect to the database */
    private $dbconfig = array("dsn" => null,
                             "username" => null,
                             "password" => null,
                             "driver_options" => null,
                             "tableprefix" => "");

    /** The anonymous user */
    private $auth = array("email" => "anonymous");

    /** Debug */
    public $debug = false;

    /** Check if the configuration of database is done */
    public function __construct($auth)
    {
        // Read the configuration and populate local variables
        $configuration = new Configuration();
        $this->dbconfig = $configuration->get("database");
        if ($this->dbconfig["dsn"] === null) {
            throw new \Exception(
                _("The database DSN parameter must be configured"),
                500
            );
        }
        if (! isset($this->dbconfig["tableprefix"])) {
            $this->dbconfig["tableprefix"] = "";
        }
        if (
            ! is_array($auth) ||
            ! array_key_exists("email", $auth)
        ) {
            throw new \Exception(
                _("Invalid auth provided to 'records' class"),
                403
            );
        }
        $this->auth = $auth;
    }

    /** Insert a new userpref
        @param array $data Data to be recorded in the new userpref :
           - string key The key of the data
           - string value The data to save
        @return string The last ID inserted */
    public function insert($data)
    {
        if (! is_array($data)) {
            throw new \Exception(_("Data provided are not an array"), 405);
        }
        if (! isset($data["key"])) {
            throw new \Exception(_("No key provided in data"), 405);
        }
        if (! isset($data["value"])) {
            throw new \Exception(_("No value provided in data"), 405);
        }
        $data["lastread"] = gmdate("Y-m-d", time());
        $data["lastupdate"] = gmdate("Y-m-d", time());
        $p = new Userpref(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $p->tableprefix = $this->dbconfig["tableprefix"];
        $p->debug = $this->debug;
        // Allow anonymous if it is allowed to see the zones
        $zone = new Zones($this->auth);
        if ($zone->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $data["zone"],
                    $data["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }

        $data["user"] = $this->auth["email"];
        // Security : Don't allow creation of data without checking if the
        // key is allowed. Risk of expose the disk capacity
        if (
            substr($data["key"], 0, 13) === "autoactivate-" &&
            $data["key"] !== "autoactivate-DEFAULT"
        ) {
            // Look at the zone-viewname
            $z = new Zones($this->auth);
            @list($zone, $viewname) = @explode("/", substr($data["key"], 13), 2);
            if ($zone === null || $viewname === null) {
                throw new \Exception(_("Incorrect zone/viewname provided"), 405);
            }
            $zonedata = $z->read($zone, $viewname);
            if (count($zonedata) === 0) {
                throw new \Exception(_("Zone/Viewname not allowed to be created"), 405);
            }
        } elseif (
            $data["key"] === "autoactivate-DEFAULT" ||
            $data["key"] === "language"
        ) {
            // OK Skip the exception
        } else {
            throw new \Exception(sprintf(
                _("The key '%s' is not allowed to be created"),
                $data["key"]
            ), 405);
        }
        $rc = $p->insert($data);
        return $rc;
    }

    /** Read a record
        @param string $key The key of the data
        @return array The data stored in the database or an empty array if
                       nothing stored */
    public function read($key)
    {
        $p = new Userpref(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $p->tableprefix = $this->dbconfig["tableprefix"];
        // The anonymous must read the data (at the beginning of index.php)
        $user = $this->auth["email"];
        $data = $p->read(array(array("user", $user), array("key", $key)));
        $data = reset($data);
        if ($data === false) {
            return array();
        }
        // Found ! Update the read date
        $tmp = $data;
        $tmp["lastread"] = gmdate("Y-m-d", time());
        $id = $data["id"];
        $p->update($id, $tmp);
        return $data;
    }

    /** Read all the records
        @return array The data stored in the database or an empty array if
                       nothing stored */
    public function getAll()
    {
        $p = new Userpref(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $p->tableprefix = $this->dbconfig["tableprefix"];
        // Allow anonymous if it is allowed to see the zones
        $zone = new Zones($this->auth);
        if ($zone->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s'",
                    $this->auth["email"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $user = $this->auth["email"];
        $data = $p->read(array(array("user", $user)));
        if ($data === false) {
            return array();
        }
        return $data;
    }

    /** Set all the provided parameters
        @param array $params The data to be stored in the database
        @return true if all is recorded
        @throw in cas of error */
    public function setAll($params)
    {
        $oldVals = array();
        $tmp = $this->getAll();
        foreach ($tmp as $vals) {
            $oldVals[$vals["key"]] = $vals["value"];
        }
        foreach ($params as $key => $val) {
            if (isset($oldVals[$key])) {
                $this->update($key, $val);
            } else {
                $this->insert(array("key" => $key, "value" => $val));
            }
        }
        return true;
    }

    /** Update a userpref
        @param string $key The key of the data
        @param string $value The data to save
        @return integer The number of records updated */
    public function update($key, $value)
    {
        $p = new Userpref(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $p->tableprefix = $this->dbconfig["tableprefix"];
        // Allow anonymous if it is allowed to see the zones
        $zone = new Zones($this->auth);
        if ($zone->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s'",
                    $this->auth["email"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $user = $this->auth["email"];
        $data = $p->read(array(array("user", $user), array("key", $key)));
        $data = reset($data);
        if ($data === false) {
            return 0;
        }
        $data["lastupdate"] = gmdate("Y-m-d", time());
        $data["value"] = $value;
        $rc = $p->update($data["id"], $data);
        return $rc;
    }

    /** Delete a userpref based on user/key
        @param string $key The associated key to delete
        @return integer The number of deleted records */
    public function delete($key)
    {
        $p = new Userpref(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $p->tableprefix = $this->dbconfig["tableprefix"];
        // Allow anonymous if it is allowed to see the zones
        $zone = new Zones($this->auth);
        if ($zone->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s'",
                    $this->auth["email"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        $user = $this->auth["email"];
        $data = $p->read(array(array("user", $user), array("key", $key)));
        $data = reset($data);
        if ($data === false) {
            return 0;
        }
        return $p->delete($data["id"]);
    }

    /** Disconnect from the database. Should be used only in the unit tests */
    public function disconnect()
    {
        $p = new Userpref(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $p->disconnect();
    }

    public function verify($values)
    {
        $res = array();
        foreach ($values as $key => $val) {
            $tmp = $this->verifyOne($key, $val);
            if (count($tmp) !== 0) {
                $res[] = $tmp;
            }
        }
        return $res;
    }

    public function verifyOne($field, $val)
    {
        $verify = new Verify();
        switch ($field) {
            case "autoactivate-DEFAULT":
                if ($val !== "0" && $val !== "1") {
                    $msg = _("Invalid value (allowed [01])");
                    return array("error", "$msg: $field");
                }
                break;
            case "language":
                if ($val !== "en_US" && $val !== "en_GB" && $val !== "fr_FR") {
                    $msg = _("Invalid language");
                    return array("error", "$msg: $field");
                }
                break;
            default:
                $msg = _("Unknown field");
                return array("error", "$msg: $field");
        }
        return array();
    }
}
