<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Controllers;

use Domframework\Authzgroups;
use Controllers\Servers;
use Models\Configuration;
use Models\Logging;
use Models\Record;
use Models\Userpref;
use Models\Zone;

/** All the management of zones */
class Zones
{
    /** Attributes to connect to the database */
    private $dbconfig = array("dsn" => null,
                             "username" => null,
                             "password" => null,
                             "driver_options" => null,
                             "tableprefix" => "");

    /** The SOA format */
    private $soa = array();

    /** The anonymous user */
    private $auth = array("email" => "anonymous");

    /** The default group when creating a new zone */
    public $defaultGroup = "";

    /** Debug */
    public $debug = false;

    /** Check if all the configuration is available before using it */
    public function __construct($auth)
    {
        // Read the configuration and populate local variables
        $configuration = new Configuration();
        $this->dbconfig = $configuration->get("database");
        $this->soa = $configuration->get("zone");
        $authorization = $configuration->get("authorization");
        if (isset($authorization["defaultGroup"])) {
            $this->defaultGroup = $authorization["defaultGroup"];
        }
        if ($this->dbconfig["dsn"] === null) {
            throw new \Exception(
                _("The database DSN parameter must be configured"),
                500
            );
        }
        if ($this->soa["soa-email"] === null) {
            throw new \Exception(
                _("The SOA-EMAIL parameter must be configured"),
                500
            );
        }
        if ($this->soa["soa-server"] === null) {
            throw new \Exception(
                _("The SOA-SERVER parameter must be configured"),
                500
            );
        }
        if (!isset($this->dbconfig["tableprefix"])) {
            $this->dbconfig["tableprefix"] = "";
        }
        if (
            ! is_array($auth) ||
            ! array_key_exists("email", $auth)
        ) {
            throw new \Exception(
                _("Invalid auth provided to 'records' class"),
                403
            );
        }
        $this->auth = $auth;
    }

    /** Insert a new zone and the first SOA record
        @param array $data : The data to be recorded :
          - string zone The zone name
          - string viewname The view name
          - string viewclients The IPS of clients to be used with this view
          - string comment Comments
          - string opendate The open date
          - string closedate The close date
        @return integer The last ID created */
    public function insert($data)
    {
        if ($this->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $data["zone"],
                    $data["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $data["zone"],
                    $data["viewname"]
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if (! is_array($data)) {
            throw new \Exception(_("Data provided are not an array"), 405);
        }
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $z->debug = $this->debug;
        if (isset($data["opendate"]) && $data["opendate"] !== "") {
            $data["opendate"] .= ":00";
        }
        if (isset($data["closedate"]) && $data["closedate"] !== "") {
            $data["closedate"] .= ":00";
        }
        if (! isset($data["opendate"])) {
            $data["opendate"] = gmdate("Y-m-d H:i:s");
        }
        $rcZone = $z->insert($data);
        unset($z);
        // Create the first record entry : the SOA
        $r = new Record(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $r->tableprefix = $this->dbconfig["tableprefix"];
        $r->debug = $this->debug;
        $rdata = array("zoneid" => $rcZone,
                   "source" => "@",
                   "class" => "IN",
                   "rr" => "SOA",
                   "addvalues" => "",
                   "target" => $this->soa["soa-server"] . " " .
                             $this->soa["soa-email"] . " ( " . date("Ymd") . "01 " .
                             $this->soa["soa-refresh"] . " " .
                             $this->soa["soa-retry"] . " " .
                             $this->soa["soa-expire"] . " " .
                             $this->soa["soa-minttl"] . " )",
                   "ttl" => $this->soa["soa-minttl"],
                   "opendate" => date("Y-m-d H:i:s"));
        $rc = $r->insert($rdata);

        // Insert the zone from the authorization
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $authz->debug = $this->debug;
        $authz->objectAdd("domdns", "/" . $data["zone"] . "/" . $data["viewname"]);
        if ($this->defaultGroup !== "") {
            $authz->rightAdd(
                "domdns",
                $this->defaultGroup,
                "/" . $data["zone"] . "/" . $data["viewname"],
                "RW"
            );
        }
        Logging::log(LOG_WARNING, "'" . $this->auth["email"] .
                  "' create zone '" . $data["zone"] . "' with view '" .
                  $data["viewname"] . "'");
        // Update the named.conf.local in the servers
        $servers = new Servers($this->auth);
        $servers->zonelistconfiguration();
        $servers->reconfig();
        return ($rcZone);
    }

    /** Read all the zones or all the viewnames allowed for the user
        @param string|null $zone The zone name to display (all the zones if null)
        @param string|null $viewname The view to display (all the views if null)
        @param array|null $display The fields to display
        @param array|null $order The order and direction to sort the entries */
    public function read(
        $zone = null,
        $viewname = null,
        $display = null,
        $order = null
    ) {
        if ($this->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($zone !== null && ! is_string($zone)) {
            throw new \Exception(_("Zone provided is not a string"), 405);
        }
        if ($viewname !== null && ! is_string($viewname)) {
            throw new \Exception(_("View name provided is not a string"), 405);
        }
        if ($display !== null && ! is_array($display)) {
            throw new \Exception(_("Displayed columns asked are not an array"), 405);
        }
        if ($order !== null && ! is_array($order)) {
            throw new \Exception(_("Order field asked is not an array"), 405);
        }
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $z->debug = $this->debug;
        $select = null;
        if ($zone !== null) {
            $select[] = array("zone", $zone);
            if ($viewname !== null) {
                $select[] = array("viewname", $viewname);
            }
        }
        $zones = $z->read($select, $display, $order);
        // Add the disabled flag
        $zones = $z->addDisable($zones);
        foreach ($zones as $key => $vals) {
            if (isset($vals["opendate"]) && $vals["opendate"] !== "") {
                $zones[$key]["opendate"] = substr($vals["opendate"], 0, -3);
            }
            if (isset($vals["closedate"]) && $vals["closedate"] !== "") {
                $zones[$key]["closedate"] = substr($vals["closedate"], 0, -3);
            }
        }
        if ($this->auth["email"] === "cron") {
            return $zones;
        }

        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $authz->debug = $this->debug;
        // Limit to authorization
        foreach ($zones as $k => $val) {
            if (
                $authz->allow(
                    "domdns",
                    $this->auth["email"],
                    "/" . $val["zone"] . "/" . $val["viewname"]
                ) === "NO"
            ) {
                unset($zones[$k]);
            } elseif (
                $authz->allow(
                    "domdns",
                    $this->auth["email"],
                    "/" . $val["zone"] . "/" . $val["viewname"]
                ) === "RO"
            ) {
                $zones[$k]["right"] = "RO";
            } else {
                $zones[$k]["right"] = "RW";
            }
        }

        return $zones;
    }

    /** Update a zone
        @param string $zone The zone name to update
        @param string $viewname The view name to update
        @param array $data The data to be recorded
           - string|null viewclients IP of view clients
           - string|null comment Comments
           - string|null opendate Open date
           - string|null closedate Close date */
    public function update($zone, $viewname, $data)
    {
        if ($this->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s'in RO",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->readonly($zone, $viewname) === true) {
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' : RO zone",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if (! is_string($zone)) {
            throw new \Exception(_("Zone provided is not a string"), 405);
        }
        if (! is_string($viewname)) {
            throw new \Exception(_("View name provided is not a string"), 405);
        }
        if (! is_array($data)) {
            throw new \Exception(_("Data provided are not an array"), 405);
        }
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $z->debug = $this->debug;
        $ids = $z->read(array(array("zone", $zone),
                            array("viewname", $viewname)));
        $key = reset($ids);
        if ($key === false) {
            throw new \Exception(_("Zone not found with this view name"), 404);
        }
        $key = $key["id"];
        if (isset($data["opendate"]) && $data["opendate"] !== "") {
            $data["opendate"] .= ":00";
        }
        if (isset($data["closedate"]) && $data["closedate"] !== "") {
            $data["closedate"] .= ":00";
        }
        $rc = $z->update($key, $data);
        unset($z);
        if (! isset($data["zone"])) {
            $data["zone"] = $zone;
        }
        if (! isset($data["viewname"])) {
            $data["viewname"] = $viewname;
        }
        // Update existing userprefs if zone or viewname is changed
        if (
            (isset($data["zone"]) && $data["zone"] !== $zone) ||
            (isset($data["viewname"]) && $data["viewname"] !== $viewname)
        ) {
            $p = new Userpref(
                $this->dbconfig["dsn"],
                $this->dbconfig["username"],
                $this->dbconfig["password"],
                $this->dbconfig["driver_options"]
            );
            $p->tableprefix = $this->dbconfig["tableprefix"];
            $p->debug = $this->debug;
            $edit = $p->read(array(array("key", "autoactivate-$zone/$viewname")));
            $prefs = array();
            if ((isset($data["zone"]) && $data["zone"] !== $zone)) {
                $prefs["zone"] = $data["zone"];
            } else {
                $prefs["zone"] = $zone;
            }
            if ((isset($data["viewname"]) && $data["viewname"] !== $zone)) {
                $prefs["viewname"] = $data["viewname"];
            } else {
                $prefs["viewname"] = $viewname;
            }
            foreach ($edit as $e) {
                $p->update($e["id"], array("key" => "autoactivate-" . $prefs["zone"] . "/" .
                                            $prefs["viewname"]));
            }
        }
        if ($rc === 0) {
            throw new \Exception(_("No data modified"));
        } elseif ($rc !== 1) {
            throw new \Exception(_("More than one line modified"));
        }

        // Update the zone from the authorization
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $authz->debug = $this->debug;
        $authz->objectUpdate(
            "domdns",
            "/$zone/$viewname",
            "/" . $data["zone"] . "/" . $data["viewname"]
        );
        Logging::log(LOG_WARNING, "'" . $this->auth["email"] .
                  "' update zone '$zone' with view '$viewname' to '" .
                  $data["zone"] . "' with view '" . $data["viewname"] . "'");
        // Update the named.conf.local in the servers
        $servers = new Servers($this->auth);
        $servers->zonelistconfiguration();
        $servers->reconfig();
        return true;
    }

    /** Delete a zone
        @param string $zone The zone to delete
        @param string $viewname The view name to delete */
    public function delete($zone, $viewname)
    {
        if ($this->accessright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RO",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->editright() !== true) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' in RW",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if ($this->readonly($zone, $viewname) === true) {
            Logging::log(
                LOG_WARNING,
                sprintf(
                    "Access forbidden for '%s' to '%s/%s' : RO zone",
                    $this->auth["email"],
                    $zone,
                    $viewname
                )
            );
            throw new \Exception(sprintf(
                _("Access forbidden for '%s'"),
                $this->auth["email"]
            ), 403);
        }
        if (! is_string($zone)) {
            throw new \Exception(_("Zone provided is not a string"), 405);
        }
        if (! is_string($viewname)) {
            throw new \Exception(_("View name provided is not a string"), 405);
        }
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        $z->debug = $this->debug;
        $ids = $z->read(array(array("zone", $zone),
                            array("viewname", $viewname)));
        $key = reset($ids);
        if ($key === false) {
            throw new \Exception(_("Zone not found with this view name"), 404);
        }
        $key = $key["id"];
        $rc = $z->delete($key);

        // Remove the zone from the authorization
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $authz->debug = $this->debug;
        $authz->objectDel("domdns", "/$zone/$viewname");

        // Delete userprefs for this zone
        $p = new Userpref(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $p->tableprefix = $this->dbconfig["tableprefix"];
        $edit = $p->read(array(array("key", "autoactivate-$zone/$viewname")));
        foreach ($edit as $e) {
            $p->delete($e["id"]);
        }
        unset($z);

        Logging::log(LOG_WARNING, "'" . $this->auth["email"] .
                  "' delete zone '$zone' with view '$viewname'");
        // Update the named.conf.local in the servers
        $servers = new Servers($this->auth);
        $servers->zonelistconfiguration();
        $servers->reconfig();
        return $rc;
    }

    /** Return the translation of the titles (the columns in the database) */
    public function titles()
    {
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        return $z->titles();
    }

    /** Return TRUE if the provided zone is a direct zone or FALSE otherwise */
    public function direct($zone)
    {
        if (substr($zone, -6) === ".arpa.") {
            return false;
        }
        return true;
    }

    public function verifyOne($field, $val)
    {
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        if ($field === "opendate" && $val !== "") {
            $val .= ":00";
        }
        if ($field === "closedate" && $val !== "") {
            $val .= ":00";
        }
        return $z->verifyOne($field, $val);
    }

    public function verifyAll($data)
    {
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        if (isset($data["opendate"]) && $data["opendate"] !== "") {
            $data["opendate"] .= ":00";
        }
        if (isset($data["closedate"]) && $data["closedate"] !== "") {
            $data["closedate"] .= ":00";
        }
        return $z->verifyAll($data);
    }

    public function verify($data, $updatekey = false)
    {
        $z = new Zone(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $z->tableprefix = $this->dbconfig["tableprefix"];
        if (isset($data["opendate"]) && $data["opendate"] !== "") {
            $data["opendate"] .= ":00";
        }
        if (isset($data["closedate"]) && $data["closedate"] !== "") {
            $data["closedate"] .= ":00";
        }
        return $z->verify($data, $updatekey);
    }

    /** Return TRUE if the zone/viewname is in READONLY for the user or FALSE if
        the user have the RW rights */
    public function readonly($zone, $viewname)
    {
        if ($this->auth["email"] === "cron") {
            return true;
        }
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $authz->debug = $this->debug;
        $rc = $authz->allow("domdns", $this->auth["email"], "/$zone/$viewname");
        Logging::log(
            LOG_DEBUG,
            "ZoneAuth : readonly('" . $this->auth["email"] . "','$zone'," .
            "'$viewname')=$rc"
        );
        if ($rc === "RW") {
            return false;
        }
        return true;
    }

    /** Return true if the zone is disabled, or false otherwise
      */
    public function disabled($zone, $viewname)
    {
        $zones = $this->read($zone, $viewname);
        $zoneData = reset($zones);
        return $zones[0]["disabled"];
    }

    /** Return TRUE if the user right allow to edit the zone
        Return FALSE else */
    public function editright()
    {
        if ($this->auth["email"] === "cron") {
            return false;
        }
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $authz->debug = $this->debug;
        $rc = $authz->allow("domdns", $this->auth["email"], "/");
        Logging::log(
            LOG_DEBUG,
            "ZoneAuth : editright('" . $this->auth["email"] . "','/')=$rc"
        );
        if ($rc === "RW") {
            return true;
        }
        return false;
    }

    /** Return TRUE if the user right allow to see the zones
        Return FALSE else */
    public function accessright()
    {
        if ($this->auth["email"] === "cron") {
            return true;
        }
        $authz = new Authzgroups();
        $authz->connect(
            $this->dbconfig["dsn"],
            $this->dbconfig["username"],
            $this->dbconfig["password"],
            $this->dbconfig["driver_options"]
        );
        $authz->tableprefix = $this->dbconfig["tableprefix"];
        $authz->debug = $this->debug;
        $rc = $authz->allow("domdns", $this->auth["email"], "/");
        Logging::log(
            LOG_DEBUG,
            "ZoneAuth : accessright ('" . $this->auth["email"] .
            "','/')=$rc"
        );
        if ($rc !== "NO") {
            return true;
        }
        return false;
    }
}
