<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Models;

/** Manage all the bind files (zone files, import/export Bind files) */
class Bind
{
    /** Create the content of zone file in bind format
        @param array $records An array containing source, class, rr, addvalues,
                              target, comment
        @return string The content of Bind zone file */
    public function createZoneFile($records)
    {
        if (! is_array($records)) {
            throw new \Exception(_("Records provided are not an array"), 405);
        }

        $fields = array("disabled", "source", "ttl", "class", "rr", "addvalues",
                    "target", "comment");
        $data = ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n";
        $data .= ";                                                             ;\n";
        $data .= ";     THIS FILE IS AUTOMATICALLY GENERATED FROM DATABASE      ;\n";
        $data .= ";                                                             ;\n";
        $data .= ";                 DO NOT MODIFY IT BY HAND !!!                ;\n";
        $data .= ";                                                             ;\n";
        $data .= ";;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;\n";
        // var_dump ($records);
        // Force the source @ on head of zone
        foreach ($records as $linenb => $values) {
            if ($values["source"] !== "@") {
                continue;
            }
            unset($records[$linenb]);
            array_unshift($records, $values);
        }

        // Force the SOA on first line
        foreach ($records as $linenb => $values) {
            if ($values["rr"] !== "SOA") {
                continue;
            }
            unset($records[$linenb]);
            array_unshift($records, $values);
            break;
        }

        foreach ($records as $linenb => $values) {
            foreach ($fields as $field) {
                if (!array_key_exists($field, $values)) {
                    throw new \Exception("No field $field available", 404);
                }
                switch ($field) {
                    case "disabled":
                        if ($values[$field] === true) {
                            $data .= ";";
                        }
                        break;
                    case "ttl":
                        $data .= sprintf("%-5s ", $values[$field]);
                        break;
                    case "class":
                        $data .= sprintf("%-2s ", $values[$field]);
                        break;
                    case "rr":
                        $data .= sprintf("%-10s ", $values[$field]);
                        break;
                    case "addvalues":
                          $data .= sprintf("%-4s ", $values[$field]);
                        break;
                    case "comment":
                        if (! empty($values[$field])) {
                            $data .= " ; " . $values[$field];
                        }
                        break;
                    default:
                        $data .= sprintf("%-25s ", $values[$field]);
                }
            }

            $data .= "\n";
        }

        return $data;
    }

    /** Import data from zone file.
        @param string $dataTXT The zone file from Bind
        @param string $zone The zone name
        @return array The data containing source, class, rr, addvalues,
                              target, comment */
    public function import($dataTXT, $zone)
    {
        if (! is_string($dataTXT)) {
            throw new \Exception(_("Data for zone are not a string"), 405);
        }
        $data = explode("\n", $dataTXT);
        $res = array();
        $lineold = "";
        // If there is no source in the line, use the previous one
        $lastSource = "";
        // If no $ORIGIN is provided, use the zone name
        $originVar = $zone;
        $ttlVar = "";
        $inParenthesis = false;
        $content = array();
        $commentedLine = false;
        foreach ($data as $nb => $line) {
            //var_dump($line);
            //var_dump($commentedLine);
            if (trim($line) === "") {
                // Blank lines
                continue;
            }
            if (strpos($line, ";") === 0) {
                // Comments
                $commentedLine = true;
                $line = rtrim(substr($line, 1));
            }
            unset($matches);
            if ($commentedLine === false) {
                preg_match('/^\$ORIGIN[ \t]+(\S+)/', $line, $matches);
                if (isset($matches[1])) {
                    $originVar = $matches[1];
                }
                preg_match('/^\$TTL[ \t]+(\S+)/', $line, $matches);
                if (isset($matches[1])) {
                    $ttlVar = $matches[1];
                }
                if (strpos($line, "\$") === 0) {
                    // Variables are previously used : skip the line
                    continue;
                }
            }
            // Merge multiple lines in one. Multiple lines are done with parenthesis
            // Remove the internal comments, detect the blocks to allow semi-colon in
            // blocks
            if ($lineold !== "") {
                $line = $lineold . " " . trim($line);
                $lineold = "";
            }
            $parentOpen = substr_count($line, "(");
            if ($parentOpen) {
                $inParenthesis = true;
            }
            if ($inParenthesis) {
                $parentClose = substr_count($line, ")");
                if ($parentClose) {
                    $inParenthesis = false;
                } else {
                    // If there is a block with double-quotes, manage it
                    if (($startBlock = strpos($line, '"')) !== false) {
                        $lineold .= substr($line, 0, $startBlock);
                        $endBlock = strpos($line, '"', $startBlock + 1);
                        if ($endBlock === false) {
                            throw new \Exception(_("Unbalanced double-quotes on line ") .
                              ($nb + 1), 500);
                        }
                        $block = substr($line, $startBlock, $endBlock - $startBlock);
                        $lineold .= $block;
                        $line = substr($line, $endBlock);
                    }
                    // Remove the comments on multiple lines except on lastline
                    $pos = strpos($line, ";");
                    if ($pos !== false) {
                        $lineold .= substr($line, 0, $pos + 1);
                    } else {
                        $lineold .= $line;
                    }
                    continue;
                }
            }

            $line = rtrim($line);
            unset($matches);
            $matches = preg_split("/(\s+)/", $line);
            if (! isset($matches[0]) && $commentedLine === true) {
                // Comment line without valid data are skipped
                $commentedLine = false;
                continue;
            }
            $source = $matches[0];
            $lastSource = $source;
            $matches = array_slice($matches, 1);
            if (! isset($matches[0]) && $commentedLine === true) {
                // Comment line without valid data are skipped
                $commentedLine = false;
                continue;
            }
            $ttl = "3600";
            if (strspn($matches[0], "0123456789") === strlen($matches[0])) {
                // TTL beetween source and class
                $ttl = $matches[0];
                $matches = array_slice($matches, 1);
            }

            $class = "IN";
            if ($matches[0] === "IN" || $matches[0] === "CH" || $matches[0] === "HS") {
                $class = $matches[0];
                $matches = array_slice($matches, 1);
            }

            if (! isset($matches[0]) && $commentedLine === true) {
                // Comment line without valid data are skipped
                $commentedLine = false;
                continue;
            }
            $rr = $matches[0];
            $addvalues = "";
            $matches = array_slice($matches, 1);
            if ($rr === "SOA" || $rr === "SPF" || $rr === "SRV" || $rr === "TXT") {
                // All the end of line is the target when using SOA/SPF/SRV/TXT,
                // except comment
                $target = "";
                $block = false;
                $comment = false;
                $k = 0;
                foreach ($matches as $k => $m) {
                    if (substr($m, 0, 1) === "(") {
                        $block = true;
                        if ($k > 0) {
                            $target .= " ";
                        }
                        $target .= $m;
                    } elseif (substr($m, -1) === ")") {
                        $block = false;
                        if ($k > 0) {
                            $target .= " ";
                        }
                        $target .= $m;
                    } elseif ($block === true) {
                        if ($k > 0) {
                            $target .= " ";
                        }
                        $target .= $m;
                    } elseif (substr($m, 0, 1) === ";") {
                        break;
                    } else {
                        if ($k > 0) {
                            $target .= " ";
                        }
                        $target .= $m;
                    }
                }

                if (count($matches) === ($k + 1)) {
                    // There is NO ending comment
                    $matches = array();
                } else {
                    $matches = array_slice($matches, $k);
                }
            } elseif ($rr === "CAA") {
                // Additionnal values are present. Defined on two fields : "0 issue" or
                // "0 issuedef" or "0 iodef"
                $addvalues = $matches[0] . " " . $matches[1];
                ;
                $target = $matches[2];
                $matches = array_slice($matches, 3);
                @list($target, $comment) = explode(";", $target, 2);
                if ($comment !== null) {
                    // If the comment are stick with the target (without space)
                    array_unshift($matches, $comment);
                    array_unshift($matches, ";");
                }
            } elseif ($rr === "MX") {
                // Additionnal values are present : use them
                $addvalues = $matches[0];
                $target = $matches[1];
                $matches = array_slice($matches, 2);
                @list($target, $comment) = explode(";", $target, 2);
                if ($comment !== null) {
                    // If the comment are stick with the target (without space)
                    array_unshift($matches, $comment);
                    array_unshift($matches, ";");
                }
            } elseif ($rr === "SSHFP") {
                // Additionnal values are present. Defined on two fields : "[1234] [12]"
                $addvalues = $matches[0] . " " . $matches[1];
                ;
                $target = $matches[2];
                $matches = array_slice($matches, 3);
                @list($target, $comment) = explode(";", $target, 2);
                if ($comment !== null) {
                    // If the comment are stick with the target (without space)
                    array_unshift($matches, $comment);
                    array_unshift($matches, ";");
                }
            } elseif (isset($matches[0])) {
                // Additionnal values are not presents : skip them
                $target = $matches[0];
                $matches = array_slice($matches, 1);
                @list($target, $comment) = explode(";", $target);
                if ($comment !== null) {
                    // If the comment are stick with the target (without space)
                    array_unshift($matches, $comment);
                    array_unshift($matches, ";");
                }
            } elseif ($commentedLine === false) {
                throw new \Exception(_("Error on source file line ") . ($nb + 1), 500);
            }

            // The comments start by a semi-colon (which is removed in the resulting
            // array)
            $comment = implode(" ", $matches);
            if (strlen($comment)) {
                $comment = substr($comment, 2);
            }

            $closedate = "";
            if ($commentedLine === false) {
                // A not commented Line with an error will be catched
                if (empty($source) || empty($rr) || empty($target)) {
                    throw new \Exception(_("Error on source file line ") . ($nb + 1), 500);
                }
            } else {
                if (empty($source) || empty($rr) || empty($target)) {
                    // A line with only comments will not be inserted and just skipped
                    $commentedLine = false;
                    continue;
                }
                // A line in comment with valid data will be stored but in disabled
                // state
                $closedate = gmdate("Y-m-d H:i:s");
            }
            // Overload source or target with ORIGIN
            if (
                filter_var($source, FILTER_VALIDATE_IP) === false &&
                substr($source, -1) !== "."
            ) {
                if ($source === "@") {
                    // Nothing to do
                } elseif ($originVar !== ".") {
                    $val = "$source.$originVar";
                    $source = $val;
                } else {
                    $val = "$source$originVar";
                    $source = $val;
                }
            }
            if (
                filter_var($target, FILTER_VALIDATE_IP) === false &&
                substr($target, -1) !== "." &&
                $rr !== "CAA" && $rr !== "SOA" && $rr !== "TXT" &&
                $rr !== "SPF" && $rr !== "SRV" && $rr !== "SSHFP"
            ) {
                if ($originVar !== ".") {
                    $val = "$target.$originVar";
                    $target = $val;
                } else {
                    $val = "$target$originVar";
                    $target = $val;
                }
            }

            // Removing the zone name in FQDN if it is the actual zone
            if (substr($source, -1 * strlen($zone)) === $zone) {
                if ($zone === ".") {
                    $source = substr($source, 0, -1);
                } else {
                    $source = substr($source, 0, -1 - 1 * strlen($zone));
                }
            }
            if ($source === false || $source === "" || $rr === "SOA") {
                $source = "@";
            }

            $resLine = array("source" => $source, "class" => $class,
                       "rr" => $rr, "addvalues" => $addvalues,
                       "target" => $target, "ttl" => $ttl,
                       "comment" => $comment, "closedate" => $closedate);
            $res[] =  $resLine;
            $commentedLine = false;
        }

        return $res;
    }

    /** Create the content of conf.local file with the zones. This configuration
        don't support the views.
        @param array $zones All the Zones to be generated in Bind
        @param string $zonepath The path in the Bind server
        @return string The file content to be included in Bind configuration */
    public function createConfLocalWithoutView($zones, $zonepath)
    {
        if (! is_array($zones)) {
            throw new \Exception(_("Zones provided are not an array"), 405);
        }
        if (! is_string($zonepath)) {
            throw new \Exception(_("Zonepath provided is not a string"), 405);
        }
        if (substr($zonepath, -1) !== "/") {
            $zonepath .= "/";
        }
        // Don't allow the creation of zones with multiple views
        $zonesWrited = array();
        $currentDate = date("Y-m-d H:i:s");
        $res = "";
        $res .= "dnssec-policy domdns-dnssec-policy-unlimited {\n";
        $res .= "  # Force les clés à rester unlimited et non changées\n";
        $res .= "  # 8 est le type de clé (écrit dans le nom du fichier de clé)\n";
        $res .= "  keys {\n";
        $res .= "    ksk lifetime unlimited algorithm 8;\n";
        $res .= "    zsk lifetime unlimited algorithm 8;\n";
        $res .= "  };\n";
        $res .= "};\n";

        foreach ($zones as $zoneArr) {
            $zone = $zoneArr["zone"];
            $viewname = $zoneArr["viewname"];
            // Manage the closed zones
            if ($zoneArr["disabled"] === true) {
                $disabled = "#";
            } else {
                $disabled = "";
            }
            if ($disabled === "" && in_array($zone, $zonesWrited)) {
                throw new \Exception(sprintf(
                    _("Zone '%s' can't be used twice in bind without views"),
                    $zone
                ), 406);
            }
            if ($disabled === "") {
                $zonesWrited[] = $zone;
            }
            $res .= $disabled . "zone \"$zone\" {\n";
            if ($zoneArr["comment"] !== null && $zoneArr["comment"] !== "") {
                $res .= $disabled . "  # " . $zoneArr["comment"] . "\n";
            }
            $res .= $disabled . "  type master;\n";
            $res .= $disabled . "  allow-query { any; };\n";
            $res .= $disabled . "  file \"$zonepath$zone-any\";\n";
            $res .= $disabled . "  notify yes;\n";
            if ($zoneArr["dnssec"] === "1") {
                $res .= $disabled . "  ##auto-dnssec maintain;\n";
                $res .= $disabled . "  dnssec-policy domdns-dnssec-policy-unlimited;\n";
                $res .= $disabled . "  update-policy local;\n";
                $res .= $disabled . "  inline-signing yes;\n";
            }
            $res .= $disabled . "};\n\n";
        }
        return $res;
    }

    /** Create the content of conf.local file with the zones. This configuration
        support the views.
        @param array $zones All the Zones to be generated in Bind
        @param string $zonepath The path in the Bind server
        @return string The file content to be included in Bind configuration */
    public function createConfLocalWithView($zones, $zonepath)
    {
        if (! is_array($zones)) {
            throw new \Exception(_("Zones provided are not an array"), 405);
        }
        if (! is_string($zonepath)) {
            throw new \Exception(_("Zonepath provided is not a string"), 405);
        }
        if (substr($zonepath, -1) !== "/") {
            $zonepath .= "/";
        }
        $currentDate = date("Y-m-d H:i:s");
        // Look at the differents view clients
        $viewclients = array();
        foreach ($zones as $zoneArr) {
            if (
                $zoneArr["viewclients"] === null ||
                trim($zoneArr["viewclients"]) === ""
            ) {
                $viewclients["any"][] = $zoneArr["zone"] . "-" . $zoneArr["viewname"];
                continue;
            }
            foreach (explode(",", $zoneArr["viewclients"]) as $viewclient) {
                $viewclients[$viewclient][] = $zoneArr["zone"] . "-" . $zoneArr["viewname"];
            }
        }
        // The sort of the views allow the "any" to be pushed after the IPs
        // TODO : Sort by netmask depth from /128 to /0
        ksort($viewclients, SORT_NATURAL);
        $res = "";
        $res .= "dnssec-policy domdns-dnssec-policy-unlimited {\n";
        $res .= "  # Force les clés à rester unlimited et non changées\n";
        $res .= "  # 8 est le type de clé (écrit dans le nom du fichier de clé)\n";
        $res .= "  keys {\n";
        $res .= "    ksk lifetime unlimited algorithm 8;\n";
        $res .= "    zsk lifetime unlimited algorithm 8;\n";
        $res .= "  };\n";
        $res .= "};\n";

        foreach ($viewclients as $viewclient => $clients) {
            $res .= "view \"$viewclient\" {\n";
            $res .= "  match-clients { $viewclient; };\n";
            $res .= "  allow-query { $viewclient; };\n";
            foreach ($zones as $zoneArr) {
                $zone = $zoneArr["zone"];
                $viewname = $zoneArr["viewname"];
                if (! in_array($zoneArr["zone"] . "-" . $zoneArr["viewname"], $clients)) {
                    continue;
                }
                // Manage the closed zones
                if ($zoneArr["disabled"] === true) {
                    $disabled = "#";
                } else {
                    $disabled = "";
                }
                $res .= "  " . $disabled . "zone \"$zone\" {\n";
                if ($zoneArr["comment"] !== null && $zoneArr["comment"] !== "") {
                    $res .= "  " . $disabled . "  # " . $zoneArr["comment"] . "\n";
                }
                $res .= "  " . $disabled . "  type master;\n";
                // We need one zone file by viewclient not by viewname, or DNSSEC won't
                // work
                $res .= "  " . $disabled . "  file \"$zonepath$zone-" .
                str_replace("/", "_", $viewclient) . "\";\n";
                $res .= "  " . $disabled . "  notify yes;\n";
                if ($zoneArr["dnssec"] === "1" || $zoneArr["dnssec"] === 1) {
                    $res .= "  " . $disabled . "  ##auto-dnssec maintain;\n";
                    $res .= "  " . $disabled . "  dnssec-policy domdns-dnssec-policy-unlimited;\n";
                    $res .= "  " . $disabled . "  update-policy local;\n";
                    $res .= "  " . $disabled . "  inline-signing yes;\n";
                }
                $res .= "  " . $disabled . "};\n";
            }
            $res .= "};\n\n";
        }
        return $res;
    }
}
