<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Models;

use Domframework\Dblayer;
use Domframework\Verify;

/** All the methods to manage a record */
class Record extends Dblayer
{
    /* The database must be initialized with
       CREATE TABLE `dns_records` (id INTEGER NOT NULL,
                                   zoneid INTEGER NOT NULL,
                                   source VARCHAR(255) NOT NULL,
                                   class VARCHAR (2) NOT NULL,
                                   rr VARCHAR (10) NOT NULL,
                                   addvalues VARCHAR (10),
                                   target VARCHAR(2048) NOT NULL,
                                   ttl INTEGER,
                                   comment VARCHAR (1024),
                                   opendate DATETIME NOT NULL,
                                   closedate DATETIME,
                                   PRIMARY KEY(id),
                                   FOREIGN KEY(zoneid) REFERENCES dns_zones(id)
                                        ON UPDATE CASCADE ON DELETE CASCADE);
    */

    /** The name of the table in the database */
    public $table = "dns_records";
    /** The prefix defined in the configuration */
    public $tableprefix = "";

    /** The fields in the table with their specifications */
    public $fields = array(
    "id" => array("integer", "not null", "autoincrement"),
    "zoneid" => array("integer", "not null"),
    "source" => array("varchar", "255", "not null"),
    "class" => array("varchar", "2", "not null"),
    "rr" => array("varchar", "10", "not null"),
    "addvalues" => array("varchar", "10"),
    "target" => array("varchar", "2048", "not null"),
    "ttl" => array("integer"),
    "comment" => array("varchar", "1024"),
    "opendate" => array("datetime", "not null"),
    "closedate" => array("datetime"),
    );
    /** The primary key of the table */
    public $primary = "id";
    /** The unique constrain of the table */
    public $unique = array("id",
                   array("zoneid","source","class","rr","addvalues","target"));
    /** The foreign keys of the table */
    public $foreign = array(
    "zoneid" => array("dns_zones", "id", "ON UPDATE CASCADE ON DELETE CASCADE"),
    );
    /** SQL Debug */
    public $debug = false;

    /** Translation */
    public function titles()
    {
        return  array(
        "id" => _("ID"),
        "zoneid" => _("ID Zone"),
        "source" => _("Source"),
        "class" => _("Class"),
        "rr" => _("RR"),
        "addvalues" => _("Add values"),
        "target" => _("Target"),
        "ttl" => _("TTL"),
        "comment" => _("Comment"),
        "opendate" => _("Open date"),
        "closedate" => _("Close date"),
        );
    }

    public function verifyOne($field, $val)
    {
        $verify = new Verify();
        switch ($field) {
            case "id":
                break;
            case "zoneid":
                if (strlen($val) < 1) {
                    $msg = _("Not enough chars");
                    return array("error", $msg);
                }
                if (strlen($val) > 30) {
                    $msg = _("Too much chars");
                    return array("warning", $msg);
                }
                if (is_numeric($val) !== true) {
                    $msg = _("Not numeric");
                    return array("error", $msg);
                }
                break;
            case "source":
                if (strlen($val) < 1) {
                    $msg = _("Not enough chars");
                    return array("error", $msg);
                }
                if (strlen($val) > 255) {
                    $msg =  _("Too much chars");
                    return array("warning", $msg);
                }
                if (
                    strspn($val, "ABCDEFGHIJKLMNOPQRSTUVWXYZ" .
                        "abcdefghijklmnopqrstuvwxyz" .
                        "0123456789.@*-_") !== strlen($val)
                ) {
                    $msg =  _("Invalid character provided in source");
                    return array("error", $msg);
                }
                if (strpos($val, "*") !== false && strpos($val, "*") !== 0) {
                    $msg = _("The joker must be on first char");
                    return array("error", $msg);
                }
                break;
            case "class":
                if (strlen($val) <= 1) {
                    $msg = _("Not enough chars");
                    return array("error", $msg);
                }
                if (strlen($val) > 2) {
                    $msg = _("Too much chars");
                    return array("warning", $msg);
                }
                if (in_array($val, array('IN','CH','HS')) !== true) {
                    $msg = _("Unknown class");
                    return array("error", $msg);
                }
                break;
            case "rr":
                if (strlen($val) < 1) {
                    $msg = _("Not enough chars");
                    return array("error", $msg);
                }
                if (strlen($val) > 10) {
                    $msg = _("Too much chars");
                    return array("warning", $msg);
                }
                if (
                    in_array($val, array("A", "AAAA", "CAA", "CNAME", "MX", "NS", "PTR",
                                 "SOA", "SPF", "SRV", "SSHFP", "TXT"))
                    !== true
                ) {
                    $msg = _("Unknown RR");
                    return array("error", $msg);
                }
                break;
            case "addvalues":
                if (strlen($val) > 10) {
                    $msg = _("Too much chars");
                    return array("warning", $msg);
                }
                break;
            case "target":
                if (strlen($val) <= 1) {
                    $msg = _("Not enough chars");
                    return array("error", $msg);
                }
                if (strlen($val) > 2048) {
                    $msg = _("Too much chars");
                    return array("warning", $msg);
                }
                if (substr_count($val, "\"") % 2) {
                    $msg = _("The number of double quotes are not even");
                    return array("error", $msg);
                }
                if (substr_count($val, "(") !== substr_count($val, ")")) {
                    $msg = _("Unbalanced parenthesis");
                    return array("error", $msg);
                }
                break;
            case "ttl":
                if ($val === null) {
                    return true;
                }
                if (strlen($val) > 30) {
                    $msg = _("Too much chars");
                    return array("warning", $msg);
                }
                if ($val !== "" && $verify->is_integer($val) !== true) {
                    $msg = _("Not numeric");
                    return array("error", $msg);
                }
                if ($val !== "" && $val < 0) {
                    $msg = _("Not negative value");
                    return array("error", $msg);
                }
                if ($val !== "" && $val > 2147483647) {
                    $msg = _("Value too high");
                    return array("error", $msg);
                }

                break;
            case "comment":
                if (strlen($val) > 1024) {
                    $msg = _("Too much chars");
                    return array("warning", $msg);
                }
                break;
            case "opendate":
                if ($verify->is_datetimeSQL($val) !== true) {
                    $msg = _("Not a valid date");
                    return array("error", $msg);
                }
                break;
            case "closedate":
                if ($val !== "" && $verify->is_datetimeSQL($val) !== true) {
                    $msg = _("Not a valid date");
                    return array("error", $msg);
                }
                break;
            default:
                $msg = _("Unknown field");
                return array("error", "$msg: $field");
        }

        return array();
    }

    /** Verify all the fields consistency and return an error array */
    public function verifyAll($data)
    {
        $errors = array();
        $verify = new Verify();
        if (! isset($data["rr"])) {
            throw new \Exception(_("No RR field provided in the record"), 500);
        }
        switch ($data["rr"]) {
            case "A":
                if (
                    substr($data["source"], -1 * strlen($data["zone"])) ===
                    $data["zone"]
                ) {
                    $msg = _("Don't add the domain in the source");
                    $errors["source"] = array("error", $msg);
                }
                if (
                    substr($data["source"], 1 - 1 * strlen($data["zone"])) ===
                    substr($data["zone"], 0, -1)
                ) {
                    $msg = _("Don't add the domain in the source (and without final dot)");
                    $errors["source"] = array("error", $msg);
                }
                if (strpos($data["source"], "_") !== false) {
                    $msg = _("Underscore is forbidden in source");
                    $errors["source"] = array("error", $msg);
                }
                if ($verify->is_ipv4($data["target"]) !== true) {
                    $msg = _("Not a valid IPv4 address");
                    $errors["target"] = array("error", $msg);
                }
                break;
            case "AAAA":
                if (
                    substr($data["source"], -1 * strlen($data["zone"])) ===
                    $data["zone"]
                ) {
                    $msg = _("Don't add the domain in the source");
                    $errors["source"] = array("error", $msg);
                }
                if (
                    substr($data["source"], 1 - 1 * strlen($data["zone"])) ===
                    substr($data["zone"], 0, -1)
                ) {
                    $msg = _("Don't add the domain in the source (and without final dot)");
                    $errors["source"] = array("error", $msg);
                }
                if (strpos($data["source"], "_") !== false) {
                    $msg = _("Underscore is forbidden in source");
                    $errors["source"] = array("error", $msg);
                }
                if ($verify->is_ipv6($data["target"]) !== true) {
                    $msg = _("Not a valid IPv6 address");
                    $errors["target"] = array("error", $msg);
                }
                break;
            case "CAA":
                if (
                    substr($data["source"], -1 * strlen($data["zone"])) ===
                    $data["zone"]
                ) {
                    $msg = _("Don't add the domain in the source");
                    $errors["source"] = array("error", $msg);
                }
                if (
                    $data["addvalues"] !== "0 issue" &&
                    $data["addvalues"] !== "0 issuedef" &&
                    $data["addvalues"] !== "0 iodef"
                ) {
                    $msg = _("Must be \"0 issue\" or \"0 issuedef\" or \"0 iodef\", " .
                    "without the double-quotes");
                    $errors["addvalues"] = array("error", $msg);
                }
                if (
                    substr($data["target"], -1) !== "\"" ||
                    substr($data["target"], 0, 1) !== "\""
                ) {
                    $msg = _("Must start and finish by double-quotes");
                    $errors["target"] = array("error", $msg);
                }
                break;
            case "CNAME":
                if ($verify->is_FQDN(substr($data["target"], 0, -1)) !== true) {
                    $msg = _("Not a valid FQDN");
                    $errors["target"] = array("error", $msg);
                } elseif (substr($data["target"], -1) !== ".") {
                    $msg = _("Missing end dot");
                    $errors["target"] = array("error", $msg);
                }
                if (strpos($data["source"], "_") !== false) {
                    $msg = _("Underscore is forbidden in source");
                    $errors["source"] = array("error", $msg);
                }
                break;
            case "MX":
                if (strlen($data["addvalues"]) < 1) {
                    $msg = _("Not enough chars");
                    $errors["addvalues"] = array("error", $msg);
                }
                if (strpos($data["source"], "_") !== false) {
                    $msg = _("Underscore is forbidden in source");
                    $errors["source"] = array("error", $msg);
                }
                if (!$verify->is_hostname($data["target"])) {
                    // The target can be a FQDN or a Hostname
                    if (substr($data["target"], -1) !== ".") {
                        $msg = _("Missing end dot");
                        $errors["target"] = array("error", $msg);
                    } elseif ($verify->is_FQDN(substr($data["target"], 0, -1)) !== true) {
                        $msg = _("Not a valid FQDN");
                        $errors["target"] = array("error", $msg);
                    }
                    if (strpos($data["target"], "_") !== false) {
                        $msg = _("Underscore is forbidden in target");
                        $errors["target"] = array("error", $msg);
                    }
                }
                break;
            case "NS":
                if (strpos($data["source"], "_") !== false) {
                    $msg = _("Underscore is forbidden in source");
                    $errors["source"] = array("error", $msg);
                }
                if ($verify->is_FQDN(substr($data["target"], 0, -1)) !== true) {
                    $msg = _("Not a valid FQDN");
                    $errors["target"] = array("error", $msg);
                } elseif (substr($data["target"], -1) !== ".") {
                    $msg = _("Missing end dot");
                    $errors["target"] = array("error", $msg);
                }
                break;
            case "PTR":
                $source = $data["source"] . "." . $data["zone"];
                $pos = strpos($source, ".in-addr.arpa.");
                if ($pos !== false) {
                    $revip = substr($source, 0, $pos);
                    $ip = implode(".", array_reverse(explode(".", $revip)));
                    if ($verify->is_ipv4($ip) !== true) {
                        $msg = _("Not a valid IPv4 with zone depth");
                        $errors["source"] = array("error", $msg);
                    }
                } elseif (($pos = strpos($source, ".ip6.arpa.")) !== false) {
                    $revip = substr($source, 0, $pos);
                    $ip = str_replace(".", "", $revip);
                    $ip = implode("", array_reverse(str_split($ip)));
                    $ip = implode(":", str_split($ip, 4));
                    if ($verify->is_ipv6($ip) !== true || strlen($ip) !== 39) {
                        $msg = _("Not a valid IPv6 with zone depth");
                        $errors["source"] = array("error", $msg);
                    }
                } else {
                    $msg = _("An IP in source is not allowed in direct zones");
                    $errors["source"] = array("error", $msg);
                }
                if (substr($data["target"], -1) !== ".") {
                    $msg = _("Missing end dot");
                    $errors["target"] = array("error", $msg);
                } elseif ($verify->is_FQDN(substr($data["target"], 0, -1)) !== true) {
                    $msg = _("Not a valid FQDN");
                    $errors["target"] = array("error", $msg);
                }
                if (strpos($data["target"], "_") !== false) {
                    $msg = _("Underscore is forbidden in target");
                    $errors["target"] = array("error", $msg);
                }
                break;
            case "SOA":
                if ($data["source"] !== "@") {
                    $msg = _("In SOA, only @ is allowed");
                    $errors["source"] = array("error", $msg);
                }
                $soarow = $this->read(array(array("zoneid", $data["zoneid"]),
                             array("rr", "SOA")));
                if (count($soarow) > 1) {
                    $msg = _("More than one SOA defined in the zone");
                    $errors["source"] = array("error", $msg);
                } elseif (isset($soarow[0]) && $soarow[0]["id"] !== $data["id"]) {
                    $msg = _("You can't define a second SOA in the zone");
                    $errors["source"] = array("error", $msg);
                }
                break;
            case "SSHFP":
                if (
                    substr($data["source"], -1 * strlen($data["zone"])) ===
                    $data["zone"]
                ) {
                    $msg = _("Don't add the domain in the source");
                    $errors["source"] = array("error", $msg);
                }
                if (
                    $data["addvalues"] !== "1 1" &&
                    $data["addvalues"] !== "1 2" &&
                    $data["addvalues"] !== "2 1" &&
                    $data["addvalues"] !== "2 2" &&
                    $data["addvalues"] !== "3 1" &&
                    $data["addvalues"] !== "3 2" &&
                    $data["addvalues"] !== "4 1" &&
                    $data["addvalues"] !== "4 2"
                ) {
                    $msg = _("Must be [1234] [12] format");
                    $errors["addvalues"] = array("error", $msg);
                }
                if (
                    strspn($data["target"], "0123456789abcdef") !==
                    strlen($data["target"])
                ) {
                    $msg = _("Must only be hexadecimal string");
                    $errors["target"] = array("error", $msg);
                }
                break;
            case "SPF":
                break;
            case "SRV":
              // _xmpp-server._tcp SRV    5   0 5269  xmpp.fournier38.fr.
                break;
            case "TXT":
                if (
                    strpos($data["target"], ";") !== false &&
                    substr($data["target"], 0, 1) !== '"' &&
                    substr($data["target"], 0, 1) !== '('
                ) {
                    $msg = _("You can't add destination with semi-colon, without " .
                    "encapsulate it in double-quotes or parenthesis");
                    $errors["target"] = array("error", $msg);
                }
                break;
        }

        return $errors;
    }

    /** Add to the $record array the disable state. The $record must coming from
        record::read */
    public function addDisable($records)
    {
        $currentDate = date("Y-m-d H:i:s");
        foreach ($records as $k => $val) {
            if (
                ! array_key_exists("opendate", $val) ||
                ! array_key_exists("closedate", $val)
            ) {
                continue;
            }
            if (
                $val["opendate"] !== "" && $val["closedate"] !== "" &&
                $val["opendate"] !== null && $val["closedate"] !== null &&
                $val["closedate"] < $val["opendate"] &&
                $currentDate > $val["opendate"]
            ) {
                $records[$k]["disabled"] = false;
            } elseif (
                $val["opendate"] !== "" && $val["closedate"] !== "" &&
                $val["opendate"] !== null && $val["closedate"] !== null &&
                $val["closedate"] < $val["opendate"] &&
                $currentDate < $val["closedate"]
            ) {
                $records[$k]["disabled"] = false;
            } elseif (
                $val["opendate"] !== "" &&
                $val["opendate"] !== null &&
                $currentDate < $val["opendate"]
            ) {
                $records[$k]["disabled"] = true;
            } elseif (
                $val["closedate"] !== "" &&
                $val["closedate"] !== null &&
                $currentDate > $val["closedate"]
            ) {
                $records[$k]["disabled"] = true;
            } else {
                $records[$k]["disabled"] = false;
            }
        }
        return $records;
    }
}
