<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Models;

use Domframework\Logger;

/** Allow all the SSH transmissions */
class Ssh
{
    /** The username used to login into the server */
    public $username = false;
    /** The password used to login into the server in password mode */
    public $password = false;
    /** The allowed methods to connect to a server
        To use DSA hostkey, you need to override $methods */
    public $methods = null;
    /*array (
          'kex' => 'diffie-hellman-group1-sha1',
          'client_to_server' => array(
            'crypt' => '3des-cbc',
            'comp' => 'none'),
          'server_to_client' => array(
            'crypt' => 'aes256-cbc,aes192-cbc,aes128-cbc',
            'comp' => 'none'),
          'hostkey'=>'ssh-rsa');*/
    /** Fingerprint of the server
        If FALSE, not verified */
    public $fingerprint = false;
    /** PublicKey in authkey mode
        If FALSE, not verified */
    public $publickeyfile = false;
    /** PrivateKey in authkey mode
        If FALSE, not verified */
    public $privatekeyfile = false;
    /** Passphrase needed to unlock the private key (if protected) */
    public $passphrase = "";
    /** The connection identifier */
    private $connection = false;
    /** The server name */
    private $server = null;

    /** Create a SSH connection identifier if the SSH support is available in PHP
      */
    public function __construct()
    {
        if (!function_exists("ssh2_connect")) {
            throw new \Exception(
                _("PHP without SSH support. Add php-ssh2 package"),
                500
            );
        }
    }

    /*  private function my_ssh_disconnect ($reason, $message, $language)
      {
        throw new \Exception(sprintf (
                     _("Server disconnected with reason code [%d] and message: %s"),
                     $reason, $message), 500);
      }*/

    /** Connect to a distant server
        @param string $server Name or IP of the server to connect
        @param integer $port TCP Port to use (22 by default) */
    public function connect($server, $port = 22)
    {
        // The ssh2_connect timeout is forced to 60s. I test the server with a
        // connect test limit to 5s, in case of servers failure
        if (! is_string($server)) {
            throw new \Exception(_("Server provided is not a string"), 405);
        }
        if (! is_int($port)) {
            throw new \Exception(_("Server port provided is not an integer"), 405);
        }
        $logger = new Logger();
        $this->connection = @ssh2_connect($server, $port, $this->methods);
        if (!$this->connection) {
            throw new \Exception(sprintf(
                _("Error when connecting to server %s"),
                $server
            ), 500);
        }

        $this->server = $server;

        // Fingerprint verification
        $fingerprint = @ssh2_fingerprint(
            $this->connection,
            SSH2_FINGERPRINT_SHA1 | SSH2_FINGERPRINT_HEX
        );
        if ($this->fingerprint !== false && $fingerprint !== $this->fingerprint) {
            throw new \Exception(sprintf(
                _("Unable to verify server identity ! %s"),
                $fingerprint
            ), 500);
        }

        // Connect by shared key
        if ($this->publickeyfile !== false && $this->privatekeyfile !== false) {
            if (
                @ssh2_auth_pubkey_file(
                    $this->connection,
                    $this->username,
                    $this->publickeyfile,
                    $this->privatekeyfile,
                    $this->passphrase
                )
            ) {
                return true;
            }
            $logger->log(LOG_ERR, "Wrong SSH private/public key provided");
        }

        // Connect by login/password
        if ($this->username !== false && $this->password !== false) {
            if (
                @ssh2_auth_password(
                    $this->connection,
                    $this->username,
                    $this->password
                )
            ) {
                return true;
            }
            $logger->log(LOG_ERR, "Wrong SSH password for user '$this->username'");
        }

        throw new \Exception(_("No connection method available"), 500);
    }

    /** Execute a command on a distant server
        Return an array with stdout and stderr data
        Don't forget to escape parameters !
        @param string $command The command to be run on the distant server
        @return array Contain a stdout and a stderr keys with the result of
                      the command */
    public function exec($command)
    {
        if (! is_string($command)) {
            throw new \Exception(_("Command provided is not a string"), 405);
        }
        if ($this->connection === false) {
            throw new \Exception(
                _("No opened SSH connection available to exec"),
                500
            );
        }
        $stdout_stream = @ssh2_exec($this->connection, $command);
        $err_stream = @ssh2_fetch_stream($stdout_stream, SSH2_STREAM_STDERR);
        $dio_stream = @ssh2_fetch_stream($stdout_stream, SSH2_STREAM_STDIO);
        stream_set_blocking($err_stream, true);
        stream_set_blocking($dio_stream, true);
        $result_err = stream_get_contents($err_stream);
        $result_dio = stream_get_contents($dio_stream);
        return array("stdout" => trim($result_dio), "stderr" => trim($result_err));
    }

    /** Send a local file to the previous connected ssh server by SCP
        @param string $localfile Localfile to be sent
        @param string $distantfile Distant file to be written
        @return TRUE */
    public function scpSend($localfile, $distantfile)
    {
        if (! is_string($localfile)) {
            throw new \Exception(_("Localfile provided is not a string"), 405);
        }
        if (! is_string($distantfile)) {
            throw new \Exception(_("Distantfile provided is not a string"), 405);
        }
        if ($this->connection === false) {
            throw new \Exception(
                _("No opened SSH connection available to exec"),
                500
            );
        }
        if (! @ssh2_scp_send($this->connection, $localfile, $distantfile)) {
            throw new \Exception(sprintf(
                _("Can't send file '%s' to server '%s' in '%s'"),
                $localfile,
                $this->server,
                $distantfile
            ), 500);
        }
        return true;
    }

    /** Receive a file from the previous connected ssh server by SCP
        @param string $distantfile Distant file to be written
        @param string $localfile Localfile to be sent
        @return TRUE */
    public function scpRecv($distantfile, $localfile)
    {
        if (! is_string($distantfile)) {
            throw new \Exception(_("Distantfile provided is not a string"), 405);
        }
        if (! is_string($localfile)) {
            throw new \Exception(_("Localfile provided is not a string"), 405);
        }
        if ($this->connection === false) {
            throw new \Exception(
                _("No opened SSH connection available to exec"),
                500
            );
        }
        if (! @ssh2_scp_recv($this->connection, $distantfile, $localfile)) {
            throw new \Exception(sprintf(
                _("Can't receive file '%s' from server '%s' in '%s'"),
                $distantfile,
                $this->server,
                $localfile
            ), 500);
        }
        return true;
    }

    /** Disconnect an existing connection */
    public function disconnect()
    {
        $this->exec('echo "EXITING" && exit;');
        $this->connection = null;
    }
}
