<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Models;

use Domframework\Dblayer;
use Domframework\Verify;

/** All the methods to manage a zone */
class Zone extends Dblayer
{
    /* The database must be initialized with
       CREATE TABLE `dns_zones` (id INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
                                 zone VARCHAR(255) NOT NULL,
                                 viewname VARCHAR(255) NOT NULL DEFAULT 'default',
                                 viewclients VARCHAR(255),
                                 comment VARCHAR(1024),
                                 opendate DATETIME NOT NULL,
                                 closedate DATETIME,
                                 UNIQUE (zone,viewname)); */
    /** The name of the table in the database */
    public $table = "dns_zones";
    /** The prefix defined in the configuration */
    public $tableprefix = "";

    /** The fields in the table with their specifications */
    public $fields = array(
    "id" => array("integer", "not null", "autoincrement"),
    "zone" => array("varchar", "255", "not null"),
    "viewname" => array("varchar", "255"),
    "viewclients" => array("varchar", "255"),
    "dnssec" => array("integer"),
    "comment" => array("varchar", "1024"),
    "opendate" => array("datetime", "not null"),
    "closedate" => array("datetime"),
    );
    /** The primary key of the table */
    public $primary = "id";
    /** The unique constrain of the table */
    public $unique = array(
    "id",
    array("zone", "viewname"),
    array("zone", "viewclients"));

    /** Translation */
    public function titles()
    {
        return  array(
        "id" => gettext("ID"),
        "zone" => _("Zone"),
        "viewname" => _("View"),
        "viewclients" => _("View clients"),
        "dnssec" => _("DNSSec support"),
        "comment" => _("Comments"),
        "opendate" => _("Open date"),
        "closedate" => _("Close date"),
        );
    }

    public function verifyOne($field, $val)
    {
        $verify = new Verify();
        switch ($field) {
            case "id":
                break;
            case "zone":
                if (strlen($val) < 1) {
                    $msg = _("Not enough chars");
                    return array("error", $msg);
                }
                if (strlen($val) > 255) {
                    $msg = _("Too much chars");
                    return array("error", $msg);
                }
                if (substr($val, -1) !== ".") {
                    $msg = _("Missing ending dot");
                    return array("error", $msg);
                }
                if (strtolower($val) !== $val) {
                    $msg = _("Only lowercase characters are allowed");
                    return array("error", $msg);
                }
                if (
                    strspn($val, "abcdefghijklmnopqrstuvwxyz" .
                        "01234567489-.") !== strlen($val)
                ) {
                    $msg = _("Only letters, digits, dash and dot are allowed");
                    return array("error", $msg);
                }
                break;
            case "viewname":
                if (strlen($val) < 1) {
                    $msg = _("Not enough chars");
                    return array("error", $msg);
                }
                if (strlen($val) > 255) {
                    $msg = _("Too much chars");
                    return array("error", $msg);
                }
                if (
                    strspn($val, "ABCDEFGHIJKLMNOPQRSTUVWXYZ" .
                        "abcdefghijklmnopqrstuvwxyz" .
                        "01234567489-.") !== strlen($val)
                ) {
                    $msg = _("Only letters, digits, dash and dot are allowed");
                    return array("error", $msg);
                }
                break;
            case "viewclients":
                if (strlen($val) > 255) {
                    $msg = _("Too much chars");
                    return array("error", $msg);
                }
                break;
            case "dnssec":
                if ($val !== "0" && $val !== "1") {
                    $msg = _("Must be 0 or 1");
                    return array("error", $msg);
                }
                // no break
            case "comment":
                if (strlen($val) > 1024) {
                    $msg = _("Too much chars");
                    return array("error", $msg);
                }
                break;
            case "opendate":
                if ($verify->is_datetimeSQL($val) !== true) {
                    $msg = _("Not a valid date");
                    return array("error", $msg);
                }
                break;
            case "closedate":
                if ($val !== "" && $verify->is_datetimeSQL($val) !== true) {
                    $msg = _("Not a valid date");
                    return array("error", $msg);
                }
                break;
            default:
                $msg = _("Unknown field");
                return array("error", "$msg: $field");
        }

        return array();
    }

    /** Add to the $zones array the disable state. The $zones must coming from
        zone::read */
    public function addDisable($zones)
    {
        $currentDate = date("Y-m-d H:i:s");
        foreach ($zones as $k => $val) {
            if (
                ! array_key_exists("opendate", $val) ||
                ! array_key_exists("closedate", $val)
            ) {
                continue;
            }
            if (
                $val["opendate"] !== "" && $val["closedate"] !== "" &&
                $val["opendate"] !== null && $val["closedate"] !== null &&
                $val["closedate"] < $val["opendate"] &&
                $currentDate > $val["opendate"]
            ) {
                $zones[$k]["disabled"] = false;
            } elseif (
                $val["opendate"] !== "" && $val["closedate"] !== "" &&
                $val["opendate"] !== null && $val["closedate"] !== null &&
                $val["closedate"] < $val["opendate"] &&
                $currentDate < $val["closedate"]
            ) {
                $zones[$k]["disabled"] = false;
            } elseif (
                $val["opendate"] !== "" &&
                $val["opendate"] !== null &&
                $currentDate < $val["opendate"]
            ) {
                $zones[$k]["disabled"] = true;
            } elseif (
                $val["closedate"] !== "" &&
                $val["closedate"] !== null &&
                $currentDate > $val["closedate"]
            ) {
                $zones[$k]["disabled"] = true;
            } else {
                $zones[$k]["disabled"] = false;
            }
        }
        return $zones;
    }

    /** Verify all the fields consistency and return an error array */
  /*public function verify ($data, $updatekey=false)
  {
    return parent::verify ($data, $updatekey);
  }*/
}
