<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Modules\Authorization\Controllers;

use Domframework\Authzgroups;
use Models\Configuration;

/** All the groupmembers management for a group of users */
class GroupMembers
{
    /** The object authzgroups */
    private $authzgroups;

    /** The anonymous user */
    private $auth = array("email" => "anonymous");

    /** The administrators list */
    private $administrators = array();

    /** Check if all the configuration is available before using it */
    public function __construct($auth)
    {
        // Read the configuration and populate local variables
        $configuration = new Configuration();
        $dbconfig = $configuration->get("database");
        if (!isset($dbconfig["tableprefix"])) {
            $dbconfig["tableprefix"] = "";
        }
        $this->authzgroups = new Authzgroups();
        $this->authzgroups->connect(
            $dbconfig["dsn"],
            $dbconfig["username"],
            $dbconfig["password"],
            $dbconfig["driver_options"]
        );
        $authconfig = $configuration->get("authorization");
        if (! isset($authconfig["administrators"])) {
            $authconfig["administrators"] = array();
        }
        if (is_string($authconfig["administrators"])) {
            $authconfig["administrators"] = explode(
                "|",
                $authconfig["administrators"]
            );
        }
        $this->administrators = $authconfig["administrators"];
        if (
            ! is_array($auth) ||
            ! array_key_exists("email", $auth)
        ) {
            throw new \Exception(
                _("Invalid auth provided to 'records' class"),
                403
            );
        }
        $this->auth = $auth;
    }

    /** Add a new group in the database of authzgroups */
    public function add($group, $user, $comment = "")
    {
        return $this->authzgroups->groupmemberAdd(
            "domdns",
            $group,
            $user,
            $comment
        );
    }

    /** Remove an group from database and all the groups using it */
    public function del($group, $user)
    {
        die("TBTESTED : " . __FILE__ . ":" . __LINE__ . "\n");
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupmemberDel("domdns", $group, $user);
    }

    /** Remove an group from database and all the groups using it */
    public function delByID($idgroup, $iduser)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupmemberDelByID("domdns", $idgroup, $iduser);
    }

    /** Update an user in the database */
    public function updateByID($idgroup, $idgroupmember, $newuser, $comment = "")
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupmemberUpdateByID(
            "domdns",
            $idgroup,
            $idgroupmember,
            $newuser,
            $comment
        );
    }

    /** Return an array with all the groups where the user is in and in the module
      */
    public function readuser($user)
    {
        die("TBTESTED : " . __FILE__ . ":" . __LINE__ . "\n");
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupmemberReadUser("domdns", $user);
    }

    /** Return an array with all information of a user in a specific group */
    public function readuserDataByID($idgroup, $iduser)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupmemberReadUserDataByID(
            "domdns",
            $idgroup,
            $iduser
        );
    }

    /** Return an array with all the groups where the user is in and in the module
      */
    public function readuserByID($idgroupmember)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }

        return $this->authzgroups->groupmemberReadUserByID("domdns", $idgroupmember);
    }

    /** Return an array with all the available users in the group and in the
        module */
    public function readgroup($group)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupmemberReadGroup("domdns", $group);
    }

    /** Return an array with all the available users in the group and in the
        module */
    public function readgroupByID($idgroup)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupmemberReadGroupByID("domdns", $idgroup);
    }

    /** Return an array containing the titles of the table translating in the user
        language */
    public function titles()
    {
        return $this->authzgroups->groupmembersTitles();
    }

    /** Check if the provided data are compilant with the group specification
        @return array The errors found in the data */
    public function verify($data, $idgroupmember = false)
    {
        return $this->authzgroups->groupmembersVerify($data, $idgroupmember);
    }
}
