<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Modules\Authorization\Controllers;

use Domframework\Authzgroups;
use Models\Configuration;

/** All the groups management for a group of users */
class Groups
{
    /** The object authzgroups */
    private $authzgroups;

    /** The anonymous user */
    private $auth = array("email" => "anonymous");

    /** The administrators list */
    private $administrators = array();

    /** Check if all the configuration is available before using it */
    public function __construct($auth)
    {
        // Read the configuration and populate local variables
        $configuration = new Configuration();
        $dbconfig = $configuration->get("database");
        if (!isset($dbconfig["tableprefix"])) {
            $dbconfig["tableprefix"] = "";
        }
        $this->authzgroups = new Authzgroups();
        $this->authzgroups->connect(
            $dbconfig["dsn"],
            $dbconfig["username"],
            $dbconfig["password"],
            $dbconfig["driver_options"]
        );
        $authconfig = $configuration->get("authorization");
        if (! isset($authconfig["administrators"])) {
            $authconfig["administrators"] = array();
        }
        if (is_string($authconfig["administrators"])) {
            $authconfig["administrators"] = explode(
                "|",
                $authconfig["administrators"]
            );
        }
        $this->administrators = $authconfig["administrators"];
        if (
            ! is_array($auth) ||
            ! array_key_exists("email", $auth)
        ) {
            throw new \Exception(
                _("Invalid auth provided to 'records' class"),
                403
            );
        }
        $this->auth = $auth;
    }

    /** Add a new group in the database of authzgroups */
    public function add($group, $comment = "")
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupAdd("domdns", $group, $comment);
    }

    /** Remove an group from database and all the user associations using it */
    public function del($group)
    {
        die("TBTESTED : " . __FILE__ . ":" . __LINE__ . "\n");
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupDel("domdns", $group);
    }

    public function delByID($idgroup)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupDelByID("domdns", $idgroup);
    }

    /** Update an group in the database */
    public function update($group, $newgroup, $comment = "")
    {
        die("TBTESTED : " . __FILE__ . ":" . __LINE__ . "\n");
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupUpdate(
            "domdns",
            $group,
            $newgroup,
            $comment
        );
    }

    /** Update an group in the database */
    public function updateByID($idgroup, $newgroup, $comment = "")
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupUpdateByID(
            "domdns",
            $idgroup,
            $newgroup,
            $comment
        );
    }

    /** Return an array with all the available groups in the module, or the
        group if provided */
    public function read($group = null)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupRead("domdns", $group);
    }

    /** Return an array with all the available groups in the module, or the
        group if provided */
    public function readByID($idgroup = null)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->groupReadByID("domdns", $idgroup);
    }

    /** Return an array containing the titles of the table translating in the user
        language */
    public function titles()
    {
        return $this->authzgroups->groupTitles();
    }

    /** Check if the provided data are compilant with the group specification
        @return array The errors found in the data */
    public function verify($data, $idgroup = false)
    {
        return $this->authzgroups->groupVerify($data, $idgroup);
    }
}
