<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Modules\Authorization\Controllers;

use Domframework\Authzgroups;
use Models\Configuration;

/** All the objects management for a object of users */
class Objects
{
    /** The object authzgroups */
    private $authzgroups;

    /** The anonymous user */
    private $auth = array("email" => "anonymous");

    /** The administrators list */
    private $administrators = array();

    /** Check if all the configuration is available before using it */
    public function __construct($auth)
    {
        // Read the configuration and populate local variables
        $configuration = new Configuration();
        $dbconfig = $configuration->get("database");
        if (!isset($dbconfig["tableprefix"])) {
            $dbconfig["tableprefix"] = "";
        }
        $this->authzgroups = new Authzgroups();
        $this->authzgroups->connect(
            $dbconfig["dsn"],
            $dbconfig["username"],
            $dbconfig["password"],
            $dbconfig["driver_options"]
        );
        $authconfig = $configuration->get("authorization");
        if (! isset($authconfig["administrators"])) {
            $authconfig["administrators"] = array();
        }
        if (is_string($authconfig["administrators"])) {
            $authconfig["administrators"] = explode(
                "|",
                $authconfig["administrators"]
            );
        }
        $this->administrators = $authconfig["administrators"];
        if (
            ! is_array($auth) ||
            ! array_key_exists("email", $auth)
        ) {
            throw new \Exception(
                _("Invalid auth provided to 'records' class"),
                403
            );
        }
        $this->auth = $auth;
    }

    /** Add a new object in the database of authzgroups */
    public function add($object, $comment = "")
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->objectAdd("domdns", $object, $comment);
    }

    /** Remove an object from database and all the objects using it */
    public function del($object)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->objectDel("domdns", $object);
    }

    /** Remove an object from database and all the objects using it */
    public function delByID($idobject)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->objectDelByID("domdns", $idobject);
    }

    /** Update an object in the database */
    public function update($object, $newobject, $newcomment = "")
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->objectUpdate(
            "domdns",
            $object,
            $newobject,
            $newcomment
        );
    }

    /** Update an object in the database */
    public function updateByID($idobject, $newobject, $newcomment = "")
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->objectUpdateByID(
            "domdns",
            $idobject,
            $newobject,
            $newcomment
        );
    }

    /** Return an array with all the available objects in the module, or the
        object if provided */
    public function read($object = null)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->objectRead("domdns", $object);
    }

    /** Return an array with all the available objects in the module, or the
        object if provided */
    public function readByID($idobject = null)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->objectReadByID("domdns", $idobject);
    }

    /** Return an array containing the titles of the table translating in the user
        language */
    public function titles()
    {
        return $this->authzgroups->objectTitles();
    }

    /** Check if the provided data are compilant with the object specification
        @return array The errors found in the data */
    public function verify($data, $idobject = false)
    {
        return $this->authzgroups->objectVerify($data, $idobject);
    }
}
