<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

namespace Modules\Authorization\Controllers;

use Domframework\Authzgroups;
use Models\Configuration;

/** All the rights management for a group of users */
class Rights
{
    /** The object authzgroups */
    private $authzgroups;

    /** The anonymous user */
    private $auth = array("email" => "anonymous");

    /** The administrators list */
    private $administrators = array();

    /** Check if all the configuration is available before using it */
    public function __construct($auth)
    {
        // Read the configuration and populate local variables
        $configuration = new Configuration();
        $dbconfig = $configuration->get("database");
        if (!isset($dbconfig["tableprefix"])) {
            $dbconfig["tableprefix"] = "";
        }
        $this->authzgroups = new Authzgroups();
        $this->authzgroups->connect(
            $dbconfig["dsn"],
            $dbconfig["username"],
            $dbconfig["password"],
            $dbconfig["driver_options"]
        );
        $authconfig = $configuration->get("authorization");
        if (! isset($authconfig["administrators"])) {
            $authconfig["administrators"] = array();
        }
        if (is_string($authconfig["administrators"])) {
            $authconfig["administrators"] = explode(
                "|",
                $authconfig["administrators"]
            );
        }
        $this->administrators = $authconfig["administrators"];
        if (
            ! is_array($auth) ||
            ! array_key_exists("email", $auth)
        ) {
            throw new \Exception(
                _("Invalid auth provided to 'records' class"),
                403
            );
        }
        $this->auth = $auth;
    }

    /** Add a new right in the database of authzgroups */
    public function add($group, $object, $right, $comment = "")
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightAdd(
            "domdns",
            $group,
            $object,
            $right,
            $comment
        );
    }

    /** Add a new right in the database of authzgroups */
    public function addByID($idgroup, $idobject, $right, $comment = "")
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightAddByID(
            "domdns",
            $idgroup,
            $idobject,
            $right,
            $comment
        );
    }

    /** Remove an right from database and all the user associations using it */
    public function del($right)
    {
        die("TBTESTED : " . __FILE__ . ":" . __LINE__ . "\n");
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightDel("domdns", $right);
    }

    public function delByID($idright)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightDelByID("domdns", $idright);
    }

    /** Update an right in the database */
    public function update($right, $newright, $comment = "")
    {
        die("TBTESTED : " . __FILE__ . ":" . __LINE__ . "\n");
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightUpdate(
            "domdns",
            $right,
            $newright,
            $comment
        );
    }

    /** Update an right in the database */
    public function updateByID($idright, $newidobject, $newright, $comment = "")
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightUpdateByID(
            "domdns",
            $idright,
            $newidobject,
            $newright,
            $comment
        );
    }

    /** Return an array with all the available rights in the module, or the
        right if provided */
    public function read($right = null)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightRead("domdns", $right);
    }

    /** Return an array with all the available rights in the module, or the
        right if provided */
    public function readByID($idright = null)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightReadByID("domdns", $idright);
    }

    /** Return an array with all the defined rights for the group provided */
    public function readByGroupByID($idgroup)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightReadByGroupByID("domdns", $idgroup);
    }

    /** Return an array with all the defined rights for the object provided */
    public function readByObjectByID($idobject)
    {
        if (! in_array($this->auth["email"], $this->administrators)) {
            if ($this->auth["email"] === "anonymous") {
                throw new \Exception(_("Anonymous not allowed"), 401);
            }
            throw new \Exception(_("Access forbidden"), 403);
        }
        return $this->authzgroups->rightReadByObjectByID("domdns", $idobject);
    }

    /** Return an array containing the titles of the table translating in the user
        language */
    public function titles()
    {
        return $this->authzgroups->rightTitles();
    }

    /** Check if the provided data are compilant with the right specification
        @return array The errors found in the data */
    public function verify($data, $idright = false)
    {
        return $this->authzgroups->rightVerify($data, $idright);
    }

    /** Return all the types of rights available (RO and RW) */
    public function types()
    {
        return $this->authzgroups->rightTypes();
    }
}
