<?php

/**
 * DomDNS : a DNS manager
 * @package domdns
 * @author Dominique Fournier <dominique@fournier38.fr>
 */

use Domframework\Authentication;
use Domframework\Language;
use Domframework\Csrf;
use Domframework\Form;
use Domframework\Rest;
use Domframework\Renderer;
use Domframework\Route;
use Controllers\Records;
use Controllers\Servers;
use Controllers\Userprefs;
use Controllers\Zones;
use Models\Configuration;

/** The main HTML/REST router */
chdir(dirname(__FILE__));
require_once("vendor/autoload.php");
session_start();

if (! function_exists("mb_strlen")) {
    die("PHP must have the MBString support");
}


$modulesEnabled = array("Authorization","Configuration");


ini_set("html_errors", "off");
$configuration = new Configuration();
$timezone = $configuration->get("timezone");
ini_set("date.timezone", $timezone);

$route = new Route();
$route->debug = 0;

// Authentification
$authentication = new Authentication($route);
$authentication->appName = "DomDNS";
$configAuth = $configuration->get("authentication");
$authentication->htmlMethods = $configAuth["html"];
$authentication->restMethods = $configAuth["rest"];
$authentication->authMethods = $configAuth["methods"];
$authentication->ratelimitDir = $configAuth["ratelimitDir"] ?? "/tmp/ratelimit/";
$authentication->authServers = array(
  "authldap" => array($configAuth["authldap"] ?? []),
  "authhtpasswd" => array($configAuth["authhtpasswd"] ?? []),
);

// Selection of the language
$authHTML = $authentication->verifAuthHTML();
$userprefs = new Userprefs($authHTML);
$values = $userprefs->read("language");
$language = new Language();
if (count($values) === 0) {
    $lang = $language->activeLanguage("domdns", "DOMDNS-Language");
    $_SESSION["domdns"]["language"] = $lang;
} else {
    $languageCode = $values["value"];
    $lang = $language->activeLanguage(
        "domdns",
        "DOMDNS-Language",
        $languageCode
    );
    $_SESSION["domdns"]["language"] = $lang;
}

$modulesDisplayed = array();
$modulesDisplayed[] = "zone";
foreach ($modulesEnabled as $key => $module) {
    if (file_exists("Modules/$module/init.php")) {
        require("Modules/$module/init.php");
    }
}
foreach ($modulesEnabled as $key => $module) {
    if (file_exists("Modules/$module/index.php")) {
        require("Modules/$module/index.php");
    }
}

// The definitions of error managment must be set before calling
// $authentication->routes() to catch and display the autentication errors
$route->errors = null;
$route->layout = "Views/layout.html";
$route->replacement = array("{baseurl}" => $route->baseURL(),
                             "{baseurlresource}" => $route->baseURLresource());
$route->viewErrorClass = "\Views\Viewerror";
$route->viewErrorMethod = "html";
$authentication->routes();
/////////////////////////
////    REST PART    ////
/////////////////////////

/////   REST SERVERS PART   /////
$route->output = "rest";
//$route->errors = array ("viewerror", "rest");
/** Reload only this zone/view */
$route->get(
    "rest/servers/reload/{zone}/{viewname}",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authREST = $authentication->verifAuthREST();
        $servers = new Servers($authREST);
        return $servers->reload($zone, $viewname);
    }
)

/** Reload all the zones */
->get(
    "rest/servers/reload",
    function () use ($authentication, $route) {
        $authREST = $authentication->verifAuthREST();
        $servers = new Servers($authREST);
        return $servers->reload();
    }
)

/** Update the zone list files of the server */
->get(
    "rest/servers/zonelistconfiguration",
    function () use ($authentication, $route) {
        $authREST = $authentication->verifAuthREST();
        $servers = new Servers($authREST);
        return $servers->zonelistconfiguration();
    }
)

->get(
    "rest/servers/export/{zone}/{viewname}",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authREST = $authentication->verifAuthREST();
        $servers = new Servers($authREST);
        return $servers->zoneexport($zone, $viewname);
    }
)

/////   REST RECORDS PART   /////
// curl -X POST --data-binary @tests/grenoble.cnrs.fr
//   http://localhost/ProgSVN/domdns/rest/records/zonetest/view/importBind
->post(
    "rest/records/{zone}/{viewname}/importBind",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authREST = $authentication->verifAuthREST();
        $records = new Records($authREST);
        return $records->importBind(
            $zone,
            $viewname,
            file_get_contents("php://input")
        );
    }
)

->get(
    "rest/records/{zone}/{viewname}/{record}",
    function ($zone, $viewname, $record) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authREST = $authentication->verifAuthREST();
        $records = new Records($authREST);
        return $records->read($zone, $viewname, $record);
    }
)

->get(
    "rest/records/{zone}/{viewname}(\?{params})?",
    function ($zone, $viewname, $params) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authREST = $authentication->verifAuthREST();
        parse_str($params, $params);
        if (!isset($params["display"])) {
            $params["display"] = null;
        }
        if (!isset($params["order"])) {
            $params["order"] = null;
        }
        $records = new Records($authREST);
        return $records->read(
            $zone,
            $viewname,
            null,
            $params["display"],
            $params["order"]
        );
    }
)

->get(
    "rest/records/{zone}/{viewname}",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authREST = $authentication->verifAuthREST();
        $records = new Records($authREST);
        return $records->read($zone, $viewname);
    }
)

->put(
    "rest/records/{record}",
    function ($record) use ($authentication, $route) {
        $authREST = $authentication->verifAuthREST();
        parse_str(file_get_contents("php://input"), $data);
        $records = new Records($authREST);
        return $records->update($record, $data);
    }
)

->delete(
    "rest/recordsWithoutZoneUpdate/{record}",
    function ($record) use ($authentication, $route) {
        // Delete an existing record
        $authREST = $authentication->verifAuthREST();
        $records = new Records($authREST);
        return $records->delete($record, true);
    }
)
->delete(
    "rest/records/{record}",
    function ($record) use ($authentication, $route) {
        // Delete an existing record
        $authREST = $authentication->verifAuthREST();
        $records = new Records($authREST);
        return $records->delete($record);
    }
)

->post(
    "rest/records",
    function () use ($authentication, $route) {
        // Insert a new record
        $authREST = $authentication->verifAuthREST();
        $records = new Records($authREST);
        return $records->insert($_POST);
    }
)

/////    REST ZONES PART   /////
->get(
    "rest/zones",
    function () use ($authentication, $route) {
        $authREST = $authentication->verifAuthREST();
        $zones = new Zones($authREST);
        return $zones->read();
    }
)

->get(
    "rest/zones/{zone}",
    function ($zone) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        $authREST = $authentication->verifAuthREST();
        $zones = new Zones($authREST);
        $data = $zones->read($zone);
        if (count($data) === 0) {
            throw new \Exception(_("Zone not found"), 404);
        }
        return $data;
    }
)

->post(
    "rest/zones",
    function () use ($authentication, $route) {
        $authREST = $authentication->verifAuthREST();
        $zones = new Zones($authREST);
        return $zones->insert($_POST);
    }
)

->put(
    "rest/zones/{zone}/{viewname}",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authREST = $authentication->verifAuthREST();
        parse_str(file_get_contents("php://input"), $data);
        $zones = new Zones($authREST);
        return $zones->update($zone, $viewname, $data);
    }
)

->delete(
    "rest/zones/{zone}/{viewname}",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authREST = $authentication->verifAuthREST();
        $zones = new Zones($authREST);
        return $zones->delete($zone, $viewname);
    }
)

//    REST USERPREFS PART    //
->get(
    "rest/userprefs/{key}",
    function ($key) use ($authentication, $route) {
        $authREST = $authentication->verifAuthREST();
        $userprefs = new Userprefs($authREST);
        return $userprefs->read($key);
    }
)

->post(
    "rest/userprefs/{key}",
    function ($key) use ($authentication, $route) {
        $authREST = $authentication->verifAuthREST();
        parse_str(file_get_contents("php://input"), $data);
        $userprefs = new Userprefs($authREST);
        return $userprefs->insert($data);
    }
)

->put(
    "rest/userprefs/{key}",
    function ($key) use ($authentication, $route) {
        $authREST = $authentication->verifAuthREST();
        parse_str(file_get_contents("php://input"), $data);
        $userprefs = new Userprefs($authREST);
        return $userprefs->update($key, $data["value"]);
    }
)

->delete(
    "rest/userprefs/{key}",
    function ($key) use ($authentication, $route) {
        $authREST = $authentication->verifAuthREST();
        $userprefs = new Userprefs($authREST);
        return $userprefs->delete($key);
    }
)

/////   REST UNKNOWN BLOCKED WITH 404 ERROR   /////
->multi(
    "get,post,put,delete",
    "rest/{unused}",
    function () {
        $rest = new Rest();
        $rest->display(_("REST command not found"), 404);
    }
);

//   ==>   HTML PART   <==
$route->output = "html";
//$route->errors = array ("errors", "html");
$route->errors = null;
$route->authenticationURL = "/authentication/";
$route->layout = "Views/layout.html";
$route->replacement = array("{baseurl}" => $route->baseURL(),
                             "{baseurlresource}" => $route->baseURLresource());
$route->viewErrorClass = "\Views\Viewerror";
$route->viewErrorMethod = "html";

////     MAIN REDIRECTION    ////
$route->get("", function () use ($route) {
    $route->redirect("/zone", "");
})
;

/////////////////////////
////    ZONE PART    ////
/////////////////////////
$route->viewClass = 'Views\\Zones';
$route
->get("zone", function () use ($authentication, $route) {
    $authHTML = $authentication->verifAuthHTML();
    $route->viewMethod = "read";
    $zones = new Zones($authHTML);
    $editright = $zones->editright();
    $values = $zones->read(null, null, null, array(array("zone")));
    $titles = $zones->titles();
    $configuration = new Configuration();
    $viewSupport = $configuration->get("viewSupport");
    return array(
        "zones" => $values,
        "titles" => $titles,
        "editright" => $editright,
        "viewSupport" => $viewSupport
    );
})

//$route->get ("{zone}/{view}",
//  function ($zone, $view) use ($route) {$route->redirect ("$zone/$view/");});
->get(
    "zone/{zone}/{viewname}/servers/zoneexport",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        if ($readonly) {
            $renderer = new Renderer();
            $renderer->flash("ERROR", _("Read-only zone"));
            if ($zone === ".") {
                $zone = "dot";
            }
            if ($viewname === ".") {
                $viewname = "dot";
            }
            $route->redirect("/zone/$zone/$viewname", "");
        }
        $renderer = new Renderer();
        $servers = new Servers($authHTML);
        try {
            $rc = $servers->zoneexport($zone, $viewname);
        } catch (\Exception $e) {
            $renderer->flash("ERROR", $e->getMessage());
        }
        if ($zone === ".") {
            $zone = "dot";
        }
        if ($viewname === ".") {
            $viewname = "dot";
        }
        $route->redirect("/zone/$zone/$viewname", "");
    }
)

->get(
    "zone/{zone}/{viewname}/servers/reload",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        if ($readonly) {
            $renderer = new Renderer();
            $renderer->flash("ERROR", _("Read-only zone"));
            if ($zone === ".") {
                $zone = "dot";
            }
            if ($viewname === ".") {
                $viewname = "dot";
            }
            $route->redirect("/zone/$zone/$viewname", "");
        }
        $renderer = new Renderer();
        $servers = new Servers($authHTML);
        try {
            $rc = $servers->reload($zone, $viewname);
        } catch (\Exception $e) {
            $renderer->flash("ERROR", $e->getMessage());
        }
        if ($zone === ".") {
            $zone = "dot";
        }
        if ($viewname === ".") {
            $viewname = "dot";
        }
        $route->redirect("/zone/$zone/$viewname", "");
    }
)

->get(
    "zone/{zone}/{viewname}/edit",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $route->viewMethod = "update";
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        if ($readonly) {
            $renderer = new Renderer();
            $renderer->flash("ERROR", _("Read-only zone"));
            if ($zone === ".") {
                $zone = "dot";
            }
            if ($viewname === ".") {
                $viewname = "dot";
            }
            $route->redirect("/zone", "");
        }
        $values = $zones->read($zone, $viewname, null, array(array("zone")));
        if (count($values) === 0) {
            throw new \Exception(_("Zone not found with this view name"), 404);
        }
        $values = $values[0];
        $titles = $zones->titles();
        unset($values["right"]);
        unset($titles["right"]);
        $configuration = new Configuration();
        $viewSupport = $configuration->get("viewSupport");
        return array("zone" => $zone, "viewname" => $viewname,
                    "values" => $values, "titles" => $titles,
                    "viewSupport" => $viewSupport);
    }
)

// Save the parameters if they are corrects and then redisplay the page
->post(
    "zone/{zone}/{viewname}/edit",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        if ($readonly) {
            $renderer = new Renderer();
            $renderer->flash("ERROR", _("Read-only zone"));
            if ($zone === ".") {
                $zone = "dot";
            }
            if ($viewname === ".") {
                $viewname = "dot";
            }
            $route->redirect("/zone/$zone/$viewname", "");
        }
        $form = new Form();
        try {
            $values = $form->values();
        } catch (\Exception $e) {
            throw new \Exception(_("Can not read the data from the form : " .
            "Missing CSRF Token"), 500);
        }
        $errors = $zones->verify($values, $values["id"]);
        $form->saveValuesErrors($values, $errors);
        if (count($errors)) {
            $route->redirect("/zone/$zone/$viewname/edit");
        }
        $form->saveValuesErrorsReset();
        $zones->update($zone, $viewname, $values);
        $renderer = new Renderer();
        $renderer->flash("SUCCESS", _("Update done"));
        $route->redirect("/zone", "");
    }
)

// Display the form to add a new zone
->get("zone/add", function () use ($authentication, $route) {
    $authHTML = $authentication->verifAuthHTML();
    $route->viewMethod = "update";
    $zones = new Zones($authHTML);
    $editright = $zones->editright();
    if (!$editright) {
        $renderer = new Renderer();
        $renderer->flash("ERROR", _("Don't have the right to add a new zone"));
        $route->redirect("/zone", "");
    }
    $values = array();
    $titles = $zones->titles();
    $configuration = new Configuration();
    $viewSupport = $configuration->get("viewSupport");
    return array("values" => $values, "titles" => $titles,
                "editright" => $editright,
                "viewSupport" => $viewSupport);
})

// Save the parameters if they are corrects
->post("zone/add", function () use ($authentication, $route) {
    $authHTML = $authentication->verifAuthHTML();
    $zones = new Zones($authHTML);
    $form = new Form();
    try {
        $values = $form->values();
    } catch (\Exception $e) {
        throw new \Exception(_("Can not read the data from the form : " .
        "Missing CSRF Token"), 500);
    }
    $errors = $zones->verify($values);
    $form->saveValuesErrors($values, $errors);
    if (count($errors)) {
        $route->redirect("/zone/add");
    }
    $form->saveValuesErrorsReset();
    $zones->insert($values);
    $renderer = new Renderer();
    $renderer->flash("SUCCESS", _("Creation done"));
    $route->redirect("/zone", "");
})
;

////////////////////////////
////    RECORDS PART    ////
////////////////////////////
$route->viewClass = 'Views\\Records';
$route
// Display the form to add multiple records
->get(
    "zone/{zone}/{viewname}/addmultiple",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $route->viewMethod = "updateMultiple";
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        if ($readonly) {
            throw new \Exception(_("Read-only zone"), 403);
        }
        $values = $zones->read($zone, $viewname, null, array(array("id")));
        $zoneid = $values[0]["id"];
        $directZone = $zones->direct($zone);
        $records = new Records($authHTML);
        $values = array();
        $titles = $records->titles();
        $configuration = new Configuration();
        $viewSupport = $configuration->get("viewSupport");
        return array("zone" => $zone, "viewname" => $viewname,
                    "values" => $values, "titles" => $titles,
                    "zoneid" => $zoneid, "directZone" => $directZone,
                    "readonly" => $readonly,
                    "viewSupport" => $viewSupport);
    }
)
// Save the multiple records
->post(
    "zone/{zone}/{viewname}/addmultiple",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $zones = new Zones($authHTML);
        $records = new Records($authHTML);
        $form = new Form();
        $values = $form->values();
        $errors = $records->verifyMultiple($values);
        $form->saveValuesErrors($values, $errors);
        if (count($errors)) {
            if ($zone === ".") {
                $zone = "dot";
            }
            if ($viewname === ".") {
                $viewname = "dot";
            }
            $route->redirect("/zone/$zone/$viewname/addmultiple");
        }
        $form->saveValuesErrorsReset();

        $renderer = new Renderer();
        try {
            $errors = $records->insertRange($values);
            if ($errors === true) {
                $renderer->flash("SUCCESS", _("Creation done"));
                $route->redirect("/zone/$zone/$viewname", "");
            }
        } catch (\Exception $e) {
            $renderer->flash("ERROR", $e->getMessage());
            if ($zone === ".") {
                $zone = "dot";
            }
            if ($viewname === ".") {
                $viewname = "dot";
            }
            $route->redirect("/zone/$zone/$viewname/addmultiple", "");
        }
    }
)

// Display one empty record to add
->get(
    "zone/{zone}/{viewname}/add(\?{fill})?",
    function ($zone, $viewname, $fill) use ($authentication, $route) {
        $authHTML = $authentication->verifAuthHTML();
        $route->viewMethod = "update";
        if ($fill !== null) {
            if (array_key_exists("filled", $_SESSION["domdns"])) {
                $_SESSION["domdns"]["values"] = $_SESSION["domdns"]["filled"];
            }
            unset($_SESSION["domdns"]["filled"]);
            $route->redirect("/zone/$zone/$viewname/add", "");
        }
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        if ($readonly) {
            $renderer = new Renderer();
            $renderer->flash("ERROR", _("Read-only zone"));
            if ($zone === ".") {
                $zone = "dot";
            }
            if ($viewname === ".") {
                $viewname = "dot";
            }
            $route->redirect("/zone/$zone/$viewname", "");
        }
        $values = $zones->read($zone, $viewname, null, array(array("id")));
        $zoneid = $values[0]["id"];
        $directZone = $zones->direct($zone);
        $records = new Records($authHTML);
        $values = array();
        $titles = $records->titles();
        $configuration = new Configuration();
        $viewSupport = $configuration->get("viewSupport");
        return array("zone" => $zone, "viewname" => $viewname,
                    "values" => $values, "titles" => $titles,
                    "zoneid" => $zoneid, "directZone" => $directZone,
                    "readonly" => $readonly,
                    "viewSupport" => $viewSupport);
    }
)

// Save the new record if the parameters are corrects and then redisplay the
// page
->post(
    "zone/{zone}/{viewname}/add",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        if ($readonly) {
            $renderer = new Renderer();
            $renderer->flash("ERROR", _("Read-only zone"));
            if ($zone === ".") {
                $zone = "dot";
            }
            if ($viewname === ".") {
                $viewname = "dot";
            }
            $route->redirect("/zone/$zone/$viewname", "");
        }
        $records = new Records($authHTML);
        // Create the form to use the functions
        $form = new Form();
        // Check CSRF and get the provided values if they exists
        $values = $form->values();
        $errors = $records->verify($values);
        $renderer = new Renderer();
        if (count($errors) == 0) {
            try {
                $records->insert($values);
                $renderer->flash("SUCCESS", _("Creation done"));
                // Check if the target already exists
                if (
                    $values["class"] == "IN" && $values["rr"] === "A" ||
                    $values["class"] == "IN" && $values["rr"] === "AAAA"
                ) {
                    if ($values["class"] == "IN" && $values["rr"] === "A") {
                        $revip = $records->reverseIPAddress($values["target"]);
                        $revip .= ".in-addr.arpa.";
                        $reverseData = dns_get_record($revip, DNS_PTR);
                    } else {
                        $revip = $records->reverseIPAddress($values["target"]);
                        $revip .= ".ip6.arpa.";
                        $reverseData = dns_get_record($revip, DNS_PTR);
                    }
                    if ($reverseData === false) {
                        $renderer->flash("WARNING", _("Can't check the reverse IP"));
                    } elseif (count($reverseData) === 0) {
                        // The target is not define. Check if we can propose to store the
                        // reverse in one of our zones.
                        // The user must have RW access to the zone, and the zone must match
                        // with the target
                        $zonesAllowed = array();
                        foreach ($zones->read() as $z) {
                            $tmpzone = $z["zone"];
                            $tmpviewname = $z["viewname"];
                            if ($zones->readonly($tmpzone, $tmpviewname) === true) {
                                continue;
                            }
                            if (substr($revip, -1 - strlen($tmpzone)) === ".$tmpzone") {
                                // Add the dot to be sure to not take a zone smaller than the
                                // needed
                                $zonesAllowed[] = $tmpzone . "/" . $tmpviewname;
                            }
                        }
                        if (count($zonesAllowed) > 0) {
                            // There is at least one zone available to store the missing direct
                            // record.
                            // Store the values to fill the fields in add form
                            $_SESSION["domdns"]["filled"] = array("source" => $revip,
                                                       "class" => $values["class"],
                                                       "rr" => "PTR",
                                                       "target" => $values["source"] .
                                                                 ".$zone");
                            $select = "<select id='zoneAllowed' class='form-control'>";
                            foreach ($zonesAllowed as $z) {
                                $select .= "<option>$z</option>";
                            }
                            $select .= "</select>";
                            $go = "<a id='btnReverse' class='btn btn-default'>" . _("Go!") . "</a>";
                            $js = <<< EOT
            <script>
              $('#btnReverse').click (function () {
                document.location = '{baseurl}zone/'+
                                    $('#zoneAllowed option:selected').text()+
                                    '/add?fill';
              });
            </script>
    EOT;
                            $renderer->flash("INFO", "<form style='display:inline' " .
                                          "class='form-inline'>" .
                                  sprintf(
                                      _("Reverse IP '%s' not defined. Create it in zone %s %s ?"),
                                      "<strong>$revip</strong>",
                                      $select,
                                      $go
                                  ) .
                                           "</form>$js");
                        } else {
                            // No zone available to store the target : standard message
                            $renderer->flash(
                                "INFO",
                                sprintf(
                                    _("Reverse IP '%s' not defined"),
                                    "<strong>$revip</strong>"
                                )
                            );
                        }
                    }
                } elseif ($values["class"] == "IN" && $values["rr"] === "PTR") {
                    $reverseData = dns_get_record($values["target"], DNS_ANY);
                    if ($reverseData === false) {
                        $renderer->flash("WARNING", _("Can't check the direct name"));
                    } elseif (count($reverseData) === 0) {
                        // The Direct record is not defined. Check if we can propose to store
                        // the record in one of our zones.
                        // The user must have RW access to the zone, and the zone must match
                        // with the target
                        $zonesAllowed = array();
                        foreach ($zones->read() as $z) {
                            $tmpzone = $z["zone"];
                            $tmpviewname = $z["viewname"];
                            if ($zones->readonly($tmpzone, $tmpviewname) === true) {
                                continue;
                            }
                            if (substr($values["target"], -1 - strlen($tmpzone)) === ".$tmpzone") {
                                // Add the dot to be sure to not take a zone smaller than the
                                // needed
                                $zonesAllowed[] = $tmpzone . "/" . $tmpviewname;
                            }
                        }
                        if (count($zonesAllowed) > 0) {
                            // Store the values to fill the fields in add form
                            $revip = substr(
                                $values["source"] . ".$zone",
                                0,
                                strrpos($values["source"] . ".$zone", ".", -8)
                            );
                            $revip = $records->reverseIPAddress($revip);
                            if (strpos($zone, ".ip6.arpa.") !== false) {
                                $revip = $records->compressIPAddress($revip);
                                $_SESSION["domdns"]["filled"] = array(
                                  "source" => $values["target"],
                                  "class" => $values["class"],
                                  "rr" => "AAAA",
                                  "target" => $revip);
                            } else {
                                $_SESSION["domdns"]["filled"] = array(
                                "source" => $values["target"],
                                "class" => $values["class"],
                                "rr" => "A",
                                "target" => $revip);
                            }
                            $select = "<select id='zoneAllowed' class='form-control'>";
                            foreach ($zonesAllowed as $z) {
                                $select .= "<option>$z</option>";
                            }
                            $select .= "</select>";
                            $go = "<a id='btnReverse' class='btn btn-default'>" . _("Go!") . "</a>";
                            $js = <<< EOT
            <script>
              $('#btnReverse').click (function () {
                document.location = '{baseurl}zone/'+
                                    $('#zoneAllowed option:selected').text()+
                                    '/add?fill';
              });
            </script>
    EOT;
                            $renderer->flash("INFO", "<form style='display:inline' " .
                                          "class='form-inline'>" .
                                  sprintf(
                                      _("Direct name '%s' not defined. Create it in zone %s %s ?"),
                                      "<strong>" . $values["target"] . "</strong>",
                                      $select,
                                      $go
                                  ) . "</form>$js");
                        } else {
                            // No zone available to store the target : standard message
                            $renderer->flash(
                                "INFO",
                                sprintf(
                                    _("Direct name '%s' not defined"),
                                    "<strong>" . $values["target"] . "</strong>"
                                )
                            );
                        }
                    } else {
                        // Must be a A or AAAA record
                        $found = 0;
                        foreach ($reverseData as $val) {
                            if (
                                $val["type"] === "A" || $val["type"] === "AAAA" ||
                                $val["type"] === "CNAME"
                            ) {
                                $found = 1;
                            }
                        }
                        if ($found === 0) {
                            $renderer->flash(
                                "INFO",
                                sprintf(
                                    _("Direct name '%s' don't have A or AAAA record"),
                                    "<strong>" . $values["target"] . "</strong>"
                                )
                            );
                        }
                    }
                }
                $route->redirect("/zone/$zone/$viewname", "");
            } catch (\Exception $e) {
                $renderer->flash("ERROR", $e->getMessage());
            }
        }
        $displayCount = false;
        foreach ($errors as $k => $v) {
            if (is_int($k)) {
                $renderer->flash("ERROR", $v[1]);
            } elseif ($displayCount === false) {
                $renderer->flash("ERROR", sprintf(
                    _("%s error(s) occured"),
                    count($errors)
                ));
                $displayCount = true;
            }
        }

        // If errors : save them and redirect to the page of editing to be corrected
        if ($zone === ".") {
            $zone = "dot";
        }
        if ($viewname === ".") {
            $viewname = "dot";
        }
        $_SESSION["domdns"]["errors"] = $errors;
        $_SESSION["domdns"]["values"] = $values;
        $route->redirect("/zone/$zone/$viewname/add", "");
    }
)

// Import a Bind file in the zone : the HTML to ask the information to the user
->get(
    "zone/{zone}/{viewname}/import",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $route->viewMethod = "import";
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        $values = $zones->read($zone, $viewname, null, array(array("id")));
        if (count($values) === 0) {
            throw new \Exception(_("Zone not found with this view name"), 404);
        }
        return array("zone" => $zone, "viewname" => $viewname,
                    "readonly" => $readonly);
    }
)

// Import a Bind file in the zone : the real action page
->post(
    "zone/{zone}/{viewname}/import",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        $values = $zones->read($zone, $viewname, null, array(array("id")));
        if (count($values) === 0) {
            throw new \Exception(_("Zone not found with this view name"), 404);
        }
        $records = new Records($authHTML);
        $form = new Form();
        try {
            $values = $form->values();
        } catch (\Exception $e) {
            throw new \Exception(_("Can not read the data from the form : " .
            "Missing CSRF Token"), 500);
        }
        $form->saveValuesErrorsReset();
        $records->importBind($zone, $viewname, $values["import"]);
        if ($zone === ".") {
            $zone = "dot";
        }
        if ($viewname === ".") {
            $viewname = "dot";
        }
        $route->redirect("/zone/$zone/$viewname", "");
    }
)

->get(
    "zone/{zone}/{viewname}/matrix",
    function ($zone, $viewname) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $route->viewMethod = "matrix";
        $zones = new Zones($authHTML);
        if ($zones->direct($zone)) {
            throw new \Exception(_("Can't display the matrix for a direct zone"), 406);
        }
        $records = new Records($authHTML);
        $values = $records->read($zone, $viewname);
        return array("zone" => $zone, "viewname" => $viewname,
                "values" => $values);
    }
)

// Delete one record from the zone
->get(
    "zone/{zone}/{viewname}/{record}/del/{token}",
    function ($zone, $viewname, $record, $token) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        $values = $zones->read($zone, $viewname, null, array(array("id")));
        if (count($values) === 0) {
            throw new \Exception(_("Zone not found with this view name"), 404);
        }
        $zoneid = $values[0]["id"];
        $directZone = $zones->direct($zone);
        $records = new Records($authHTML);
        $titles = $records->titles();
        $values = $records->read(
            $zone,
            $viewname,
            $record,
            null,
            array(array("source"))
        );
        if (count($values) === 0) {
            throw new \Exception(_("Record not found with this zone/view"), 404);
        }
        $csrf = new Csrf();
        if ($csrf->checkToken($token) !== true) {
            throw new \Exception(_("Invalid CSRF token provided"), 500);
        }
        $recordID = $values[0]["id"];
        $records = new Records($authHTML);
        if ($zone === ".") {
            $zone = "dot";
        }
        if ($viewname === ".") {
            $viewname = "dot";
        }
        try {
            $data = $records->delete($recordID);
        } catch (\Exception $e) {
            $_SESSION["domdns"]["errors"] = _("Can't delete the record");
        }
        $route->redirect("/zone/$zone/$viewname", "");
    }
)

// Display one record to modify
->get(
    "zone/{zone}/{viewname}/{record}",
    function ($zone, $viewname, $record) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $route->viewMethod = "update";
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        $values = $zones->read($zone, $viewname, null, array(array("id")));
        if (count($values) === 0) {
            throw new \Exception(_("Zone not found with this view name"), 404);
        }
        $zoneid = $values[0]["id"];
        $directZone = $zones->direct($zone);
        $records = new Records($authHTML);
        $titles = $records->titles();
        $values = $records->read(
            $zone,
            $viewname,
            $record,
            null,
            array(array("source"))
        );
        if (count($values) === 0) {
            throw new \Exception(_("Record not found with this zone/view"), 404);
        }
        $values = $values[0];
        // Correct the opendate and closedate
        if (isset($values["closedate"]) && strlen($values["closedate"]) === 19) {
            $values["closedate"] = substr($values["closedate"], 0, -3);
        }
        if (isset($values["opendate"]) && strlen($values["opendate"]) === 19) {
            $values["opendate"] = substr($values["opendate"], 0, -3);
        }
        return array("zone" => $zone,
                    "viewname" => $viewname,
                    "values" => $values,
                    "titles" => $titles,
                    "zoneid" => $zoneid,
                    "directZone" => $directZone,
                    "readonly" => $readonly,
                    "record" => $record);
    }
)

// Save the parameters if they are corrects and then redisplay the page
->post(
    "zone/{zone}/{viewname}/{record}",
    function ($zone, $viewname, $record) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        $zones = new Zones($authHTML);
        $readonly = $zones->readonly($zone, $viewname);
        if ($readonly) {
            $renderer = new renderer();
            $renderer->flash("ERROR", _("Read-only zone"));
            if ($zone === ".") {
                $zone = "dot";
            }
            if ($viewname === ".") {
                $viewname = "dot";
            }
            $route->redirect("/zone/$zone/$viewname", "");
        }
        $records = new Records($authHTML);
        // Create the form to use the functions
        $form = new Form();
        // Check CSRF and get the provided values if they exists
        $values = $form->values();
        $errors = $records->verify($values, $values["id"]);
        $renderer = new Renderer();
        if (count($errors) == 0) {
            try {
                $records->update($values["id"], $values);
                $renderer->flash("SUCCESS", _("Update done"));
                if ($zone === ".") {
                    $zone = "dot";
                }
                if ($viewname === ".") {
                    $viewname = "dot";
                }
                $route->redirect("/zone/$zone/$viewname", "");
            } catch (\Exception $e) {
                $renderer->flash("ERROR", $e->getMessage());
            }
        }
        // If errors : save them and redirect to the page of editing to be corrected
        $displayCount = false;
        foreach ($errors as $k => $v) {
            if (is_int($k)) {
                $renderer->flash("ERROR", $v[1]);
            } elseif ($displayCount === false) {
                $renderer->flash("ERROR", sprintf(
                    _("%s error(s) occured"),
                    count($errors)
                ));
                $displayCount = true;
            }
        }
        if ($zone === ".") {
            $zone = "dot";
        }
        if ($viewname === ".") {
            $viewname = "dot";
        }
        $_SESSION["domdns"]["errors"] = $errors;
        $_SESSION["domdns"]["values"] = $values;
        $route->redirect("/zone/$zone/$viewname/$record", "");
    }
)

// Display all the records in the zone
->get(
    "zone/{zone}/{viewname}(\.{extension})?",
    function ($zone, $viewname, $extension) use ($authentication, $route) {
        if ($zone === "dot") {
            $zone = ".";
        }
        if ($viewname === "dot") {
            $viewname = ".";
        }
        $authHTML = $authentication->verifAuthHTML();
        if ($extension === "bind") {
            $route->output = "txt";
            $servers = new Servers($authHTML);
            return $servers->zoneexportTextBind($zone, $viewname);
        }
        $route->viewMethod = "read";
        $records = new Records($authHTML);
        $values = $records->read($zone, $viewname);
        $titles = $records->titles();
        $zones = new Zones($authHTML);
        $directZone = $zones->direct($zone);
        $readonly = $zones->readonly($zone, $viewname);
        $disabled = $zones->disabled($zone, $viewname);
        $userprefs = new Userprefs($authHTML);
        $autoactivateDefault = $userprefs->read("autoactivate-DEFAULT");
        if (isset($autoactivateDefault["value"])) {
            $autoactivateDefault = $autoactivateDefault["value"];
        } else {
            $autoactivateDefault = 1;
        }
        $autoactivateZoneView = $userprefs->read("autoactivate-$zone/$viewname");
        if (isset($autoactivateZoneView["value"])) {
            $autoactivateZoneView = $autoactivateZoneView["value"];
        } else {
            $autoactivateZoneView = $autoactivateDefault;
        }
        return array("zone" => $zone,
                    "viewname" => $viewname,
                    "values" => $values,
                    "titles" => $titles,
                    "directZone" => $directZone,
                    "readonly" => $readonly,
                    "disabled" => $disabled,
                    "autoactivateDefault" => $autoactivateDefault,
                    "autoactivateZoneView" => $autoactivateZoneView);
    }
);


//////////////////////////
////    VIEWS LIST    ////
//////////////////////////
$route->viewClass = 'Views\\Zones';
$route
// Just see the zone : display all the zones and views
->get("zone/{zone}", function ($zone) use ($authentication, $route) {
    if ($zone === "dot") {
        $zone = ".";
    }
    $authHTML = $authentication->verifAuthHTML();
    $route->viewMethod = "read";
    $zones = new Zones($authHTML);
    $values = $zones->read($zone, null, null, array(array("zone")));
    $titles = $zones->titles();
    $configuration = new Configuration();
    $viewSupport = $configuration->get("viewSupport");
    return array("zones" => $values, "titles" => $titles,
                "viewSupport" => $viewSupport);
});

///////////////////
//   USERPREFS   //
///////////////////
$route->viewClass = 'Views\\Userprefs';
$route
->get("user", function () use ($authentication, $route) {
    // Allow anonymous if it is allowed to see the zones
    $authHTML = $authentication->verifAuthHTML();
    $route->viewMethod = "read";
    $userprefs = new Userprefs($authHTML);
    $values = $userprefs->getAll();
    return array("values" => $values, "user" => $authHTML);
})

->post("user", function () use ($authentication, $route) {
    $authHTML = $authentication->verifAuthHTML();
    $userprefs = new Userprefs($authHTML);
    $form = new Form();
    // Check CSRF and get the provided values if they exists
    $values = $form->values();
    unset($values["submit"]);
    $errors = $userprefs->verify($values);
    $renderer = new Renderer();
    if (count($errors) == 0) {
        try {
            $userprefs->setAll($values);
            $language = new Language();
            $lang = $language->activeLanguage(
                "domdns",
                "DOMDNS-Language",
                $values["language"]
            );
            $_SESSION["domdns"]["language"] = $lang;

            $route->redirect("/user", "");
        } catch (\Exception $e) {
            $renderer->flash("ERROR", $e->getMessage());
        }
    }
    $displayCount = false;
    foreach ($errors as $k => $v) {
        if (is_int($k)) {
            $renderer->flash("ERROR", $v[1]);
        } elseif ($displayCount === false) {
            $renderer->flash("ERROR", sprintf(
                _("%s error(s) occured"),
                count($errors)
            ));
            $displayCount = true;
        }
    }

    // If errors : save them and redirect to the page of editing to be corrected
    $_SESSION["domdns"]["errors"] = $errors;
    $_SESSION["domdns"]["values"] = $values;
    $route->redirect("/user", "");
})
;

///////////////////
//   HELP PAGE   //
///////////////////
$route->viewClass = "Views\\Help";
$route
->get("help", function () use ($authentication, $route) {
    $route->viewMethod = "help";
    return array();
})
;

$route
/////////////////////////
//    GENERAL CATCH    //
/////////////////////////
// If there is a no catch route ending with slash, try to reroute without slash
/*$route->get ("{string}/",
  function ($string)
  {
    $r=new Route ();
    $r->redirect ($string);
  });*/

->multi("get,post,put,delete", "{unused}", function () use ($route) {
    throw new \Exception(_("Page not found"), 404);
})
;
