<?php

/**
 * DomFramework
 * @package domframework
 * @author Dominique Fournier <dominique@fournier38.fr>
 * @license BSD
 */

namespace Domframework;

/**
 * The general axis management
 */
class GraphAxisGeneral
{
    /**
     * The min value of the axis. Do not use it if the axis is composed of labels
     */
    protected $min = null;

    /**
     * The max value of the axis. Do not use it if the axis is composed of labels
     */
    protected $max = null;

    /**
     * The label max to display
     */
    protected $labelMax;

    /**
     * The label min to display
     */
    protected $labelMin;

    /**
     * The data displayed as values on the axis
     */
    protected $data;

    /**
     * Set if the axis is only numerical (true) or is composed of labels (false)
     */
    protected $numerical;

    /**
     * The minimum bottom position in pixels. Used on vertical axis
     */
    protected $bottom;

    /**
     * The maximum top position in pixels. Used on vertical axis
     */
    protected $top;

    /**
     * The minimum left position in pixels. Used on horizontal axis
     */
    protected $left;

    /**
     * The maximum right position in pixels. Used on horizontal axis
     */
    protected $right;

    /**
     * The fontfile to write the labels
     */
    protected $fontfile;

    /**
     * The fontsize to write the labels
     */
    protected $fontsize = 8;

    /**
     * Axis color
     */
    protected $axisColor;

    /**
     * Grid color on the axis
     * Can be set to null or "transparent" to not display the grid
     */
    protected $gridColor;

    /**
     * Set the min value of the axis if the parameter is provided.
     * Get the min value of the axis if the parameter is not provided
     * @param integer|null $min The min value of the axis
     */
    public function min($min = null)
    {
        if ($min === null) {
            return $this->min;
        }
        if (! is_numeric($min)) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid min provided to graph Axis"
            ) . " " . get_class($this), 406);
        }
        $this->min = $min;
        return $this;
    }

    /**
     * Set the max value of the axis if the parameter is provided.
     * Get the max value of the axis if the parameter is not provided
     * @param integer|null $max The max value of the axis
     */
    public function max($max = null)
    {
        if ($max === null) {
            return $this->max;
        }
        if (! is_numeric($max)) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid max provided to graph Axis"
            ) . " " . get_class($this), 406);
        }
        $this->max = $max;
        return $this;
    }

    /**
     * Set the data of the axis if the parameter is provided.
     * Get the data of the axis if the parameter is not provided
     * @param array|null $data The data of the axis
     */
    public function data($data = null)
    {
        if ($data === null) {
            return $this->data;
        }
        if (! is_array($data)) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid data provided to graph Axis"
            ) . " " . get_class($this), 406);
        }
        $this->data = $data;
        if ($this->numerical === null) {
            // Look if the provided data are only numerical. Then define the numerical
            // property
            foreach ($data as $d) {
                if (! is_numeric($d)) {
                    $this->numerical = false;
                    break;
                }
            }
            if ($this->numerical === null) {
                $this->numerical = true;
            }
        }
        return $this;
    }

    /**
     * Set if the axis is numerical or composed of labels if the parameter is
     * provided.
     * Get if the axis is numerical if the parameter is not provided
     * @param boolean|null $numerical the axis is numerical
     */
    public function numerical($numerical = null)
    {
        if ($numerical === null) {
            return $this->numerical;
        }
        if (! is_bool($numerical)) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid numerical parameter provided to graph Axis"
            ) . " " .
            get_class($this), 406);
        }
        $this->numerical = $numerical;
        return $this;
    }

    /**
     * Set the bottom position of the axis if the parameter is provided.
     * Get the bottom position of the axis if the parameter is not provided
     * @param integer|null $bottom The bottom position of the axis
     */
    public function bottom($bottom = null)
    {
        if ($bottom === null) {
            return $this->bottom;
        }
        if (! is_integer($bottom) || $bottom < 0 || $bottom > 5000) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid bottom provided to graph Axis"
            ) . " " . get_class($this), 406);
        }
        $this->bottom = $bottom;
        return $this;
    }

    /**
     * Set the top position of the axis if the parameter is provided.
     * Get the top position of the axis if the parameter is not provided
     * @param integer|null $top The top position of the axis
     */
    public function top($top = null)
    {
        if ($top === null) {
            return $this->top;
        }
        if (! is_integer($top) || $top < 0 || $top > 5000) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid top provided to graph Axis"
            ) . " " . get_class($this), 406);
        }
        $this->top = $top;
        return $this;
    }

    /**
     * Set the left position of the axis if the parameter is provided.
     * Get the left position of the axis if the parameter is not provided
     * @param integer|null $left The left position of the axis
     */
    public function left($left = null)
    {
        if ($left === null) {
            return $this->left;
        }
        if (! is_integer($left) || $left < 0 || $left > 5000) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid left provided to graph Axis"
            ) . " " . get_class($this), 406);
        }
        $this->left = $left;
        return $this;
    }

    /**
     * Set the right position of the axis if the parameter is provided.
     * Get the right position of the axis if the parameter is not provided
     * @param integer|null $right The right position of the axis
     */
    public function right($right = null)
    {
        if ($right === null) {
            return $this->right;
        }
        if (! is_integer($right) || $right < 0 || $right > 5000) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid right provided to graph Axis"
            ) . " " . get_class($this), 406);
        }
        $this->right = $right;
        return $this;
    }

    /**
     * Set the fontfile of the labels if the parameter is provided.
     * Get the fontfile of the labels if the parameter is not provided
     * @param string|null $fontfile The fontfile of the title
     */
    public function fontfile($fontfile = null)
    {
        if ($fontfile === null) {
            return $this->fontfile;
        }
        if (
            ! is_string($fontfile) || strlen($fontfile) < 0 ||
            ! file_exists($fontfile) || ! is_readable($fontfile)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid fontfile provided to graph title"
            ), 406);
        }
        $this->fontfile = $fontfile;
        return $this;
    }

    /**
     * Set the font size of the labels if the parameter is provided.
     * Get the font size of the labels if the parameter is not provided
     * @param integer|null $fontsize The font size of the title
     */
    public function fontsize($fontsize = null)
    {
        if ($fontsize === null) {
            return $this->fontsize;
        }
        if (! is_integer($fontsize) || $fontsize < 2 || $fontsize > 100) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid fontsize provided to graph title"
            ), 406);
        }
        $this->fontsize = $fontsize;
        return $this;
    }

    /**
     * Set the axis color if the parameter is provided.
     * Get the axis color if the parameter is not provided
     * @param string|null $axisColor The axis color
     */
    public function axisColor($axisColor = null)
    {
        if ($axisColor === null) {
            return $this->axisColor;
        }
        if (
            ! is_string($axisColor) ||
            ! in_array($axisColor, Color::colorList(), true)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid axisColor provided to graph axis"
            ), 406);
        }
        $this->axisColor = $axisColor;
        return $this;
    }

    /**
     * Set the grid color if the parameter is provided.
     * Get the grid color if the parameter is not provided
     * @param string|null $gridColor The grid color
     */
    public function gridColor($gridColor = null)
    {
        if ($gridColor === null) {
            return $this->gridColor;
        }
        if (
            ! is_string($gridColor) ||
            ! in_array($gridColor, Color::colorList(), true)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid gridColor provided to graph grid"
            ), 406);
        }
        $this->gridColor = $gridColor;
        return $this;
    }

    /**
     * Calculate the labels that will be displayed on the axis
     * @param integer $nbMaxValues The maximum number of values to return
     * @return array The array of labels to display
     */
    protected function labels($nbMaxValues)
    {
        // Activate the debug of this method
        $deb = false;
        if (! is_int($nbMaxValues)) {
            throw new \Exception(
                "Invalid parameter nbMaxValues provided to graphAxisGeneral::labels",
                500
            );
        }
        if ($deb) {
            echo "=========== LABELS\n";
        }
        if ($this->min > 0) {
            $minLabel = $this->min * 0.90;
        } else {
            $minLabel = $this->min * 1.05;
        }
        if ($this->max > 0) {
            $maxLabel = $this->max * 1.05;
        } else {
            $maxLabel = $this->max * 0.90;
        }

        $minBase = intval(log10($minLabel)) - 1;
        $maxBase = intval(log10($maxLabel)) - 1;
        $base = max($minBase, $maxBase);
        if ($deb) {
            echo "minBase=$minBase, maxBase=$maxBase ===> base = $base\n";
        }

        if ($base < 0 && $base > -1) {
            $base = $base - 1;
        }
        // The while loop reduce the base to not explode the number of labels
        // If there is too much labels, remove one digit and retry
        while (1) {
            $this->labelMin = null;
            $this->labelMax = null;
            $min = round($minLabel, -1 * $base);
            $max = round($maxLabel, -1 * $base);
            if ($deb) {
                echo "min = $min, max = $max\n";
            }

            $scale = pow(10, $base);
            if ($deb) {
                echo "BASE=" . ($base) . ", SCALE = $scale\n";
            }
            if ($scale === 0 || $scale === 0.0) {
                die("Scale equal 0 on line " . __LINE__ . "\n");
            }
            $labels = [];
            if ($this->min <= 0 && $this->max >= 0) {
                // Return the 0 label and the values arround it
                $zeroAlreadyDraw = false;
                for (
                    $i = 0;
                    \bccomp($i, 1.2 * $max, abs($base) + 1) < 1;
                    $i += $scale
                ) {
                    if ($i === 0) {
                        $zeroAlreadyDraw = true;
                    }
                    if ($deb) {
                        echo "Values with Zero : add pos val $i\n";
                    }
                    $labels[] = $i;
                }
                for (
                    $i = 0;
                    \bccomp($i, 1.4 * $min, abs($base) + 1) >= 0;
                    $i -= $scale
                ) {
                    if ($zeroAlreadyDraw === true && $i === 0) {
                        continue;
                    }
                    if ($deb) {
                        echo "Values with Zero : add neg val $i\n";
                    }
                    $labels[] = $i;
                }
            } else {
                // No 0 Axis : From $this->min to $this->max
                for (
                    $i = $min;
                    \bccomp($i, $max, abs($base) + 1) < 1;
                    $i += $scale
                ) {
                    if ($deb) {
                        echo "Values no Zero : add val $i\n";
                    }
                    $labels[] = $i;
                }
            }
            if (count($labels) <= $nbMaxValues) {
                foreach ($labels as &$label) {
                    if ($base < 0) {
                        $label = sprintf("%0." . intval(abs($base)) . "f", $label);
                    } else {
                        $label = sprintf("%" . intval($base) . "d", $label);
                    }
                    if (
                        $this->labelMin === null ||
                        \bccomp($this->labelMin, $label, abs($base) + 1) >= 0
                    ) {
                        $this->labelMin = $label;
                    }
                    if (
                        $this->labelMax === null ||
                        \bccomp($this->labelMax, $label, abs($base) + 1) <= 0
                    ) {
                        $this->labelMax = $label;
                    }
                }

                if (\bccomp($this->labelMin, $this->min, abs($base) + 1) > 0) {
                    // Add a label in the minimum
                    $this->labelMin = $this->labelMin - $scale;
                    if ($deb) {
                        echo "Force Add Min $this->labelMin\n";
                    }
                    if ($base < 0) {
                        $labels[] = sprintf(
                            "%0." . intval(abs($base)) . "f",
                            $this->labelMin
                        );
                    } else {
                        $labels[] = sprintf("%" . intval($base) . "d", $this->labelMin);
                    }
                }
                if (\bccomp($this->labelMax, $this->max, abs($base) + 1) < 0) {
                    $this->labelMax = $this->labelMax + $scale;
                    if ($deb) {
                        echo "Force Add Max $this->labelMax\n";
                    }
                    if ($base < 0) {
                        $labels[] = sprintf(
                            "%0." . intval(abs($base)) . "f",
                            $this->labelMax
                        );
                    } else {
                        $labels[] = sprintf("%" . intval($base) . "d", $this->labelMax);
                    }
                }
                if ($this->labelMin > $this->min) {
                    die("labelMin > min ($this->labelMin > $this->min)\n");
                }
                if ($this->labelMax < $this->max) {
                    die("labelMax < max ($this->labelMax < $this->max)\n");
                }
                if (count($labels) <= $nbMaxValues) {
                    $labels = array_unique($labels);
                    if ($deb) {
                        print_r($labels);
                    }
                    return $labels;
                }
            }
            if ($deb) {
                echo "LOOP == " . count($labels) . " > $nbMaxValues ==========\n";
            }
            $base = $base + 0.1;
        }
    }
}
