<?php

/**
 * DomFramework
 * @package domframework
 * @author Dominique Fournier <dominique@fournier38.fr>
 * @license BSD
 */

namespace Domframework;

/**
 * The graphLegend object
 */
class GraphLegend
{
    /**
     * Show the legend (no legend by default)
     */
    private $show = false;

    /**
     * The TTF fontfile to use
     */
    private $fontfile = null;

    /**
     * The font size to use
     */
    private $fontsize = 10;

    /**
     * The legend color for the font
     */
    private $color = "black";

    /**
     * The legend background-color
     */
    private $bgcolor = "white";

    /**
     * The legend border color
     */
    private $borderColor = "black";

    /**
     * The padding arround the title (in px)
     */
    private $padding = 10;

    /**
     * Set the legend display status if the parameter is provided.
     * Get the legend display status if the parameter is not provided
     * @param boolean|null $show The legend display status
     */
    public function show($show = null)
    {
        if ($show === null) {
            return $this->show;
        }
        if (! is_bool($show)) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid show value provided to graph legend"
            ), 406);
        }
        $this->show = $show;
        return $this;
    }

    /**
     * Set the fontfile of the legend if the parameter is provided.
     * Get the fontfile of the legend if the parameter is not provided
     * @param string|null $fontfile The fontfile of the legend
     */
    public function fontfile($fontfile = null)
    {
        if ($fontfile === null) {
            return $this->fontfile;
        }
        if (
            ! is_string($fontfile) || strlen($fontfile) < 0 ||
            ! file_exists($fontfile) || ! is_readable($fontfile)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid fontfile provided to graph legend"
            ), 406);
        }
        $this->fontfile = $fontfile;
        return $this;
    }

    /**
     * Set the font size of the legend if the parameter is provided.
     * Get the font size of the legend if the parameter is not provided
     * @param integer|null $fontsize The font size of the legend
     */
    public function fontsize($fontsize = null)
    {
        if ($fontsize === null) {
            return $this->fontsize;
        }
        if (! is_integer($fontsize) || $fontsize < 2 || $fontsize > 100) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid fontsize provided to graph legend"
            ), 406);
        }
        $this->fontsize = $fontsize;
        return $this;
    }

    /**
     * Set the color of the legend if the parameter is provided.
     * Get the color of the legend if the parameter is not provided
     * @param string|null $color The color of the legend
     */
    public function color($color = null)
    {
        if ($color === null) {
            return $this->color;
        }
        if (
            ! is_string($color) ||
            ! in_array($color, Color::colorList(), true)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid color provided to graph legend"
            ), 406);
        }
        $this->color = $color;
        return $this;
    }

    /**
     * Set the background-color of the legend if the parameter is provided.
     * Get the background-color of the legend if the parameter is not provided
     * @param string|null $bgcolor The background-color of the legend
     */
    public function bgcolor($bgcolor = null)
    {
        if ($bgcolor === null) {
            return $this->bgcolor;
        }
        if (
            ! is_string($bgcolor) ||
            ! in_array($bgcolor, Color::colorList(), true)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid bgcolor provided to graph legend"
            ), 406);
        }
        $this->bgcolor = $bgcolor;
        return $this;
    }

    /**
     * Set the border color of the legend if the parameter is provided.
     * Get the border color of the legend if the parameter is not provided
     * @param string|null $borderColor The border color of the legend
     */
    public function borderColor($borderColor = null)
    {
        if ($borderColor === null) {
            return $this->borderColor;
        }
        if (
            ! is_string($borderColor) ||
            ! in_array($borderColor, Color::colorList(), true)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid borderColor provided to graph legend"
            ), 406);
        }
        $this->borderColor = $borderColor;
        return $this;
    }

    /**
     * Set the padding of the legend if the parameter is provided.
     * Get the padding of the legend if the parameter is not provided
     * @param integer|null $padding The padding of the legend
     */
    public function padding($padding = null)
    {
        if ($padding === null) {
            return $this->padding;
        }
        if (! is_integer($padding) || $padding < 0 || $padding > 200) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid padding provided to graph legend"
            ), 406);
        }
        $this->padding = $padding;
        return $this;
    }

    /**
     * Draw the legend in the $gd resource provided
     * @param resource $gd The resource to modify
     * @param array $free The free space coordinates on the graphic
     * @param object $series The series to graph
     * @return array the new free coordinates array
     */
    public function draw($gd, $free, $series)
    {
        if ($this->show === false) {
            return $free;
        }
        // Look for maxmimum width of the labels
        $maxwidth = 0;
        $height = 0;
        foreach ($series->getList() as $number => $serie) {
            $bbox = imagettfbbox($this->fontsize, 0, $this->fontfile, $serie);
            $width = abs($bbox[4] - $bbox[0]);
            // TODO : If the serie name is too long, split it !
            // 20px for the sample + the space before the label
            if ((20 + $maxwidth) > (($free[2] - $free[0]) / 2)) {
                throw new \Exception(
                    dgettext(
                        "domframework",
                        "The serie name in legend must not takes more than half of the graph"
                    ),
                    500
                );
            }
            if ($number > 0) {
                $height += $this->padding;
            }
            $height += abs($bbox[5] - $bbox[1]);
            $maxwidth = max($maxwidth, $width);
        }

        $x1 = $free[2] - $maxwidth - 20 - $this->padding * 2;
        $x2 = $free[2] - $this->padding;
        $y1 = $free[1] + $this->padding;
        $y2 = $y1 + $height + $this->padding;

        // Size of the border : 1px
        $border = 1;
        // Draw the background rectangle
        imagefilledrectangle(
            $gd,
            $x1,
            $y1,
            $x2,
            $y2,
            Color::allocateFromText($gd, $this->borderColor)
        );
        imagefilledrectangle(
            $gd,
            $x1 + $border,
            $y1 + $border,
            $x2 - $border,
            $y2 - $border,
            Color::allocateFromText($gd, $this->bgcolor)
        );

        // Display the serie names
        $y = $y1;
        foreach ($series->getList() as $number => $serie) {
            // Write the label
            $bbox = imagettfbbox($this->fontsize, 0, $this->fontfile, $serie);
            $height = abs($bbox[5] - $bbox[1]);
            $y += $height;
            imagettftext(
                $gd,
                $this->fontsize,
                0,
                $x1 + 30,
                intval($y + $height / 2),
                Color::allocateFromText($gd, $this->color),
                $this->fontfile,
                $serie
            );
            // Draw the sample
            $series->serie($serie)->style()->sample($gd, $x1 + 15, $y);
            $y += $this->padding;
        }
        $free = [$free[0], $free[1], $x1 - 5, $free[3]];
        return $free;
    }
}
