<?php

/**
 * DomFramework
 * @package domframework
 * @author Dominique Fournier <dominique@fournier38.fr>
 * @license BSD
 */

namespace Domframework;

/**
 * The serie object
 */
class GraphSerie
{
    /**
     * The name of the serie
     */
    private $name;

    /**
     * The data values for the serie
     */
    private $data;

    /**
     * The numericalKey is true if all the keys are numeric
     */
    private $numericalKey = null;

    /**
     * The minimum key of the serie
     */
    private $minKey = null;

    /**
     * The maximum key of the serie
     */
    private $maxKey = null;

    /**
     * The numericalValue is true if all the values are numeric
     */
    private $numericalValue = null;

    /**
     * The minimum value of the serie
     */
    private $minValue = null;

    /**
     * The maximum value of the serie
     */
    private $maxValue = null;

    /**
     * The style object for the serie
     */
    private $style;
    /**
     * If set, hide this serie and do not display it
     */
    private $hide;

    /**
     * The axis used to draw the serie.
     * If false for main Y axis
     * If true for secondary Y axis
     */
    private $axisYsecondary = false;

    /**
     * When creating the serie, save the name
     * @param string $name The name of the serie
     */
    public function __construct($name)
    {
        if (! is_string($name)) {
            throw new \Exception(
                dgettext(
                    "domframework",
                    "Can't create a serie if the name is not a string"
                ),
                406
            );
        }
        $this->name = $name;
    }

    /**
     * Set the data for the serie
     * If the parameter is not provided, return the actual $data value
     * @param array|null $data The data to store in the serie
     */
    public function data($data = null)
    {
        if ($data === null) {
            return $this->data;
        }
        if (! is_array($data)) {
            throw new \Exception(
                dgettext(
                    "domframework",
                    "Can't create a serie data if the value is not an array"
                ),
                406
            );
        }
        $this->data = $data;
        $this->minmax();
        return $this;
    }

    /**
     * Get the minimum and maximum values and keys
     */
    private function minmax()
    {
        if ($this->data === null) {
            return null;
        }
        foreach ($this->data as $key => $value) {
            if ($this->minValue === null && is_numeric($value)) {
                $this->minValue = $value;
            }
            if ($this->minKey === null && is_numeric($key)) {
                $this->minKey = $key;
            }
            if (is_numeric($value)) {
                $this->minValue = min($this->minValue, $value);
                $this->maxValue = max($this->maxValue, $value);
            }
            if (is_numeric($key)) {
                $this->minKey = min($this->minKey, $key);
                $this->maxKey = max($this->maxKey, $key);
            }
            if (! is_numeric($key) && $this->numericalKey === null) {
                $this->numericalKey = false;
            }
            if (
                ! is_numeric($value) && $value !== null &&
                $this->numericalValue === null
            ) {
                $this->numericalValue = false;
            }
        }
        if ($this->numericalKey === null) {
            $this->numericalKey = true;
        }
        if ($this->numericalValue === null) {
            $this->numericalValue = true;
        }
    }

    /**
     * Set/get the numeric value of the serie
     * If the parameter is not provided, return the actual state
     * @param boolean|null $numericalValue The state of the numeric value
     */
    public function numericalValue($numericalValue = null)
    {
        if ($numericalValue === null) {
            return $this->numericalValue;
        }
        if (! is_bool($numericalValue)) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid numericalValue provided to serie"
            ), 406);
        }
        $this->numericalValue = $numericalValue;
        return $this;
    }

    /**
     * The minimum value of the serie
     */
    public function minValue()
    {
        if ($this->minValue === null && $this->data !== null) {
            $this->minmax();
        }
        return $this->minValue;
    }

    /**
     * The maximum value of the serie
     */
    public function maxValue()
    {
        if ($this->maxValue === null && $this->data !== null) {
            $this->minmax();
        }
        return $this->maxValue;
    }

    /**
     * Set/get the numeric key of the serie
     * If the parameter is not provided, return the actual state
     * @param boolean|null $numericalKey The state of the numeric key
     */
    public function numericalKey($numericalKey = null)
    {
        if ($numericalKey === null) {
            return $this->numericalKey;
        }
        if (! is_bool($numericalKey)) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid numericalKey provided to serie"
            ), 406);
        }
        $this->numericalKey = $numericalKey;
        return $this;
    }

    /**
     * The minimum key of the serie
     */
    public function minKey()
    {
        if ($this->minKey === null && $this->data !== null) {
            $this->minmax();
        }
        return $this->minKey;
    }

    /**
     * The maximum key of the serie
     */
    public function maxKey()
    {
        if ($this->maxKey === null && $this->data !== null) {
            $this->minmax();
        }
        return $this->maxKey;
    }

    /**
     * The number of elements in the serie
     */
    public function count()
    {
        return count($this->data);
    }

    /**
     * Set/Get the graph style for the serie
     * If the parameter is not provided, return the actual state
     * @param string|object|null $style The graph style
     */
    public function style($style = null)
    {
        if ($style === null) {
            return $this->style;
        }
        if (is_object($style)) {
            $this->style = clone $style;
            return $this->style;
        }
        if (
            ! is_string($style) ||
            ! in_array($style, ["line", "points", "linePoints"], true)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid style provided to serie"
            ), 406);
        }
        $styleClass = "GraphStyle" . ucfirst($style);
        if ($this->style === null) {
            $this->style = new $styleClass();
        }
        return $this->style;
    }
    /**
     * Set/Get the hidden state of the serie
     * If the parameter is not provided, return the actual state
     * @param boolean|null $hide The hidden state
     */
    public function hide($hide = null)
    {
        if ($hide === null) {
            return $this->hide;
        }
        if (! is_bool($hide)) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid hide mode provided to serie"
            ), 406);
        }
        $this->hide = $hide;
        return $this;
    }

    /**
     * The serie is based on the secondary Y axis
     * Set the value if the parameter is provided, get the value if the parameter
     * is not set
     * @param boolean|null $axisYsecondary The Serie on secondary Y axis
     */
    public function axisYsecondary($axisYsecondary = null)
    {
        if ($axisYsecondary === null) {
            return $this->axisYsecondary;
        }
        if (! is_bool($axisYsecondary)) {
            throw new \Exception(
                dgettext(
                    "domframework",
                    "Invalid axisYsecondary provided to graph axisYsecondary"
                ),
                406
            );
        }
        $this->axisYsecondary = $axisYsecondary;
        return $this;
    }

    /**
     * Draw the serie with the defined style class
     * @param resource $gd The resource to modify
     * @param array $free The free space coordinates on the graphic
     * @param object $axisX The axis X used on the graph
     * @param object $axisY The axis Y used on the graph
     */
    public function draw($gd, $free, $axisX, $axisY)
    {
        if ($this->hide === true) {
            return;
        }
        $this->style->draw($gd, $free, $this->data, $axisX, $axisY);
    }
}
