<?php

/**
 * DomFramework
 * @package domframework
 * @author Dominique Fournier <dominique@fournier38.fr>
 * @license BSD
 */

namespace Domframework;

/**
 * The graphTitle object
 */
class GraphTitle
{
    /**
     * The title text
     */
    private $text = null;

    /**
     * The TTF fontfile to use
     */
    private $fontfile = null;

    /**
     * The font size to use
     */
    private $fontsize = 14;

    /**
     * The title color
     */
    private $color = "black";

    /**
     * The padding arround the title (in px)
     */
    private $padding = 10;

    /**
     * Set the text of the title if the parameter is provided.
     * Get the text of the title if the parameter is not provided
     * @param string|null $text The text of the title
     */
    public function text($text = null)
    {
        if ($text === null) {
            return $this->text;
        }
        if (! is_string($text) || strlen($text) < 0 || strlen($text) > 50) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid text provided to graph title"
            ), 406);
        }
        $this->text = $text;
        return $this;
    }

    /**
     * Set the fontfile of the title if the parameter is provided.
     * Get the fontfile of the title if the parameter is not provided
     * @param string|null $fontfile The fontfile of the title
     */
    public function fontfile($fontfile = null)
    {
        if ($fontfile === null) {
            return $this->fontfile;
        }
        if (
            ! is_string($fontfile) || strlen($fontfile) < 0 ||
            ! file_exists($fontfile) || ! is_readable($fontfile)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid fontfile provided to graph title"
            ), 406);
        }
        $this->fontfile = $fontfile;
        return $this;
    }

    /**
     * Set the font size of the title if the parameter is provided.
     * Get the font size of the title if the parameter is not provided
     * @param integer|null $fontsize The font size of the title
     */
    public function fontsize($fontsize = null)
    {
        if ($fontsize === null) {
            return $this->fontsize;
        }
        if (! is_integer($fontsize) || $fontsize < 2 || $fontsize > 100) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid fontsize provided to graph title"
            ), 406);
        }
        $this->fontsize = $fontsize;
        return $this;
    }

    /**
     * Set the color of the title if the parameter is provided.
     * Get the color of the title if the parameter is not provided
     * @param string|null $color The color of the title
     */
    public function color($color = null)
    {
        if ($color === null) {
            return $this->color;
        }
        if (
            ! is_string($color) ||
            ! in_array($color, Color::colorList(), true)
        ) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid color provided to graph title"
            ), 406);
        }
        $this->color = $color;
        return $this;
    }

    /**
     * Set the padding of the title if the parameter is provided.
     * Get the padding of the title if the parameter is not provided
     * @param integer|null $padding The padding of the title
     */
    public function padding($padding = null)
    {
        if ($padding === null) {
            return $this->padding;
        }
        if (! is_integer($padding) || $padding < 0 || $padding > 200) {
            throw new \Exception(dgettext(
                "domframework",
                "Invalid padding provided to graph title"
            ), 406);
        }
        $this->padding = $padding;
        return $this;
    }

    /**
     * Draw the title in the $gd resource provided
     * @param resource $gd The resource to modify
     * @param array $free The free space coordinates on the graphic
     * @return array the new free coordinates array
     */
    public function draw($gd, $free)
    {
        if ($this->text === null) {
            return $free;
        }
        // Look for the bounding box around the text. The bounding is not write on
        // the image and return the coordinates for the text box
        $bbox = imagettfbbox($this->fontsize, 0, $this->fontfile, $this->text);
        // Calculate the position of the text to be centered on the graph
        // The padding is only on vertical : the x is centered
        $x = floor(($free[2] - $free[0] - abs($bbox[4] - $bbox[0])) / 2);
        $y = ceil($free[1] + abs($bbox[5] - $bbox[1])) + $this->padding;
        $x += $free[0];
        $y += $free[1];
        imagettftext(
            $gd,
            $this->fontsize,
            0,
            $x,
            $y,
            Color::allocateFromText($gd, $this->color),
            $this->fontfile,
            $this->text
        );
        return [intval($free[0]), intval($free[1] + $y + $this->padding),
                  intval($free[2]), intval($free[3])];
    }
}
