<?php

/**
 * DomFramework
 * @package domframework
 * @author Dominique Fournier <dominique@fournier38.fr>
 * @license BSD
 */

namespace Domframework;

/**
 * Create a HTTP server easily
 * The server can listen on multiple IP addresses and multiple ports.
 *
 * Need HttpConnection
 */
class Httpserver
{
    private $tcpserver;
    private $listeners = [];
    private $runningUser;
    private $runningGroup;
    private $logger;
    private $maxConnection = 500;
    private $timeout = 30;

    /**
     * Add a new listener. Must all be set before entering in loop
     * @param string $ip The Listening IP Address
     * @param integer $port the Listening port
     * @param callable $callable The method called when a new request enter
     */
    public function addListener($ip, $port, $callable)
    {
        $ipaddresses = new Ipaddresses();
        if (! $ipaddresses->validIPAddress($ip)) {
            throw new \Exception(
                "Httpserver : invalid IP address provided to listen",
                500
            );
        }
        if (
            ! is_numeric($port) || ! ctype_digit($port) || $port < 0 ||
            $port > 65535
        ) {
            throw new \Exception(
                "Httpserver : invalid port provided to listen " .
                "(must be between 0 and 65535)",
                500
            );
        }
        if (! is_callable($callable)) {
            throw new \Exception(
                "Httpserver : invalid callable function provided to listen " .
                "(must be callable)",
                500
            );
        }
        foreach ($this->listeners as $listener) {
            if ($listener["ip"] === $ip && $listener["port"] === $port) {
                throw new \Exception("Httpserver : can not add multiple listeners on " .
                "the same couple IP/Port", 500);
            }
        }
        $this->listeners[] = ["ip" => $ip,
                          "port" => $port,
                          "callable" => $callable];
        return $this;
    }

    private function setLogger($logger)
    {
        $this->logger = $logger;
        return $this;
    }

    public function getMaxConnection()
    {
        return $this->maxConnection;
    }

    public function setMaxConnection($maxConnection)
    {
        $this->maxConnection = $maxConnection;
        return $this;
    }

    public function getTimeout()
    {
        return $this->timeout;
    }

    public function setTimeout($timeout)
    {
        $this->timeout = $timeout;
        return $this;
    }
    public function getRunningUser()
    {
        return $this->runningUser;
    }

    public function setRunningUser($user)
    {
        $this->runningUser = $user;
        return $this;
    }

    public function getRunningGroup()
    {
        return $this->runningGroup;
    }

    public function setRunningGroup($group)
    {
        $this->runningGroup = $group;
        return $this;
    }

    final public function loop()
    {
        $tcpserver = new Tcpserver();
        if (empty($this->listeners)) {
            throw new \Exception("Httpserver : No listener defined");
        }
        foreach ($this->listeners as $listener) {
            $tcpserver->init($listener["ip"], $listener["port"], [$this, "connect"]);
        }
        $tcpserver->maxChild($this->maxConnection);
        $tcpserver->loop();
    }

    final public function connect($tcpserver)
    {
        if ($this->runningUser) {
            $user = posix_getpwnam($this->runningUser);
            posix_setuid($user['uid']);
        }
        if ($this->runningGroup) {
            $group = posix_getgrnam($this->runningGroup);
            posix_setgid($group['gid']);
        }
        $httpConnection = new HttpConnection($tcpserver);
    }
}
